/* 
 * @(#)StateHandler 14.3.2002 version 0.2.0 
 * 
 * Copyright University of Helsinki / Department of computer science
 * 
 */ 

package tavi;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;

import tavi.graph.*;

/**
 * This class implements the event handling and functionality associated with
 * constructing graphs of Servers and TFlows. <p>
 *
 * Abstract: This class handles what happens when the user selects items from
 * the toolbar and constructs a graph on the draw area by clicking on the 
 * draw area. <p>
 *
 * In class Tavi: all events originating from the
 * TaviCanvas object drawArea and the five toolbar buttons are simply converted
 * to integers in their respective event-handling -methods and 
 * passed to StateHandler via the resolve- and resolveMousePressed-methods.
 * A state machine is used to determine the action taken. The integer
 * received by the StateHandler.resolve -method is interpreted as the 
 * transition of the 
 * state machine. The state machine itself is implemented as a switch-case 
 * -structure in the resolve -method. <p>
 *
 * @version     0.2.0 14 Mar 2002 
 * @author      Tuominen Heikki 
 * @author      Ukkonen Johannes
 */
public class StateHandler {

    //********************************************************
    // final fields representing the state machine's 
    // transitions and states  
    //********************************************************
    /**
     * Right mouse button event.
     */
    protected static final int RMB = 0;
    /**
     * Left mouse button clicked on drawing area.
     */
    protected static final int LMB_ON_DRAW_AREA = 1;
    /**
     * Left mouse button clicked on server in drawing area.
     */
    protected static final int LMB_ON_SERVER_IN_DRAW_AREA = 2;
    /**
     * Left mouse button clicked on server icon in toolbar.
     */
    protected static final int LMB_ON_TOOLBAR_SERVER = 3;
    /**
     * Left mouse button clicked on qserver icon in toolbar.
     */
    protected static final int LMB_ON_TOOLBAR_QSERVER = 4;
    /**
     * Left mouse button clicked on tflow icon in toolbar.
     */
    protected static final int LMB_ON_TOOLBAR_TFLOW = 5;
    /**
     * Left mouse button clicked on Begin icon in toolbar.
     */
    protected static final int LMB_ON_TOOLBAR_BEGIN = 6;
    /**
     * Left mouse button clicked on End icon in toolbar.
     */
    protected static final int LMB_ON_TOOLBAR_END = 7;
    /**
     * Left mouse button clicked on TFlow in drawing area.
     */
    protected static final int LMB_ON_TFLOW_IN_DRAW_AREA = 8;
    /**
     * Delete key pressed.
     */
    protected static final int DELETE = 9;
    /**
     * State 0 of automaton.
     */
    private static final int BEGIN = 0;
    /**
     * State 1 of automaton.
     */
    private static final int ADD_SERVER = 1;
    /**
     * State 2 of automaton.
     */
    private static final int ADD_QSERVER = 2;
    /**
     * State 3 of automaton.
     */
    private static final int ADD_TFLOW = 3;
    /**
     * State 4 of automaton.
     */
    private static final int TFLOW_SOURCE_SELECTED = 4;
    /**
     * State 5 of automaton.
     */
    private static final int ADD_BEGIN = 5;
    /**
     * State 6 of automaton.
     */
    private static final int ADD_END = 6;
    /**
     * State 7 of automaton.
     */
    private static final int ACTIVATE_SERVER = 7;
    /**
     * State 8 of automaton.
     */
    private static final int ACTIVATE_TFLOW = 8;
    /**
     * State 9 of automaton.
     */
    private static final int ROUTING_TFLOW = 9;
    /**
     * Current state of automaton.
     */
    private int state;
    //************************************************************
    // fields needed to implement functionality implied by state
    // machine
    //************************************************************
    /**
     * Start point of edge representing TFlow
     * set to null, unless adding a TFlow.
     */
    private Server tFlowStartPoint;
    /**
     * Reference to Server on which the left mouse button was pressed
     */
    private Server serverPressed;
    /**
     * Reference to TFlow on which the left mouse button was pressed
     */
    private TFlow tFlowPressed;
    /**
     * Link to client routed half-finished TFlow.
     */
    private TFlow routeFlow;
    /**
     * Reference to clicked server.
     */
    private Server activatedServer;
    /**
     * Reference to clicked TFlow.
     */
    private TFlow activatedTFlow;
    /**
     * Reference to GUI.
     */
    private Tavi gui;
    /**
     * Reference to graph.
     */
    private Graph graph;
    /**
     * Reference to current transaction's flowIdentifier.
     */
    private Identifier flowIdentifier;

    /**
     * Constructor sets everything to false/null and state to
     * zero. Graph is given valid instance.
     */
    public StateHandler(Tavi t, Graph g) {
	gui = t;
	graph = g;
	state = BEGIN;
	resetAll();
    }
	
    /**
     * Helper method executed when the draw area is cleared of objects. 
     * Necessary since the clearing of draw area is handled outside of 
     * StateHandler.
     */
    public void onClear() {
	resetAll();	
	graph.setBeginAllready(false);
	graph.setEndAllready(false);
	state = BEGIN;
    }

    /**
     * Sets the reference to the graph containing the model.
     * @param g Data structure containg current data objects.
     */
    public void setDrawObjects(Graph g) {
	graph = g;
    }

    /**
     * Sets the reference to the current transaction's flowIdentifier.
     */
    public void setFlowIdentifier(Identifier flowId) {
	flowIdentifier = flowId;
    }

    /** 
     * Remove all selections, ie. reset all fields to false/null.
     * Also zero toolbar icons. NOTE: this sets some fields to null so
     * whenever zeroAll is invoked the state should be changed accordingly,
     * usually to BEGIN.
     */
    private void resetAll() {
	if (state == ADD_SERVER) {
	    gui.setServerIcon(false);
	} else if (state == ADD_QSERVER) {
	    gui.setQueueServerIcon(false);
	} else if (state == ADD_TFLOW || state == TFLOW_SOURCE_SELECTED ||
		   state == ROUTING_TFLOW) {
	    gui.setFlowIcon(false);	  
	} else if (state == ADD_BEGIN) {
	    gui.setBeginIcon(false);
	} else if (state == ADD_END) {
	    gui.setEndIcon(false);
	}
	if (tFlowStartPoint != null) {
	    tFlowStartPoint.setSelected(false);
	}
	serverPressed = null;
	tFlowPressed = null;
	tFlowStartPoint = null;
	graph.setRouteFlow(null);
	routeFlow = null;

	if (activatedServer != null) {
	    activatedServer.setSelected(false);
	    activatedServer = null;
	}
	if (activatedTFlow != null) {
	    activatedTFlow.setSelected(false);
	    activatedTFlow = null;
	}
    }
	
    /**
     * Adds a new turn point to the client routed TFlow in construction. 
     * @param p Location of new routing point.
     */
    private void addLeg(Point p) {
	if (routeFlow == null) {
	    routeFlow = new TFlow(tFlowStartPoint, null, null, -1, 1, null);
	    int[] i = new int[1];
	    i[0] = p.x;
	    routeFlow.setXTurnPoints(i);
	    i[0] = p.y;
	    routeFlow.setYTurnPoints(i);
	    graph.setRouteFlow(routeFlow);
	} else {
	    int[] oldX = routeFlow.getXTurnPoints();
	    int[] newX = new int[oldX.length + 1];
	    int[] oldY = routeFlow.getYTurnPoints();
	    int[] newY = new int[oldY.length + 1];
	    for (int i = 0; i < oldX.length; ++i) {
		newX[i] = oldX[i];
		newY[i] = oldY[i];
	    }
	    newX[newX.length - 1] = p.x;
	    newY[newY.length - 1] = p.y;
	    routeFlow.setXTurnPoints(newX);
	    routeFlow.setYTurnPoints(newY);
	}
    }

    /**
     * Changes activated server's properties.
     * @param name New name of the server.
     * @param qLength New length of the server queue.
     * @param center New center point of the server.
     */
    public void editServerProperties(String name, int qLength, 
					Point center) {
	if (activatedServer == null) {
	    gui.displayMessage("Server not selected!", false);
	    return;
	}

	activatedServer.setServerName(name);
	activatedServer.setQueueLength(qLength);
        activatedServer.setLocation(center);
	
	if (qLength > 0) {
	    activatedServer.calculateQueuePoints();
	}
    }

    /**
     * Return the activated server.
     * @return Activated server.
     */
    public Server getActivatedServer() {
	return activatedServer;
    }

    
    /**
     * Changes activated TFlow's properties.
     * @param name New name of the TFlow.
     * @param scheme New scheme of the TFlow.
     * @param newTransaction New transaction for TFlow.
     */
    public void editTFlowProperties(String name, String scheme) {
	if (activatedTFlow == null) {
	    gui.displayMessage("TFlow not selected!", false);
	    return;
	}
	activatedTFlow.setName(name);
	activatedTFlow.setScheme(scheme);	
    }

    /**
     * Changes activated flow's direction if source and
     * destination servers aren't the same. Calls a method
     * in TFlow class to handle the operation.. 
     */
    public void changeTFlowDirection(JButton dirChangeButton) {
	if (activatedTFlow == null) {
	    gui.displayMessage("TFlow not selected!", false);
	    return;
	}

	/*swap non-recursive edge's source and destination*/
	if (!activatedTFlow.isRecursion()) {
	    activatedTFlow.changeDirection();
	    dirChangeButton.setText("Change direction: From " +
			activatedTFlow.getSource().getIdentifier() + " To "+
			activatedTFlow.getDestination().getIdentifier());
	}
	else {
	    /*recursive edge: find next main direction that has room*/
	    int i = activatedTFlow.giveArcDirection();
	    
	    while(true) {
		/*chance main direction*/			
		if (activatedTFlow.giveArcDirection() != i &&
		    activatedTFlow.getSource().arcsInThisDirection(i) 
		       < gui.getRecursiveMax()) {
		    
		    activatedTFlow.setArcDirection(i);
		    dirChangeButton.setText("Change main direction: "+
			activatedTFlow.directionName(
			    activatedTFlow.giveArcDirection()));
		    return;
		 }
	        ++i;
	        if (i > TFlow.WEST) {
		    i = TFlow.NORTH;
	    	}
	    }
	}
    }    
    
    /**
     * Sets all selected objects' selection status false. Is called when 
     * saving the graph.
     */
    public void clearAllSelections() {
	if (activatedServer != null) {
	    activatedServer.setSelected(false);
	    activatedServer = null;
	} 
	if (serverPressed != null) {
	    serverPressed.setSelected(false);
	    serverPressed = null;
	}
	if (activatedTFlow != null) {
	    activatedTFlow.setSelected(false);
	    activatedTFlow = null;
	}
	if (tFlowPressed != null) {
	    tFlowPressed.setSelected(false);
	    tFlowPressed = null;
	}
	if (tFlowStartPoint != null) {
	    resetAll();
	    gui.setFlowIcon(true);
	    state = ADD_TFLOW;
	} else {
	    resetAll();
	    state = BEGIN;
	}
    }

    /**
     * Handler method. Takes MouseEvent as a parameter and converts it to
     * an integer representing an action (a transition of the state
     * machine) i.e. if LMB was pressed on a server in the draw area then
     * calls resolve(LMB_ON_SERVER_IN_DRAW_AREA)
     * @param e The MouseEvent to resolve. 
     */
    public void resolveMousePressed(MouseEvent e) {
	
	/* Get location of press */
	Point p = e.getPoint();
	
	/* Check if right mousebutton is pressed,
	   set all toolbar selections false.
	   Works in both linux and windows systems */
	if (e.isPopupTrigger() ||  
                       (e.getModifiers() & InputEvent.BUTTON3_MASK)
	    == InputEvent.BUTTON3_MASK) {
	    resolve(RMB, p);
	} else {
	    /* mouse cursor was on server, tflow or neither */
	    serverPressed = graph.getClosestServer(p, Server.getRadius());
	    tFlowPressed = graph.getClosestTFlow(p);
	    
	    if (serverPressed == null && tFlowPressed == null) {
		resolve(LMB_ON_DRAW_AREA, p);
	    } else if (tFlowPressed == null) {
		resolve(LMB_ON_SERVER_IN_DRAW_AREA, p);
	    } else {
		resolve(LMB_ON_TFLOW_IN_DRAW_AREA, p);
	    }
	}
    }

    /**
     * Contains the actual implementation of the state machine.Uses
     * switch-case structure to implement the machine. Decides what 
     * to do based on current state and action received as a parameter. 
     * The switch is done by action and not by state, however. 
     * Executes functionality and alters state.
     * @param action Numeric expression of action to resolve (i.e 
     * if right mouse button (RMB) was  pressed then 0 would be
     * passed as a parameter.
     * @param p Location of mouse click.
     */ 
    public void resolve(int action, Point p) {
	    
	switch (action) {

	case RMB:
	    if (state == TFLOW_SOURCE_SELECTED) {
		tFlowStartPoint.setSelected(false);
		tFlowStartPoint = null;
		routeFlow = null;
		graph.setRouteFlow(null);
		state = ADD_TFLOW;
	    } else if (state == ROUTING_TFLOW) {
		routeFlow = null;
		graph.setRouteFlow(null);
		state = TFLOW_SOURCE_SELECTED;
	    } else {
		resetAll();
		state = BEGIN;
	    }		
	    break;

	case LMB_ON_DRAW_AREA:
	    boolean serverTooClose = false;
	    boolean tFlowTooClose  = false;
	    if (graph.getClosestServer(p, 1.0 * 
				       gui.getMinServerAddDistance()) != null) {
		serverTooClose = true;
	    }
	    if (!graph.locationOk(p)) {
		tFlowTooClose = true;
	    }

	    if (state == ADD_SERVER) {
		if (!serverTooClose && !tFlowTooClose) {
			Server server = new Server(p, 0,
			gui.identifier.getIdentifier());
			graph.addServer(server);
			    
		} else {
			gui.displayMessage("Server would overlap " +
				       "another object.", false);
		} 
	    } else if (state == ADD_QSERVER) {
		if (!serverTooClose && !tFlowTooClose) {
		    Server server = new Server(p, 1,
					       gui.identifier.getIdentifier());
		    graph.addServer(server);
		} else {
		    gui.displayMessage("Server would overlap " +
				       "another object.", false);
		}
	    } else if (state == TFLOW_SOURCE_SELECTED) {
		/* Add new turn point to polyline representing TFlow. */
		addLeg(p);
		state = ROUTING_TFLOW;
	    } else if (state == ROUTING_TFLOW) {
		/* Add new turn point to polyline representing TFlow. */
		addLeg(p);
	    } else if (state == ADD_BEGIN) {
		if (!serverTooClose && !tFlowTooClose && 
		    !graph.isBeginAllready()) {
		    Server server = new Server(p, 'B');
		    graph.addServer(server);
		    graph.setBeginAllready(true);
		    state = BEGIN;
		    gui.setBeginIcon(false);
		} else {
		    gui.displayMessage("Unable to add begin state.", false);
		}
	    } else if (state == ADD_END) {
		if (!serverTooClose && !tFlowTooClose &&
		    !graph.isEndAllready()) {
		    Server server = new Server(p, 'E');
		    graph.addServer(server);
		    graph.setEndAllready(true);
		    state = BEGIN;
		    gui.setEndIcon(false);
		} else {
		    gui.displayMessage("Unable to add end state.", false);
		}
	    }
	    break;

	case LMB_ON_SERVER_IN_DRAW_AREA:
	    /* select TFlow source */
	    if (state == ADD_TFLOW) {
		tFlowStartPoint = serverPressed;
		if (!tFlowStartPoint.isEnd()) {
		    tFlowStartPoint.setSelected(true);
		    state = TFLOW_SOURCE_SELECTED;
		} else if (tFlowStartPoint.isEnd()) {
		    gui.displayMessage("Cannot start flow from End state.", 
				       false);
		}                                                               
		/* select TFlow destination */
	    } else if (state == TFLOW_SOURCE_SELECTED) {
		if (serverPressed.isBegin()) {
		    gui.displayMessage("Flows cannot have Begin as"+
				       " destination", false);
		    /* if recursion edge requested */
		    // **********************************************
		} else if (tFlowStartPoint.equals(serverPressed)) {
		    int dir = TFlow.isInMainDirection(serverPressed, p);
		    if (dir < 0) {
			break;
		    }
		    TFlow tryLine = new TFlow(tFlowStartPoint, 
					       serverPressed, p, -1, 1, null);
		    tryLine.drawSelf(null);
		    if(serverPressed.arcsInThisDirection(dir)
		       < gui.getRecursiveMax() &&
		       graph.lineOk(tryLine)) { 
			graph.addTFlow(tFlowStartPoint, serverPressed, p,
				       flowIdentifier.getNumber(), 1);
		    } else if (graph.lineOk(tryLine) ||
			       serverPressed.arcsInThisDirection(dir) >=
			       gui.getRecursiveMax()) {
			gui.displayMessage(TFlow.directionName(dir) + 
					   " is full", false);
		    } else if (!graph.lineOk(tryLine)) {
			gui.displayMessage("Flow would overlap" +
					   " a server.", false);
		    }
		    
		    /* flow between two servers when there are less than */
		    /* MAX_PARALLEL_TFLOWS_BETWEEN_TWO_SERVERS TFlows */
		    /* between those servers. */
		    // **********************************************
		} else if ((tFlowStartPoint.edgesTo(serverPressed) + 
			    serverPressed.edgesTo(tFlowStartPoint)) < 
			   Server.MAX_PARALLEL_TFLOWS_BETWEEN_TWO_SERVERS) {
		    TFlow tryLine = new TFlow(tFlowStartPoint, 
					       serverPressed, p, -1, 1, null); 
		    if ((graph.lineOk(tryLine) && 
			 (TFlow.distanceBetween(tFlowStartPoint, 
						serverPressed) <=
			  6000) || 
			 (TFlow.distanceBetween(tFlowStartPoint, 
						serverPressed) >
			  6000 &&
			  tFlowStartPoint.edgesTo(serverPressed) + 
			  serverPressed.edgesTo(tFlowStartPoint) == 0))) {
			tryLine = null;
			TFlow t = graph.addTFlow(tFlowStartPoint, 
						 serverPressed,
				 p, flowIdentifier.getNumber(),
				 tFlowStartPoint.edgesTo(serverPressed) 
				 + serverPressed.edgesTo(tFlowStartPoint) + 1);
		    } else {
			tryLine = null;
			gui.displayMessage("Please route by hand", false);
		    }
		    
		}
		else { 		    
		    gui.displayMessage("Please route by hand", false);
		}
	    } else if (state == ROUTING_TFLOW) {
		if (serverPressed.isBegin()) {
		    gui.displayMessage("Flows cannot have Begin as"+
				       " destination", false);
		} else {
		    TFlow t = graph.addTFlow(tFlowStartPoint, 
					     serverPressed, null,
					     flowIdentifier.getNumber(), 1);
		    t.setXTurnPoints(routeFlow.getXTurnPoints());
		    t.setYTurnPoints(routeFlow.getYTurnPoints());
		    routeFlow.setDestination(serverPressed);
		    routeFlow = null;
		    graph.setRouteFlow(null);
		    state = TFLOW_SOURCE_SELECTED;
		}
	    } else if (state == ADD_SERVER || state == ADD_QSERVER) {
		gui.displayMessage("Server would overlap " +
			       "another object.", false);
	    } else if (state == BEGIN) {
		activatedServer = serverPressed;
		activatedServer.setSelected(true);
		state = ACTIVATE_SERVER;
	    } else if (state == ACTIVATE_SERVER) {
		if (activatedServer == serverPressed) {
		    /* Editing Begin or End states is not
		       allowed. */
		    if (activatedServer.isBegin() || 
			activatedServer.isEnd()) {
			break;
		    }		
		    gui.doStatusWindow(activatedServer);
		} else {
		    activatedServer.setSelected(false);
		    activatedServer = serverPressed;
		    activatedServer.setSelected(true);
		}
	    } else if (state == ACTIVATE_TFLOW) {
		activatedTFlow.setSelected(false);
		activatedTFlow = null;
		activatedServer = serverPressed;
		activatedServer.setSelected(true);
		state = ACTIVATE_SERVER;
	    }
	    break;

	case LMB_ON_TOOLBAR_SERVER:
	    resetAll();
	    if (state != ADD_SERVER) {
		gui.setServerIcon(true);	
		state = ADD_SERVER;
	    } else {
		state = BEGIN;
	    }		
	    break;

	case LMB_ON_TOOLBAR_QSERVER:
	    resetAll();
	    if (state != ADD_QSERVER) {
		gui.setQueueServerIcon(true);
		state = ADD_QSERVER;
	    } else {
		state = BEGIN;
	    }
	    break;
		
	case LMB_ON_TOOLBAR_TFLOW:
	    resetAll();
	    if (state != ADD_TFLOW) {
		gui.setFlowIcon(true);
		state = ADD_TFLOW;
	    } else {
		state = BEGIN;
	    }
	    break;

	case LMB_ON_TOOLBAR_BEGIN:
	    if (state != ADD_BEGIN) {
		if (graph.isBeginAllready()) {
		    gui.displayMessage("Begin already added.", false);
		} else {
		    resetAll();
		    gui.setBeginIcon(true);
		    state = ADD_BEGIN;
		}
	    } else {
		resetAll();
		state = BEGIN;
	    }
	    break;

	case LMB_ON_TOOLBAR_END:
	    if (state != ADD_END) {
		if (graph.isEndAllready()) {
		    gui.displayMessage("End already added.", false);
		} else {
		    resetAll();
		    gui.setEndIcon(true);
		    state = ADD_END;
		}
	    } else {
		resetAll();
		state = BEGIN;
	    }
	    break;

	case LMB_ON_TFLOW_IN_DRAW_AREA:
	    if (state == BEGIN || state == ACTIVATE_SERVER) {
		if (activatedServer != null) {
		    activatedServer.setSelected(false);
		    activatedServer = null;
		}
		activatedTFlow = tFlowPressed;
		activatedTFlow.setSelected(true);
		state = ACTIVATE_TFLOW;
	    } else if (state == ACTIVATE_TFLOW) {
		if (activatedTFlow == tFlowPressed) {
		    gui.doStatusWindow(activatedTFlow);
		} else {
		    activatedTFlow.setSelected(false);
		    activatedTFlow = tFlowPressed;
		    activatedTFlow.setSelected(true);
		}
	    }
	    break;

	case DELETE:
	    if (state == ACTIVATE_SERVER) {
		if (activatedServer != null) {
		    if (activatedServer.isBegin()) {
			graph.setBeginAllready(false);
		    } else if (activatedServer.isEnd()) {
			graph.setEndAllready(false);
		    }
		    graph.removeServer(activatedServer, gui.identifier);
		}
	    } else if (state == ACTIVATE_TFLOW) {
		graph.removeTFlow(activatedTFlow, gui.identifier);
	    } else if (state == ROUTING_TFLOW) {
		routeFlow = null;
		graph.setRouteFlow(null);
		state = TFLOW_SOURCE_SELECTED;
	    }
	    break;

	    // for debugging purposes ...
	default:
	    System.out.println("Unrecognized action number " + action +
			       " received in StateHandler.resolve()");
	    break;
	}

	/* set all objects to fit the drawing area*/
	int [] graphMaxXMaxY = gui.graph.scaleServers((double)1);

	/*resize canvas according to contents*/
	if (graphMaxXMaxY[2] < gui.getWidth()) {
	    graphMaxXMaxY[2] = gui.getWidth();
	}
	if (graphMaxXMaxY[3] < gui.getHeight()) {
	    graphMaxXMaxY[3] = gui.getHeight();
	}
	gui.drawArea.windowResized(new Dimension(graphMaxXMaxY[2], graphMaxXMaxY[3]));	

	/* after resolving state/action show changes */
	gui.drawArea.updateCanvas();
    }
}
