/* 
 * @(#)Tavi 11.5.2002 version 1.0.0 
 * 
 * Copyright University of Helsinki / Department of computer science
 * 
 */ 

package tavi;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;

import java.util.*;
import java.util.zip.*;
import java.io.*;

import java.net.*;

import tavi.graph.*;

/** 
 * Main class of TAVI. <p>
 *
 * Contains the implementation of the main window and the components in 
 * it, excluding the {@link #drawArea drawArea}, which is implemented in 
 * {@link tavi.TaviCanvas
 * TaviCanvas} class. The main window consists of a standard menubar (topmost), 
 * toolbar (left), drawArea (center) and a text field indicating the current
 * transaction (left, under the toolbar). <p>
 *
 * <B>Toolbar:</B> the toolbar is a JToolbar, a collection of JButtons with GIFs 
 * representing the objects that are used to draw queue-models (graphs) of
 * event-based systems. <p> 
 * 
 * <B>DrawArea:</B> the drawArea is a TaviCanvas object, a general drawing area 
 * on which a picture of the queue-model is displayed as a graph of 
 * Server-objects 
 * (vertices, service points, represented as circles) and TFlow-objects 
 * (edges, transaction flows, repsented as straight lines, curved lines or
 * polylines)
 *
 * <B>Event handling:</B> all the events originating from the menubar are 
 * handled here. Events originating from the toolbar and drawArea are passed
 * to {@link tavi.StateHandler StateHandler} via the resolve and 
 * resoveMousePressed methods. Some of the menubar items launch dialog 
 * windows requiring user input. These windows are implemented in {@link
 * tavi.TaviDialogs TaviDialogs} class.
 *
 * <B>Load/save</B> -functions are also implemented in this class.
 * 
 * @version     1.0.0 11 May 2002 
 * @author      Tuominen Heikki 
 * @author	Kupiainen Esko
 * @author      Ukkonen Johannes
 * @author      Airamaa Kimmo
 * @author      Petj Janne
 */ 
public class Tavi extends JFrame implements ActionListener{

    /**
     * Window height for small window.
     */
    public static final int HEIGHT_SMALL = 300;
    /**
     * Window width for small window.
     */
    public static final int WIDTH_SMALL = 300;
    /**
     * Window height for medium window.
     */
    public static final int HEIGHT_REG = 480;
    /**
     * Window witdh for medium window.
     */
    public static final int WIDTH_REG = 640;
    /**
     * Window height for large window.
     */
    public static final int HEIGHT_LARGE = 600;
    /**
     * Window height for large window.
     */
    public static final int WIDTH_LARGE = 800;
    /**
     * Window width for huge window.
     */
    public static final int HEIGHT_HUGE = 768;
    /**
     * Window height for huge window.
     */
    public static final int WIDTH_HUGE = 1024;
    /**
     * Customer selected maximum for recursive edges in one direction.
     */
    public int tempRecursiveMax = 
	Server.MAX_RECURSIVE_TFLOWS_PER_DIRECTION / 2;

    /**
     * New server must be at least this far from  all existing servers.
     * Used when checking that drawable objects don't overlap each other.
     */
    protected static  int minServerAddDistance = 
					7 * Server.getRadius() / 2;
    /**
     * Interval length between message window opening and closing.
     */
    protected int timeOut = 2000;

    /**
     * Graph that contains all servers in the current model. Flows can
     * be accessed via servers.
     */
    protected Graph graph;
    /**
     * Identifier gives object a unique identification. This identifier
     * assigns identifier values to transactions and servers.
     */
    protected Identifier identifier;
    /**
     * TaviCanvas used to draw objects.
     */
    protected TaviCanvas drawArea;
    /**
     * Implementation of a StateHandler, a finite automaton that performs
     * the event handling of drawing area objects.
     */
    protected StateHandler stateHandler;
    /**
     * Transactionmanager is a class that contains all the transactions.
     * Transaction handling is made using transaction manager. 
     */
    protected TransactionManager manager;
    /**
     * Flag to indicate whether edges should show their selection area.
     */
    protected static boolean selectionArea = true;
 
    /**
     * The directory of the gifs representing the toolbar buttons
     * relative to the location of class Tavi.
     */
    private static final String GIF_DIR = "gifs" + File.separator;
    /**
     * Path to gif representing the select TFlow toolbar button.
     */
    private static final String TFLOW_YES = GIF_DIR + "vector.gif";
    /**
     * Path to gif representing the deselect TFlow toolbar button.
     */
    private static final String TFLOW_NO = GIF_DIR + "vector_p.gif";    
    /**
     * Path to gif representing the select Server toolbar button.
     */
    private static final String SERVER_YES = GIF_DIR + "server.gif";
    /**
     * Path to gif representing the deselect Server toolbar button.
     */
    private static final String SERVER_NO = GIF_DIR + "server_p.gif";
    /**
     * Path to gif representing the select QServer toolbar button.
     */
    private static final String QSERVER_YES = GIF_DIR + "qserver.gif";
    /**
     * Path to gif representing the deselect QServer toolbar button.
     */
    private static final String QSERVER_NO = GIF_DIR + "qserver_p.gif";
    /**
     * Path to gif representing the select Begin toolbar button.
     */
    private static final String BEGIN_YES = GIF_DIR + "begin.gif";
    /**
     * Path to gif representing the deselect Begin toolbar button.
     */
    private static final String BEGIN_NO = GIF_DIR + "begin_p.gif";
    /**
     * Path to gif representing the select End toolbar button.
     */
    private static final String END_YES = GIF_DIR + "end.gif";
    /**
     * Path to gif representing the deselect End toolbar button.
     */
    private static final String END_NO = GIF_DIR + "end_p.gif";
    /**
     * Window menubar. All menus are located in this menubar.
     */
    private JMenuBar menuBar; 
    /**
     * Menubar file menu.
     */
    private JMenu fileMenu;
    /**
     * Menu for editor settings.
     */
    private JMenu settingsMenu;
    /**
     * Submenu for background color.
     */
    private JMenu colorSubmenu;
    /**
     * Submenu for changing drawArea size.
     */
    private JMenu sizeSubmenu;
    /**
     * Menubar for transaction control.
     */
    private JMenu transactionMenu;
    /**
     * Submenu for editing current transaction's properties.
     */
    private JMenu transactionSubmenu;
    /**
     * Submenu for global variables.
     */
    private JMenu globalVariableSubmenu;
    /**
     * Filemenu item for exiting program.
     */
    private JMenuItem exitItem;    
    /**
     * Filemenu item for clearing drawing area.
     */
    private JMenuItem clearItem; 
   /**
     * Filemenu item for loading model/graph.
     */
    private JMenuItem loadItem;    
    /**
     * Filemenu item for saving model/graph.
     */
    private JMenuItem saveItem; 
    /**
     * Menu selection to change drawing area color to yellow.
     */
    private JMenuItem yellowColor;
    /**
     * Menu selection to change drawing area color to white.
     */
    private JMenuItem whiteColor;
    /**
     * Menu selection to change drawing area color to pink.
     */
    private JMenuItem pinkColor;
    /**
     * Menu selection to change drawing area color to green.
     */
    private JMenuItem greenColor;
    /**
     * Menu selection to change drawing area color to blue.
     */
    private JMenuItem blueColor;
    /**
     * Menu selection used to change drawing area size.
     */
    private JMenuItem smallSize;
    /**
     * Menu selection used to change drawing area size.
     */
    private JMenuItem regSize;
    /**
     * Menu selection used to change drawing area size.
     */
    private JMenuItem largeSize;
    /**
     * Menu selection used to change drawing area size.
     */
    private JMenuItem hugeSize;
    /**
     * Menu selection used to change drawing area size.
     */
    private JMenuItem customSize;
    /**
     * Menu selection used to change system settings.
     */
    private JMenuItem systemSettings;
    /**
     * Menu selection to add a new transaction.
     */
    private JMenuItem newTransaction;
    /**
     * Menu selection to view all transactions by name.
     * It is possible to edit or delete transactions after
     * selecting this item.
     */
    private JMenuItem viewTransaction;
    /**
     * Menu selection to change a currently active transaction.
     */
    private JMenuItem changeTransaction;    
    /**
     * Menu selection to add new local variables to currently active
     * transaction.
     */
    private JMenuItem addVariable;
    /**
     * Menu selection to view active transaction's variables.
     */
    private JMenuItem viewVariables;
    /**
     * Menu selection to add new global variable.
     */
    private JMenuItem addGlobalVariable;
    /**
     * Menu selection to view global variables.
     */
    private JMenuItem viewGlobalVariables;

    /**
     * Editor toolbar. Buttons used to select object type to be added
     * are located here.
     */
    private JToolBar toolList;
    /**    
     * Button used to add server.
     */
    private JButton addServer;
    /**
     * Adds server with queue.
     */
    private JButton addQServer;
    /**
     * Button used to add vectors between servers.
     */
    private JButton addTFlow;
    /**
     * Add start state.
     */
    private JButton addBegin;
    /**
     * Add end state.
     */
    private JButton addEnd;
    /** 
     * Textfield used to display currently active transaction.
     */
    private JLabel activeTransactionLabel;

    /** 
     * TFlow is such a heavy class that it necessary to initialize
     * it separately. Otherwise, when adding the first TFlow there 
     * would be a noticeable delay. 
     */   
    private TFlow tFlowInitializer;
    
    /**
     * Thread to count time. So far used to close dialogs after a certain
     * timeout. 
     */
    protected Timer timer;

    /**
     * Reference to class that creates dialog windows when needed.
     */
    private TaviDialogs dialogManager;
   
    /** 
     * Scrollpane for TaviCanvas. With this the drawing area is scrollable.
     */
    private JScrollPane scroller;

    /**
     * Initializes the Tavi window components and assigns
     * listeners to them. Initializes TFlow class, because it is so 
     * complex that it would take noticeable delay when drawing the first 
     * line.
     * @param title Window title.
     */
    public Tavi(String title) {
	/* Put parameter as title */
	super(title);

	/* Initialize TFlow class. */
	Server a = new Server(new Point(100,100), 0, "INIT");
	Server b = new Server(new Point(200,200), 0, "INIT");
	tFlowInitializer = new TFlow(a, b, 1, 1, null);
	tFlowInitializer.drawSelf(null); 
	
	identifier = new Identifier();
	manager = new TransactionManager();

	/* Add menubar to window */
	menuBar = new JMenuBar();
	fileMenu = new JMenu("File");
	clearItem = new JMenuItem("Clear");
	loadItem = new JMenuItem("Load");
	saveItem = new JMenuItem("Save");
	exitItem = new JMenuItem("Exit");
	fileMenu.add(clearItem);
	fileMenu.addSeparator();
	fileMenu.add(loadItem);
	fileMenu.add(saveItem);
	fileMenu.addSeparator();
	fileMenu.add(exitItem);

	settingsMenu = new JMenu("Settings");
	colorSubmenu = new JMenu("Background color");
	settingsMenu.add(colorSubmenu);
	yellowColor = new JMenuItem("yellow");
	whiteColor = new JMenuItem("white");
	pinkColor = new JMenuItem("pink");
	greenColor = new JMenuItem("green");
	blueColor = new JMenuItem("blue");
	colorSubmenu.add(yellowColor);
	colorSubmenu.add(whiteColor);
	colorSubmenu.add(pinkColor);
	colorSubmenu.add(greenColor);
	colorSubmenu.add(blueColor);
	
	sizeSubmenu = new JMenu("Size");
	smallSize = new JMenuItem("300 x 300");
	regSize = new JMenuItem("640 x 480");
	largeSize = new JMenuItem("800 x 600");
	hugeSize = new JMenuItem("1024 x 768");
	customSize = new JMenuItem("modify");
	settingsMenu.add(sizeSubmenu);
	sizeSubmenu.add(smallSize);
	sizeSubmenu.add(regSize);
	sizeSubmenu.add(largeSize);
	sizeSubmenu.add(hugeSize);
	sizeSubmenu.addSeparator();
	sizeSubmenu.add(customSize);

	systemSettings = new JMenuItem("Preferences");
	settingsMenu.add(systemSettings);

	transactionMenu = new JMenu("Transactions");
	transactionSubmenu = new JMenu("Current");
	globalVariableSubmenu = new JMenu("Global");
	newTransaction = new JMenuItem("New");
	viewTransaction = new JMenuItem("Modify");
	addGlobalVariable = new JMenuItem("add global variable");
	viewGlobalVariables = new JMenuItem("view global variables");
	changeTransaction = new JMenuItem("change");
	addVariable = new JMenuItem("add variable");
	viewVariables = new JMenuItem("view variables");
	transactionMenu.add(newTransaction);
	transactionMenu.add(viewTransaction);
	transactionMenu.add(globalVariableSubmenu);
	transactionMenu.addSeparator();
	transactionMenu.add(transactionSubmenu);
	transactionSubmenu.add(changeTransaction);
	transactionSubmenu.addSeparator();
	transactionSubmenu.add(addVariable);
	transactionSubmenu.add(viewVariables);
	globalVariableSubmenu.add(addGlobalVariable);
	globalVariableSubmenu.add(viewGlobalVariables);

	menuBar.add(fileMenu);
	menuBar.add(settingsMenu);
	menuBar.add(transactionMenu);

	getContentPane().add("North", menuBar);

	/* Create a toolbar with icons. */
	toolList = new JToolBar(JToolBar.VERTICAL);
	addTFlow = new JButton(new ImageIcon(TFLOW_YES));
	addServer = new JButton(new ImageIcon(SERVER_YES));
	addQServer = new JButton(new ImageIcon(QSERVER_YES));
	addBegin = new JButton(new ImageIcon(BEGIN_YES));
	addEnd = new JButton(new ImageIcon(END_YES));

	/* Create tooltips for toolbar buttons. */
	addTFlow.setToolTipText("Add new transaction flows between servers.");
	addServer.setToolTipText("Add new queueless servers.");
	addQServer.setToolTipText("Add new servers with queue.");
	addBegin.setToolTipText("Add begin state.");
	addEnd.setToolTipText("Add end state.");
	toolList.add(addTFlow);
	toolList.add(addQServer);
	toolList.add(addServer);
	toolList.add(addBegin);
	toolList.add(addEnd);

	/* Add labels indicating which transaction is currently active. */
	JLabel currentLabel = new JLabel("Current:");
	currentLabel.setFont(new Font(null, Font.PLAIN, 10));
	activeTransactionLabel = new JLabel("Tr0");
	activeTransactionLabel.setFont(new Font("Courier", Font.BOLD, 10));
	toolList.add(currentLabel);
	toolList.add(activeTransactionLabel);

      	/* Add non-floatable toolbar */
	toolList.setFloatable(false);
	getContentPane().add("West", toolList);

	/* create Graph for holding the objects to be drawn */
	graph = new Graph();

	/* initialize draw area */
	drawArea = new TaviCanvas(graph);

	/* insert scrollpane */
	scroller = new JScrollPane(drawArea);
	scroller.setPreferredSize(new Dimension(640, 480));

	/* insert scrollpane to JFrame */
	getContentPane().add("Center", scroller);

	/* Add listeners to menubar components */
	clearItem.addActionListener(this);
	loadItem.addActionListener(this);
	saveItem.addActionListener(this);
	exitItem.addActionListener(this);
	
	/* add listeners to color submenu components */
	yellowColor.addActionListener(this);
	whiteColor.addActionListener(this);
	pinkColor.addActionListener(this);
	blueColor.addActionListener(this);
	greenColor.addActionListener(this);

	/* add listeners to size submenu components */
	smallSize.addActionListener(this);
	regSize.addActionListener(this);
	largeSize.addActionListener(this);
	hugeSize.addActionListener(this);
	customSize.addActionListener(this);

	/*add listeners to system settings component*/
	systemSettings.addActionListener(this);

	/* add listeners to transaction menu components */
	newTransaction.addActionListener(this);
	viewTransaction.addActionListener(this);
	changeTransaction.addActionListener(this);
	addVariable.addActionListener(this);
	viewVariables.addActionListener(this);
	addGlobalVariable.addActionListener(this);
	viewGlobalVariables.addActionListener(this);

	drawArea.addMouseListener(new MouseAdapter(){
		public void mousePressed(MouseEvent e){
		    stateHandler.resolveMousePressed(e);
		}
	    }
				  );
	
	/* Add a keylistener. */
	addKeyListener(new KeyAdapter() {
		/* Called, when some key has been pressed. */
		public void keyPressed(KeyEvent key) {
		    char typed = key.getKeyChar();
		    if (typed == KeyEvent.VK_DELETE) {
			stateHandler.resolve(StateHandler.DELETE, 
					     new Point(0, 0));
		    }
		}
	    }
		       );

	/* Add listeners to toolbar components */
	addTFlow.addActionListener(this);
	addServer.addActionListener(this);
	addQServer.addActionListener(this);
	addBegin.addActionListener(this);
	addEnd.addActionListener(this);

	/* Add listener to close program when window is closed */	
	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    System.exit(0);
		}
	    });

	/* Add component listener to listen for window resizing. */
	addComponentListener(new ComponentAdapter() {
		public void componentResized(ComponentEvent e) {
		    drawArea.windowResized(getSize());
		    
		}
	    });

	/* Create state handler */
	stateHandler = new StateHandler(this, graph);
	stateHandler.setFlowIdentifier(
		     manager.getCurrent().getFlowIdentifier());
	/* Set window location and size */
	setSize(640, 480);
	setLocation(50, 50);
	/* Pack the ui and make it visible */
       	
	pack();
	validate();
	setResizable(true);
 	setVisible(true);

	/*create timer instance*/
	timer = new Timer("Timer");

	/* Set timer priority. */
	timer.setPriority(Thread.NORM_PRIORITY);

	/* Start the timer thread. */
	timer.start();	

	/* Initializes the TaviDialogs object. */
	dialogManager = new TaviDialogs(this, manager.getTransactions());
	/* Update active transaction. */
	graph.changeTransaction(manager.getCurrent());
    }

    /**
     * Handles events other than those originating from drawArea.
     * @param e Event received.
     */
    public void actionPerformed(ActionEvent e) throws IllegalStateException {

	Object o = e.getSource();
	/* if o is JMenuItem get its parent (which is the JPopupMenu 
	   associated with the menu it belongs to.) */
	JPopupMenu oParent = null;
	if (o instanceof JMenuItem) {
	    oParent = (JPopupMenu) ((JMenuItem) o).getParent();
	}

	if (o == exitItem) {
	    System.exit(0);
	} else if (o == clearItem) {
	    identifier = new Identifier();
	    manager = new TransactionManager();
	    dialogManager = new TaviDialogs(this, manager.getTransactions());
	    graph.removeAll();
	    stateHandler.onClear();
	    stateHandler.setFlowIdentifier(
		manager.getCurrent().getFlowIdentifier());
 	    dialogManager.setTransactions(manager.getTransactions());
	    graph.changeTransaction(manager.getCurrent());
	    setActiveLabel("Tr0");
	} else if (o == loadItem) {
	    loadModel();
	} else if (o == saveItem) {
	    saveModel();
	} else if (o == addServer) {                   
	   stateHandler.resolve(StateHandler.LMB_ON_TOOLBAR_SERVER, 
				new Point(100, 100));
	} else if (o == addQServer) {
	   stateHandler.resolve(StateHandler.LMB_ON_TOOLBAR_QSERVER, 
				new Point(100, 100));
	} else if (o == addTFlow) {
	   stateHandler.resolve(StateHandler.LMB_ON_TOOLBAR_TFLOW, 
				new Point(100, 100));
	} else if (o == addBegin) {
	    stateHandler.resolve(StateHandler.LMB_ON_TOOLBAR_BEGIN, 
				 new Point(100, 100));
	} else if (o == addEnd) {
	    stateHandler.resolve(StateHandler.LMB_ON_TOOLBAR_END, 
				 new Point(100, 100));  
	} else if (o == yellowColor) {
	    drawArea.changeColor(new Color(255, 250, 155));
	} else if (o == whiteColor) {
	    drawArea.changeColor(Color.white);
	} else if (o == pinkColor) {
	    drawArea.changeColor(Color.pink);
	} else if (o == greenColor) {
	    drawArea.changeColor(new Color(165, 255, 155));
	} else if (o == blueColor) {
	    drawArea.changeColor(new Color(190, 190, 255));
	    //*************************************************************
	    //* handling of size submenu components
	    //*************************************************************
	} else if (oParent == sizeSubmenu.getPopupMenu()) {
	    if (o == customSize) {
		dialogManager.windowSizeDialog();
		return;
	    }
	    /* get current display resolution */
	    Toolkit tk = this.getToolkit();
	    Dimension displayResolution = tk.getScreenSize();
	    int displayHt = (int) displayResolution.getHeight();
	    int displayWt = (int) displayResolution.getWidth();
	    int requestHt = 0;
	    int requestWt = 0;
	    if (o == smallSize) { 
		requestHt = HEIGHT_SMALL;
		requestWt = WIDTH_SMALL;
	    } else if (o == regSize) {
		requestHt = HEIGHT_REG;
		requestWt = WIDTH_REG;
	    } else if (o == largeSize) {
		requestHt = HEIGHT_LARGE;
		requestWt = WIDTH_LARGE;
	    } else if (o == hugeSize) {
		requestHt = HEIGHT_HUGE;
		requestWt = WIDTH_HUGE;
	    } 
	    /* upper limit to window size is display resolution */
	    int windowHt = Math.min(requestHt, displayHt);
	    int windowWt = Math.min(requestWt, displayWt);
	    
	    if (!graph.isSizeChangeRecommended(requestWt, 
						   requestHt)) {
		dialogManager.messageDialog("All objects wouldn't fit in.", 
					    false);
		return;
	    }
	    setSize(windowWt, windowHt);
	    drawArea.changeSize(requestWt, requestHt);
	} else if (o == newTransaction) {
	    /* Because of few null pointer exceptions, it's probably safer to
	       get identifier first and then give string to method. */
	    String idToBeGiven = identifier.getTransactionId();
	    dialogManager.transactionNameDialog(idToBeGiven);
	} else if (o == changeTransaction) {
	    dialogManager.changeActiveTransaction();  
	} else if (o == viewTransaction) {
	    dialogManager.viewTransactions();
	} else if (o == systemSettings) {
	   dialogManager.changePreferences();
	} else if (o == addVariable) {
	    dialogManager.newVariableDialog();
	} else if (o == viewVariables) {
	    dialogManager.viewVariablesDialog();
	} else if (o == addGlobalVariable) {
	    dialogManager.newGlobalVariableDialog();
	} else if (o == viewGlobalVariables) {
	    dialogManager.viewGlobalVariablesDialog();
	}

	drawArea.updateCanvas();
    } 

    /**
     * Method that creates a modal dialog window with text and ok button.
     * Used to displays messages to user, for example when trying to add a 
     * server on another server a window indicating that this is not
     * allowed would be displayed. It either vanishes in a while or 
     * requires an acknowledgement.
     * @param text Text that will be displayed on window.
     * @param acknowledge True if messages need acknowledgement. False 
     * otherwise (window vanishes after determended time).
     */
    public void displayMessage(String text, boolean acknowledge) {
	dialogManager.messageDialog(text, acknowledge);
    }

    /**
     * Creates a window, where object's properties are displayed and
     * can be changed by typing on the fields. Buttons either commit
     * or abort changes. 
     * @param server Server which status window is to be displayed.
     */
    public void doStatusWindow(Server server) {
	dialogManager.statusDialog(server);
    }

    /**
     * Creates a window displaying the details of a transaction
     * flow. Some properties can be changed by editing the 
     * textarea they are displayed on. Flow's direction cannot be
     * changed so that it starts from End state or it ends in 
     * Begin state. The direction of recursive flows cannot be
     * changed.
     * @param flow TFlow to be displayed in detail.
     */
    public void doStatusWindow(TFlow flow) {
	dialogManager.statusDialog(flow);
    }

    /**
     * Returns current minimum server add distance. This distance defines
     * how close servers can be to each other.
     * @return Current minimum server add distance
     */
    public static int getMinServerAddDistance() {
	return 7 * Server.getRadius() / 2;;
    } 

    /**
     * Returns the user selection for maximum recursive edges. 
     * @return Current maximum allowed recursive edges per direction.
     */
    public int getRecursiveMax() {
	return tempRecursiveMax;
    }

    /**
     * Returns the flag that indicates whether edges' selection area
     * is to be made visible. 
     * @return True if edge selection area is made visible, false otherwise.
     */
    public static boolean getShowSelectionArea() {
	return selectionArea;
    }

    /**
     * Returns current message timeout. 
     * @return Current message timeout.
     */
    public int getTimeOut() {
	return timeOut;
    }  	

    /**
     * Main method for running the whole program. Creates UI and all
     * its components by calling the constructor.
     * @param args Command line arguments. Not used.
     */
    public static void main(String[] args) {
	Tavi frame = new Tavi("TAVI - Version 1.0");
    }

    /**
     * Sets begin server's toolbar icon to show whether it's activated 
     * or not. Activated icons' colors are reversed.
     * @param selection Indicates whether begin server is selected or not. 
     */
    public void setBeginIcon(boolean selection) {
	if (selection) {
	    addBegin.setIcon(new ImageIcon(BEGIN_NO));
	} else {
	    addBegin.setIcon(new ImageIcon(BEGIN_YES));
	}
    }

    /**
     * Sets end server's toolbar icon to show whether it's activated 
     * or not.Activated icons' colors are reversed.
     * @param selection Indicates whether end server is selected or not. 
     */
    public void setEndIcon(boolean selection) {
	if (selection) {
	    addEnd.setIcon(new ImageIcon(END_NO));
	} else {
	    addEnd.setIcon(new ImageIcon(END_YES));
	}
    }

    /**
     * Sets flow's toolbar icon to show whether it's activated or not.
     * Activated icons' colors are reversed.
     * @param selection Indicates whether flow is selected or not. 
     */
    public void setFlowIcon(boolean selection) {
	if (selection) {
	    addTFlow.setIcon(new ImageIcon(TFLOW_NO));
	} else {
	    addTFlow.setIcon(new ImageIcon(TFLOW_YES));
	}
    }

    /**
     * Sets server with queue's toolbar icon to show whether it's activated 
     * or not. Activated icons' colors are reversed.
     * @param selection Indicates whether server with a queue is selected 
     * or not. 
     */
    public void setQueueServerIcon(boolean selection) {
	if (selection) {
	    addQServer.setIcon(new ImageIcon(QSERVER_NO));
	} else {
	    addQServer.setIcon(new ImageIcon(QSERVER_YES));
	}
    }

    /** 
     * Changes the text in label that indicates current transaction
     * in toolbar. Transaction's identifier is displayed. Used for user's
     * convenience.
     * @param text Text to be displayed in toolbar.
     */
    public void setActiveLabel(String text) {
	if (text == null || text.equals("")) {
	    return;
	}
	activeTransactionLabel.setText(text);
    }

    /**
     * Sets the user selection for max recursive edges.
     * @param recmax Current maximum allowed recursive edges per direction.
     */
    public void setRecursiveMax(int recmax) {
	tempRecursiveMax = recmax;
    }

    /**
     * Sets server's toolbar icon to show whether it's activated or not.
     * @param selection Indicates whether server is selected or not. 
     */
    public void setServerIcon(boolean selection) {
	if (selection) {
	    addServer.setIcon(new ImageIcon(SERVER_NO));
	} else {
	    addServer.setIcon(new ImageIcon(SERVER_YES));
	}
    }

    /**
     * Sets current message timeout. It defines, how long message dialogs
     * are shown before they are closed.
     * @param timeout Current message timeout.
     */
    public void setTimeOut(int timeout) {
	timeOut = timeout;
    }   

    /**
     * Loads a model/graph from file. Current model is overwritten.
     */
    private void loadModel() {
	FileDialog f = new FileDialog(this, "Load model", FileDialog.LOAD);
	f.show();
	String filename = f.getFile();
	if (filename != null) {
	    try {
		FileInputStream fis = new FileInputStream(filename);
 		GZIPInputStream gzis = new GZIPInputStream(fis);
		ObjectInputStream in = new ObjectInputStream(gzis);
		Graph g = (Graph) in.readObject();
		Identifier i = (Identifier) in.readObject();
		TransactionManager tm = (TransactionManager) in.readObject();
		in.close();
		graph = g;
		drawArea.setDrawObjects(graph);
		stateHandler.setDrawObjects(graph);
		/* set all objects to fit the drawing area*/
		int [] graphMaxXMaxY = graph.scaleServers(
			(double)Server.getRadius() / 
			(double)Server.INITIAL_RADIUS);

		/*resize canvas according to contents*/
		if (graphMaxXMaxY[2] < getWidth()) {
	    	    graphMaxXMaxY[2] = getWidth();
		}
		if (graphMaxXMaxY[3] < getHeight()) {
	    	    graphMaxXMaxY[3] = getHeight();
		}
		drawArea.windowResized(new Dimension(graphMaxXMaxY[2], 
						     graphMaxXMaxY[3]));	

		drawArea.updateCanvas();
		identifier = i;
		manager = tm;
		graph.changeTransaction(manager.getCurrent());
		dialogManager = new TaviDialogs(this, manager.getTransactions());
		setActiveLabel(manager.getCurrent().getIdentifier());
		displayMessage("Model loaded succesfully!", false);
	    }
	    catch (Exception e) {
		displayMessage("Error! Model not loaded.", true);
	    }
	}
    }

    /**
     * Saves the current model/graph displayed on draw area to file. 
     */
    private void saveModel() {
	FileDialog f = new FileDialog(this, "Save model", FileDialog.SAVE);
	f.show();
	String filename = f.getFile();
	if (filename != null) {

 	    /* Scale the graph to initial radius. */
	    double multipl = (double)Server.INITIAL_RADIUS / 
		(double)Server.getRadius();
	    /*memorize the customer set server radius*/
	    int memRadius = Server.getRadius();
	    /*set server radius to default value*/
	    Server.setRadius(Server.INITIAL_RADIUS);
	    /*scale server coordinates according to radius change*/
	    graph.scaleServers(multipl);

	    /* Clears all selections. */
	    stateHandler.clearAllSelections();

	    /* Just in case, cut external references to graph. */
	    drawArea.setDrawObjects(null);

	    stateHandler.setDrawObjects(null);

	    try {
		FileOutputStream fos = new FileOutputStream(filename);
 		GZIPOutputStream gzos = new GZIPOutputStream(fos);
		ObjectOutputStream out = new ObjectOutputStream(gzos);
		out.writeObject(graph);
		out.writeObject(identifier);
		out.writeObject(manager);
		out.flush();
		out.close();
		/* restore references to graph before doing anything else. */
		drawArea.setDrawObjects(graph);
 		stateHandler.setDrawObjects(graph);
		/* Scale graph back to its original size. */
		Server.setRadius(memRadius);
	 	graph.scaleServers(1 / multipl);	
		displayMessage("Model saved succesfully!", false);
	    }
	    catch (IOException e) {
		/* Restore references to graph before doing anything else. */
		drawArea.setDrawObjects(graph);
 		stateHandler.setDrawObjects(graph);
		/* Scale graph back to its original size. */
		Server.setRadius(memRadius);
	 	graph.scaleServers(1 / multipl);
		displayMessage("Error! Save not succesfull.", true);
	    }
	}
    }
}
