/* 
 * @(#)TaviCanvas 4.2.2002 version 0.1.2 
 * 
 * Copyright University of Helsinki / Department of computer science
 * 
 */ 

package tavi;

import javax.swing.*;

import java.awt.Graphics;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Rectangle;

import java.util.LinkedList; 

import tavi.graph.*;

/** 
 * Implementation of drawing area. TaviCanvas is very light class, because
 * drawing is made in each drawable object's drawSelf method. Size handling
 * of this class is somewhat complex, because the drawing area must be
 * resized when the Tavi window's size is increased. 
 * 
 * Functions related to the drawing area (itself) are implemented here.
 * For example changing the size,background color and 
 * updating of the drawing area.
 * <p>
 * Drawing individual objects is done by calling their drawSelf() -methods.
 * Graphics instance from this class is passed to the objects. 
 *
 * @version     0.1.2 11 Feb 2002 
 * @author      Ukkonen Johannes
 * @author      Kimmo Airamaa
 */ 
public class TaviCanvas extends JPanel {
    /**
     * Width of draw area.
     */
    protected int width;
    /**
     * Height of draw area.
     */
    protected int height;
    /** 
     * Link to Graph of objects to be drawn.
     */
    protected Graph drawObjects;
    /**
     * Current background color for the drawing area.
     */
    protected Color editorColor;
    /**
     * Canvas constructor. Sets drawing area's background color white and
     * sets area's size 640 * 480. Gets an link to an implementation of
     * graph which includes all objects (servers and flows).
     * @param graph Graph containing all servers and flows.
     */
    public TaviCanvas(Graph graph) {
	editorColor = Color.white;
	drawObjects = graph;
	width = 640;
	height = 480;
	changeSize(width, height);
    }

    /**
     * Changes the size of this component. Width and height must
     * exceed 100 pixels.
     * @param newWidth New width of drawing area.
     * @param newHeight New height of drawing area.
     */
    public void changeSize(int newWidth, int newHeight) {
	if (newWidth > 100 && newHeight >100) {
	    width = newWidth;
	    height = newHeight;
	    setPreferredSize(new Dimension(newWidth, newHeight));
	    revalidate();
	}
    }

    /**
     * Changes the preferred size of TaviCanvas. The new width and height is 
     * maximum of the existing and supplied. 
     * @param d Size of the resized window.
     */
    public void windowResized(Dimension d) {
	int i = (int)d.getWidth();
	int j = (int)d.getHeight();
	Dimension dd = getPreferredSize();
	int ii = (int)dd.getWidth();
	int jj = (int)dd.getHeight();
	i = Math.max(i, ii);
	j = Math.max(j, jj);
	changeSize(i, j);
    }

    /**
     * Method used to change TaviCanvas background color.
     * @param c New color.
     */
    public void changeColor(Color c) {
	if (c != null) {
	    editorColor = c;
	}
	updateCanvas();
    }
  
    /** 
     * Handles custom painting of TaviCanvas object. 
     * @param g Standard Graphics object required for painting.
     */
    public void paintComponent(Graphics g) {
	g.setColor(editorColor);
        g.fillRect(0, 0, width, height);
	/* draw every object */
	drawObjects.drawSelf(g);
    } 

    /**
     * Returns the width of drawing area.
     * @return Actual drawing area height.
     */
    public int getActualWidth() {
	return width;
    }

    /**
     * Returns the height of drawing area.
     * @return Actual drawing area height.
     */
    public int getActualHeight() {
	return height;
    }

    /**
     * Updates the drawing area. Simply calls repaint(). 
     */
    public void updateCanvas(){
	repaint();
    }

    /**
     * Sets the reference to the graph which is to be displayed.
     * Used when changing the graph without creating new TaviCanvas object.
     */
    public void setDrawObjects(Graph g) {
	drawObjects = g;
    }
}
