/* 
 * @(#)TaviDialogs 11.5.2002 1.0 
 * 
 * Copyright University of Helsinki / Department of computer science
 * 
 */ 

package tavi;

import java.awt.*;
import java.awt.event.*;

import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

import tavi.graph.*;

/** 
 * TaviDialogs is a class that handles dialog creation, destruction and
 * event handling for the dialogs' items. Dialogs can be created by
 * method calls. 
 * <p>
 * Note that a new window usually writes over the previous one, so it is
 * not recommended to open several windows at a time. Editing transactions
 * and variables is so far done with provisional dialog and buttons and 
 * therefore there can be two dialog windows open at the same time. This
 * isn't quite stylish implementation, however. Creating three (or more)
 * overlapping dialogs with TaviDialogs is highly discouraged. 
 * <p>
 * This class has some instances of data structures that are local for
 * transactions. It should be noted that when removing transactions 
 * these instances should be corrected accordingly. Otherwise user has
 * access to wrong variable lists, for example. These instances are added 
 * to make code a bit more readable.
 *
 * @version     1.0 11 May 2002 
 * @author      Ukkonen Johannes
 * @author      Airamaa Kimmo
 * @author      Kupiainen Esko
 * @author      Tuominen Heikki
 */ 
public class TaviDialogs implements ActionListener, ListSelectionListener {
    /**
     * Constant representing the value used when running
     * message dialog.
     */
    private static final int MESSAGE_DIALOG = 0;
    /**
     * Constant representing the value used when running
     * server's property dialog.
     */
    private static final int SERVER_PROPERTIES = 1;
    /**
     * Constant representing the value used when running
     * flow's property dialog.
     */
    private static final int FLOW_PROPERTIES = 2;
    /**
     * Constant representing the value used when running
     * drawing area size dialog.
     */
    private static final int DRAWING_AREA_SIZE = 3;
    /**
     * Constant representing the value used when running
     * a dialog where the name of new transaction is set.
     */
    private static final int TRANSACTION_NAME = 4;
    /**
     * Constant representing the value used when running
     * a dialog that displays all transactions.
     */
    private static final int VIEW_TRANSACTIONS = 5;
    /**
     * Constant representing the value used when running
     * a dialog that displays all transactions in a combobox.
     */
    private static final int CHANGE_CURRENT = 6;
    /**
     * Constant representing the value used when running
     * a dialog that displays system preferences.
     */
    private static final int PREFERENCES = 7;
    /**
     * Constant representing the value used when running
     * a dialog where new variable can be created.
     */
    private static final int CREATE_VARIABLE = 8;
    /**
     * Constant representing the value used when running
     * a dialog that displays all variables.
     */
    private static final int VIEW_VARIABLES = 9;
    /**
     * Constant representing the value used when running
     * a dialog where new global variable can be created.
     */
    private static final int CREATE_GLOBAL_VARIABLE = 10;
    /**
     * Constant representing the value used when running
     * a dialog that displays all global variables.
     */
    private static final int VIEW_GLOBAL_VARIABLES = 11;
    /**
     * Dialog that is used to display all information required in
     * the current task.
     */
    private JDialog taviDialog;
    /**
     * Dialog to be used when taviDialog is reserved and there is a need
     * for other dialogs. 
     */
    private JDialog messageDialog;

    /**
     * Button that usually closes windows ('OK') or commits changes.
     */
    private JButton okButton;
    /** 
     * Another button that closes windows and commits changes. Used when
     * okButton is already assigned for some purpose.
     */
    private JButton applyButton;
    /**
     * Button that aborts changes in any case. 
     */
    private JButton cancelButton;
    /**
     * Button that edits selection.
     */
    private JButton editButton;
    /**
     * Button that deletes selection.
     */
    private JButton deleteButton;
    /**
     * Button that is used only when viewing TFlow properties. Changes
     * the direction of TFlow.
     */
    private JButton changeFlowDirection;
    
    /**
     * Textfield that is used to display object's name. It is also
     * possible to change object's name.
     */
    private JTextField objectNameField;
    /**
     * Textfield used to display and change server's queue length.
     */ 
    private JTextField serverQueueLength;
    /**
     * Textfield used to display and edit transaction flow's scheme.
     */
    private JTextField flowScheme;
    /**
     * Textfield used to input new drawing area width.
     */
    private JTextField inputWidth;
    /**
     * Textfield used to input new drawing area height.
     */
    private JTextField inputHeight;
    /**
     * Textfield used to input initial values.
     */
    private JTextField initValue;

    /**
     * List to display vector of some objects, for example transactions
     * and variables. 
     */
    private JList objectList;
    /** 
     * Combobox from which user can select one transaction.
     */
    private JComboBox selectTransaction;
    
    /**
     * Displays flow's start and end servers.
     */
    private JPanel flowRoute;
    /**
     * Common panel used for various purposes.
     */
    private JPanel dialogPanel;

    /**
     * The 'temporary' storage place of identifier.
     */
    private String identifier;

   /**
     * Reference to Tavi object.
     */
    private Tavi gui;
    
    /**
     * Vector containing all transactions. Instance to the transactions is
     * fetched in the constructor. 
     */
    private Vector transactions;
    /**
     * Vector containing currently active transaction's variables.
     * It is imperative that these variables are all the time up to
     * date.
     */
    private Vector variables;
    /**
     * Vector that contains all global variables. 
     */
    private Vector globalVariables;

    /**
     * Variable used to store current window's type. Used in the event
     * handling.
     */
    private int currentWindow;
    
    /**
     * Field for customer to enter max number of recursive edges.
     */
    private JTextField inputRecursive;
    /**
     * Field for customer to enter server radius.
     */
    private JTextField inputRadius;
    /**
     * Field for customer to enter message timeout. 
     */
    private JTextField inputTimeOut;
    /**
     * Field for customer to toggle edge selection area visibility.
     * Selection area is the blue ellipse where flow's identification
     * number is printed.
     */
    private JCheckBox inputShowSelection;
 
    /**
     * Initializes the class to be used with specific Tavi window
     * and Timer object. Initially, there are no active windows.
     * @param owner Reference to Tavi window. Needed to create modal
     * dialogs.
     * @param allTransactions Vector where all transactions reside.
     */
    public TaviDialogs(Tavi owner, Vector allTransactions) {
	gui = owner;
	transactions = allTransactions;
	/* Fetches currently active transaction's variables. */
	variables = gui.manager.getCurrent().getVariables();
	globalVariables = gui.manager.getGlobalVariables();
	currentWindow = -1;
    }

    /**
     * Sets the reference to Vector containing all transactions. 
     * @param t Reference to Vector containing all transactions.
     */
    public void setTransactions(Vector t) {
	transactions = t;
    }

    /**
     * Creates a message window with requested text. Used when opening
     * more than one dialog at a time.
     * @param text Text to be displayed in window.
     */
    public void message(String text) {
	/* If there is a message window already, don't create new */
	if (gui.timer.isCountDownRunning()) {
	    return;
	}
	messageDialog = new JDialog(gui, "Notice", true);
	
	gui.timer.startCountDown(gui.timeOut, messageDialog);
	
	JPanel messagePanel = new JPanel();
	messagePanel.setBackground(Color.white);
	messagePanel.add(new JLabel(text));

	/* Add components to message dialog */
	messageDialog.getContentPane().add("North", messagePanel);

	/* Set location of dialog window and make it visible*/
	messageDialog.pack();
	messageDialog.setLocation(200, 200);
	messageDialog.setVisible(true);
    }

    /**
     * Method that creates a message dialog containing the requested text.
     * The dialog can self-close after a (specified) timeout, or require
     * an acknowledgement from the user.
     * @param text Text to be displayed in the dialog.
     * @param ack When true, creates a dialog that requires an
     * acknowledgement from the user. Otherwise closes the window after
     * specified time.
     */
    public void messageDialog(String text, boolean acknowledge) {
	/* If there is a message window already, don't create new */
	if (gui.timer.isCountDownRunning()) {
	    return;
	}
	/* Mark the window type so that there's no question about it. */
	currentWindow = MESSAGE_DIALOG;

	taviDialog = new JDialog(gui, "Notice", true);
	
	if (!acknowledge) {
	    /*start countdown to close the message window after a timeout*/
	    gui.timer.startCountDown(gui.timeOut, taviDialog);
	} 

	JPanel messagePanel = new JPanel();
	messagePanel.setBackground(Color.white);
	messagePanel.add(new JLabel(text));

	/* Add components to message dialog */
	taviDialog.getContentPane().add("North", messagePanel);

	/* Add ok button if window needs acknowledgement. */
	if (acknowledge) {
	    okButton = new JButton("OK");
	    okButton.addActionListener(this);
	    taviDialog.getContentPane().add("South", okButton);
	}
	taviDialog.addWindowListener(new WindowAdapter() {
		    public void windowClosing(WindowEvent e) {
			gui.timer.stopCountDown();
		    }
	});
	
	/* Set location of dialog window and make it visible*/
	taviDialog.pack();
	taviDialog.setLocation(200, 200);
	taviDialog.setVisible(true);
    }

    /** 
     * Constructs a window that displays all transactions in a JList
     * component. Transactions can be edited and deleted from the list.
     */
    public void viewTransactions() {
	currentWindow = VIEW_TRANSACTIONS;

	taviDialog = new JDialog(gui, "Viewing transactions", true);
	objectList = new JList(transactions);
	objectList.setFont(new Font("Courier", Font.PLAIN, 12));
	/* Add components to dialog. */
	okButton = new JButton("OK");
	deleteButton = new JButton("Delete");
	editButton = new JButton("Edit");
	/* When nothing has been selected, nothing can be edited or
	   deleted. */
	editButton.setEnabled(false);
	deleteButton.setEnabled(false);
	dialogPanel = new JPanel();
	JPanel buttonPanel = new JPanel();
	JPanel okPanel = new JPanel();
	dialogPanel.add(objectList);
	okPanel.add(okButton);
	buttonPanel.add(editButton);
	buttonPanel.add(deleteButton);

	taviDialog.getContentPane().add("North", dialogPanel);
	taviDialog.getContentPane().add("Center", buttonPanel);
	taviDialog.getContentPane().add("South", okPanel);
	
	okButton.addActionListener(this);
	editButton.addActionListener(this);
	deleteButton.addActionListener(this);
	objectList.addListSelectionListener(this); 

	taviDialog.pack();
	taviDialog.setLocation(200, 200);
	taviDialog.setVisible(true);
    }

    /**
     * Constructs a dialog window that contains variable's value field.
     * Value can be altered in this window.
     * @param variable Variable whose value is to be displayed.
     */
    public void statusDialog(Variable variable) {
	messageDialog = new JDialog(gui, "Variable's value", true);
	messageDialog.getContentPane().setLayout(new GridLayout(2, 1));
	messageDialog.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    ;
		}
	    });
	
	JPanel valuePanel = new JPanel();
	valuePanel.add(new JLabel("" + variable.getName() + ": "));
	initValue = new JTextField("" + variable.getValue(), 10);
	valuePanel.add(initValue);

	JPanel buttonPanel = new JPanel();
	applyButton = new JButton("OK");
	cancelButton = new JButton("Cancel");
	buttonPanel.add(applyButton);
	buttonPanel.add(cancelButton);
	
	/* Add panels to the dialog. */
	messageDialog.getContentPane().add(valuePanel);
	messageDialog.getContentPane().add(buttonPanel);

	/* Add listeners to buttons. */
	applyButton.addActionListener(this);
	cancelButton.addActionListener(this);
	
	/* Set location of a dialog window and make it visible. */
	messageDialog.pack();
	messageDialog.setLocation(200, 200);
	messageDialog.setVisible(true);
    }

    /**
     * Constructs a dialog window that contains global variable's value field.
     * Value can be altered in this window.
     * @param variable Global variable whose value is to be displayed.
     */
    public void variableStatusDialog(Variable variable) {
	messageDialog = new JDialog(gui, "Variable's value", true);
	messageDialog.getContentPane().setLayout(new GridLayout(2, 1));
	messageDialog.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    ;
		}
	    });
	
	JPanel valuePanel = new JPanel();
	valuePanel.add(new JLabel("" + variable.getName() + ": "));
	initValue = new JTextField("" + variable.getValue(), 10);
	valuePanel.add(initValue);

	JPanel buttonPanel = new JPanel();
	applyButton = new JButton("OK");
	cancelButton = new JButton("Cancel");
	buttonPanel.add(applyButton);
	buttonPanel.add(cancelButton);
	
	/* Add panels to the dialog. */
	messageDialog.getContentPane().add(valuePanel);
	messageDialog.getContentPane().add(buttonPanel);

	/* Add listeners to buttons. */
	applyButton.addActionListener(this);
	cancelButton.addActionListener(this);
	
	/* Set location of a dialog window and make it visible. */
	messageDialog.pack();
	messageDialog.setLocation(200, 200);
	messageDialog.setVisible(true);
    }

    /**
     * Constructs a dialog window containing transaction's properties.
     * @param transaction Transaction whose properties are to be displayed.
     */
    public void statusDialog(Transaction transaction) {
	messageDialog = new JDialog(gui, "Properties", true);
	messageDialog.getContentPane().setLayout(new GridLayout(3, 1));
	messageDialog.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    ;
		}
	    });    
	/* Make the uppermost panel in the dialog window. Server's
	   identifier and full name are displayed. */
	JPanel temp1 = new JPanel();
	temp1.add(new JLabel(transaction.getIdentifier() + ": "));
	objectNameField = new JTextField(15);
	objectNameField.setText("" + transaction.getName());
	temp1.add(objectNameField);
	
	/* Make the final panel of the window. Contains buttons to commit
	   or abort changes to textfields. */
	JPanel temp2 = new JPanel();
	applyButton = new JButton("OK");
	cancelButton = new JButton("Cancel");
	temp2.add(applyButton);
	temp2.add(cancelButton);
	/* Add listeners to buttons. */
	applyButton.addActionListener(this);
	cancelButton.addActionListener(this);
	
	/* Add all panels to dialog */
	messageDialog.getContentPane().add(temp1);
	messageDialog.getContentPane().add(temp2);
	
	/* Set location of a dialog window and make it visible. */
	messageDialog.pack();
	messageDialog.setLocation(150, 150);
	messageDialog.setVisible(true);
    }
    
    
    /**
     * Constructs a dialog window containing server's properties. Everything
     * except identifier can be changed by editing textfields.
     * @param server Server whose properties are to be displayed.
     */
    public void statusDialog(Server server) {
	/* Mark the current window as it should be. */
	currentWindow = SERVER_PROPERTIES;

	taviDialog = new JDialog(gui, "Server " + 
		server.getIdentifier() + " Properties", true);
	taviDialog.getContentPane().setLayout(
							new GridLayout(4, 1));
	taviDialog.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    ;
		}
	    });    
	/* Make the uppermost panel in the dialog window. Server's
	   identifier and full name are displayed. */
	JPanel temp1 = new JPanel();
	temp1.add(new JLabel("Name"));
	objectNameField = new JTextField(15);
	objectNameField.setText("" + server.getName());
	temp1.add(objectNameField);
	
	/* Make the 'middle' panel in the dialog. Contains so far
	   object's queue length information. */
	JPanel temp2 = new JPanel();
	serverQueueLength = new JTextField(3);
	serverQueueLength.setText("" + server.queueLength());
	temp2.add(new JLabel("Queue length: "));
	temp2.add(serverQueueLength);
	JPanel temp4 = new JPanel();
	inputWidth = new JTextField(4);
	inputHeight = new JTextField(4);
	inputWidth.setText("" + server.getLocation().x);
	inputHeight.setText("" + server.getLocation().y);
	temp4.add(new JLabel("Center x y"));
	temp4.add(inputWidth);
	temp4.add(inputHeight);
	
	/* Make the final panel of the window. Contains buttons to commit
	   or abort changes to textfields. */
	JPanel temp3 = new JPanel();
	okButton = new JButton("OK");
	cancelButton = new JButton("Cancel");
	temp3.add(okButton);
	temp3.add(cancelButton);
	/* Add listeners to buttons. */
	okButton.addActionListener(this);
	cancelButton.addActionListener(this);
	
	/* Add all panels to dialog */
	taviDialog.getContentPane().add(temp1);
	taviDialog.getContentPane().add(temp2);
	taviDialog.getContentPane().add(temp4);
	taviDialog.getContentPane().add(temp3);
	
	/* Set location of a dialog window and make it visible. */
	taviDialog.pack();
	taviDialog.setLocation(150, 150);
	taviDialog.setVisible(true);
    }

    /** 
     * Creates a window containing flow's properties. Some properties can
     * be changed in this dialog.
     * @param flow Flow whose properties are to be displayed.
     */
    public void statusDialog(TFlow flow) {
	/* Mark the window type. */
	currentWindow = FLOW_PROPERTIES;

	taviDialog = new JDialog(gui, "Edge " + 
		flow.getIdentifier() +" Properties", true);
	taviDialog.getContentPane().setLayout(new GridLayout(6, 1));
	taviDialog.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    ;
		}
	    });   
	objectNameField = new JTextField(flow.getName(), 15);
	flowScheme = new JTextField(flow.getScheme(), 30);
	
	/* Create a label indicating flow's origin and destination. */
	JLabel fromLabel = new JLabel("From ");
	JLabel sourceLabel = new JLabel((flow.getSource()).getIdentifier());
	JLabel toLabel = new JLabel(" To "); 
	JLabel destinationLabel = new JLabel(
				  (flow.getDestination()).getIdentifier());
	
	/* Variable to indicate whether there is room for
	   recursive arc in other main directions. */
	boolean isRoom = false;
	if (flow.isRecursion()) {
	    destinationLabel.setText("Recursive " + flow.directionName(
		flow.giveArcDirection()));
	    for (int i = TFlow.NORTH; i <= TFlow.WEST; ++i) {
		if (i != flow.giveArcDirection() && 
		    flow.getSource().arcsInThisDirection(i) 
		    < gui.getRecursiveMax()) {
		    isRoom = true;
		    break;
		}
	    }
	    changeFlowDirection = new JButton("Change main direction: "+
				flow.directionName(flow.giveArcDirection()));
	}
	else {
	    changeFlowDirection = new JButton("Change direction: From " +
				flow.getSource().getIdentifier() + " To "+
				flow.getDestination().getIdentifier());
	}
	flowRoute = new JPanel();
	
	if (!flow.isRecursion()) {
	    flowRoute.add(fromLabel);
	    flowRoute.add(sourceLabel);
	    flowRoute.add(toLabel);
	}
	else {
	    flowRoute.add(sourceLabel);
	}
	flowRoute.add(destinationLabel);
	okButton = new JButton("OK");
	cancelButton = new JButton("Cancel");
	
	/* Check that Begin state cannot be set as destination,
	   end state cannot be set as source and recursion edges
	   cannot be swapped. */
	if (((flow.getSource()).getIdentifier()).equals("B") ||
	    ((flow.getDestination()).getIdentifier()).equals("E") ||
	    flow.isRecursion() && !isRoom) {
	    changeFlowDirection.setEnabled(false);
	}
	
	JPanel namePanel = new JPanel();
	namePanel.add(new JLabel("Name: "));
	namePanel.add(objectNameField);
	JPanel schemePanel = new JPanel();
	schemePanel.add(new JLabel("Scheme: "));
	schemePanel.add(flowScheme);
	JPanel temp = new JPanel();
	temp.add(okButton);
	temp.add(cancelButton);
	
	/* Add components to dialog */
	taviDialog.getContentPane().add(namePanel);
	taviDialog.getContentPane().add(schemePanel);
	taviDialog.getContentPane().add(flowRoute);
	taviDialog.getContentPane().add(changeFlowDirection);
	taviDialog.getContentPane().add(temp);
	
	/* Add listeners to buttons. */
	changeFlowDirection.addActionListener(this);
	okButton.addActionListener(this);
	cancelButton.addActionListener(this);
	
	/* Set location of a dialog window and make it visible. */
	taviDialog.pack();
	taviDialog.setLocation(150, 150);
	taviDialog.setVisible(true);
    }

    /**
     * Creates a window that is used to change the currently
     * active transaction.
     */
    public void changeActiveTransaction() {
	currentWindow = CHANGE_CURRENT;
	taviDialog = new JDialog(gui, "Change active transaction", true);
	taviDialog.getContentPane().setLayout(
					      new GridLayout(2, 1));
	/* Create a combobox that contains all the transactions. */
	selectTransaction = new JComboBox(transactions);
	selectTransaction.setFont(new Font("Courier", Font.PLAIN, 12));
	dialogPanel = new JPanel();
	dialogPanel.add(selectTransaction);

	okButton = new JButton("OK");
	cancelButton = new JButton("Cancel");
	JPanel buttonPanel = new JPanel();
	buttonPanel.add(okButton);
	buttonPanel.add(cancelButton);
	
	taviDialog.getContentPane().add(dialogPanel);
	taviDialog.getContentPane().add(buttonPanel);

	/* Add listeners to all dialog components. */
	okButton.addActionListener(this);
	cancelButton.addActionListener(this);
	
	taviDialog.pack();
	taviDialog.setResizable(false);
	taviDialog.setLocation(150, 150);
	taviDialog.setVisible(true);
    }

    /**
     * Creates a window in which new variable's name and initial
     * value can be set. 
     */
    public void newVariableDialog() {
	currentWindow = CREATE_VARIABLE;
	variables = gui.manager.getCurrent().getVariables();
	taviDialog = new JDialog(gui, "New variable", true);
	taviDialog.getContentPane().setLayout(new GridLayout(2, 1));
	
	objectNameField = new JTextField("", 20);
	initValue = new JTextField("0", 10);

	okButton = new JButton("OK");
	cancelButton = new JButton("Cancel");

	JPanel nameFieldPanel = new JPanel();
	JPanel buttonPanel = new JPanel();
	nameFieldPanel.add(new JLabel("name: "));
	nameFieldPanel.add(objectNameField);
	nameFieldPanel.add(new JLabel("value: "));
	nameFieldPanel.add(initValue);
	buttonPanel.add(okButton);
	buttonPanel.add(cancelButton);

	taviDialog.getContentPane().add(nameFieldPanel);
	taviDialog.getContentPane().add(buttonPanel);
	
	/* Add listeners to buttons. */
	okButton.addActionListener(this);
	cancelButton.addActionListener(this);
	
	/* Set location of a dialog window and make it visible. */
	taviDialog.pack();
	taviDialog.setLocation(200, 200);
	taviDialog.setVisible(true);
    }
    
    /**
     * Creates a window in which new global variable's name and initial
     * value can be set. 
     */
    public void newGlobalVariableDialog() {
	currentWindow = CREATE_GLOBAL_VARIABLE;
	taviDialog = new JDialog(gui, "New global variable", true);
	taviDialog.getContentPane().setLayout(new GridLayout(2, 1));
	
	objectNameField = new JTextField("", 20);
	initValue = new JTextField("0", 10);

	okButton = new JButton("OK");
	cancelButton = new JButton("Cancel");

	JPanel nameFieldPanel = new JPanel();
	JPanel buttonPanel = new JPanel();
	nameFieldPanel.add(new JLabel("name: "));
	nameFieldPanel.add(objectNameField);
	nameFieldPanel.add(new JLabel("value: "));
	nameFieldPanel.add(initValue);
	buttonPanel.add(okButton);
	buttonPanel.add(cancelButton);

	taviDialog.getContentPane().add(nameFieldPanel);
	taviDialog.getContentPane().add(buttonPanel);
	
	/* Add listeners to buttons. */
	okButton.addActionListener(this);
	cancelButton.addActionListener(this);
	
	/* Set location of a dialog window and make it visible. */
	taviDialog.pack();
	taviDialog.setLocation(200, 200);
	taviDialog.setVisible(true);
    }

    /**
     * Creates a window in which all currently active transaction's 
     * variables can be viewed and deleted. 
     */
    public void viewVariablesDialog() {
	currentWindow = VIEW_VARIABLES;

	taviDialog = new JDialog(gui, "Variables", true);
	objectList = new JList(variables);
	objectList.setFont(new Font("Courier", Font.PLAIN, 12));
	/* Add components to dialog. */
	okButton = new JButton("OK");
	deleteButton = new JButton("Delete");
	editButton = new JButton("Edit");
	/* When nothing has been selected, nothing can be edited or
	   deleted. */
	editButton.setEnabled(false);
	deleteButton.setEnabled(false);
	dialogPanel = new JPanel();
	JPanel buttonPanel = new JPanel();
	JPanel okPanel = new JPanel();
	dialogPanel.add(objectList);
	okPanel.add(okButton);
	buttonPanel.add(editButton);
	buttonPanel.add(deleteButton);

	taviDialog.getContentPane().add("North", dialogPanel);
	taviDialog.getContentPane().add("Center", buttonPanel);
	taviDialog.getContentPane().add("South", okPanel);
	
	okButton.addActionListener(this);
	editButton.addActionListener(this);
	deleteButton.addActionListener(this);
	objectList.addListSelectionListener(this); 

	taviDialog.pack();
	taviDialog.setLocation(200, 200);
	taviDialog.setVisible(true);
    }

    /**
     * Creates a window in which global variables can be viewed and deleted. 
     * Uses a JList component to display all global variables.
     */
    public void viewGlobalVariablesDialog() {
	currentWindow = VIEW_GLOBAL_VARIABLES;

	taviDialog = new JDialog(gui, "Global variables", true);
	objectList = new JList(globalVariables);
	objectList.setFont(new Font("Courier", Font.PLAIN, 12));
	/* Add components to dialog. */
	okButton = new JButton("OK");
	deleteButton = new JButton("Delete");
	editButton = new JButton("Edit");
	/* When nothing has been selected, nothing can be edited or
	   deleted. */
	editButton.setEnabled(false);
	deleteButton.setEnabled(false);
	dialogPanel = new JPanel();
	JPanel buttonPanel = new JPanel();
	JPanel okPanel = new JPanel();
	dialogPanel.add(objectList);
	okPanel.add(okButton);
	buttonPanel.add(editButton);
	buttonPanel.add(deleteButton);

	taviDialog.getContentPane().add("North", dialogPanel);
	taviDialog.getContentPane().add("Center", buttonPanel);
	taviDialog.getContentPane().add("South", okPanel);
	
	okButton.addActionListener(this);
	editButton.addActionListener(this);
	deleteButton.addActionListener(this);
	objectList.addListSelectionListener(this); 

	taviDialog.pack();
	taviDialog.setLocation(200, 200);
	taviDialog.setVisible(true);
    }

    /**
     * Creates a window where new transaction could be created. It is 
     * possible to cancel the operation.
     * @param id Identifier of new transaction.
     */
    public void transactionNameDialog(String id) {
	identifier = id;
	currentWindow = TRANSACTION_NAME;
	
	taviDialog = new JDialog(gui, "New transaction", true);
	taviDialog.getContentPane().setLayout(new GridLayout(2, 1));
	JLabel nameLabel = new JLabel("Name for transaction " + id + ":");
	objectNameField = new JTextField("", 30);
	okButton = new JButton("OK");
	cancelButton = new JButton("Cancel");

	/* Add all components to dialog. */
	JPanel namePanel = new JPanel();
	namePanel.add(nameLabel);
	namePanel.add(objectNameField);
	JPanel buttonPanel = new JPanel();
	buttonPanel.add(okButton);
	buttonPanel.add(cancelButton);
	taviDialog.getContentPane().add(namePanel);
	taviDialog.getContentPane().add(buttonPanel);

	/* Add listeners to buttons. */
	okButton.addActionListener(this);
	cancelButton.addActionListener(this);

	taviDialog.pack();
	taviDialog.setResizable(false);
	taviDialog.setLocation(150, 150);
	taviDialog.setVisible(true);
    }

    /**
     * Creates a window where new drawing area size can be set.
     */
    public void windowSizeDialog() {
	currentWindow = DRAWING_AREA_SIZE;
	/* initialize objects used in JDialog*/ 
	inputWidth = new JTextField("" + gui.drawArea.getActualWidth(), 4);
	inputHeight = new JTextField("" + gui.drawArea.getActualHeight(), 4);
	okButton = new JButton("Ok");
	
	/* construct a dialog that prompts for a new size */
	taviDialog = new JDialog(gui, "Drawing area size", true);
	JPanel widthPanel = new JPanel();
	widthPanel.add(new JLabel("width (> 300): "));
	widthPanel.add(inputWidth);
	JPanel heightPanel = new JPanel();
	heightPanel.add(new JLabel("height (> 300 ): "));
	heightPanel.add(inputHeight);
	
	/* add components to JDialog */
	taviDialog.getContentPane().add("North", widthPanel);
	taviDialog.getContentPane().add("Center", heightPanel);
	taviDialog.getContentPane().add("South", okButton);
	
	/* add a listener for 'ok' button */
	okButton.addActionListener(this);
	
	/* set location of dialog window and make it visible*/
	taviDialog.pack();
        taviDialog.setLocation(100, 100);
	taviDialog.setVisible(true);
    }

    /**
     * Method for changing prefences. Maximum number of recursive edges,
     * server's drawing radius, selection area visibility and message
     * timeout can be set in this dialog window. 
     */
    public void changePreferences() {
	/* initialize objects used in JDialog*/
	currentWindow = PREFERENCES;
	inputRecursive = new JTextField("" + 
		Math.max(gui.getRecursiveMax(), gui.graph.getRecMax()), 2);
	inputRadius = new JTextField("" + Server.getRadius(), 2);
	inputTimeOut = new JTextField(""+gui.getTimeOut() / 1000, 2);
	inputShowSelection = new JCheckBox("", gui.getShowSelectionArea());
	okButton = new JButton("Ok");

	/* construct a dialog that prompts for system preferences*/
	taviDialog = new JDialog(gui, "System Preferences", true);
	JPanel recursivePanel = new JPanel();
	recursivePanel.add(new JLabel(
		"Max recursions per direction "+gui.graph.getRecMax()+
		".."+Server.MAX_RECURSIVE_TFLOWS_PER_DIRECTION));
        recursivePanel.add(inputRecursive);
	JPanel radiusPanel = new JPanel();
	radiusPanel.add(new JLabel("Server radius 20..50"));
	radiusPanel.add(inputRadius);
	JPanel timeOutPanel = new JPanel();
	timeOutPanel.add(new JLabel("Message timeout 1..10"));
	timeOutPanel.add(inputTimeOut);
	JPanel showSelectionPanel = new JPanel();
	showSelectionPanel.add(new JLabel("Show selection area"));
	showSelectionPanel.add(inputShowSelection);
	    
	/* add components to JDialog */	    
	taviDialog.getContentPane().add("North", recursivePanel);
	taviDialog.getContentPane().add("West", radiusPanel);
	taviDialog.getContentPane().add("Center", showSelectionPanel);
	taviDialog.getContentPane().add("East", timeOutPanel);
	taviDialog.getContentPane().add("South", okButton);
	    
	/* Add a listener for 'ok' button */
	okButton.addActionListener(this);

	/* Set location of dialog window and make it visible. */
	taviDialog.pack();
	taviDialog.setLocation(100, 100);
	taviDialog.setVisible(true);
    }

    /**
     * Sets the dialog and all components to null. Resets the variable
     * indicating the currently active window type.
     */
    public void resetAll() {
	taviDialog = null;
	applyButton = null;
	okButton = null;
	cancelButton = null;
	editButton = null;
	deleteButton = null;
	changeFlowDirection = null;
	objectNameField = null;
	serverQueueLength = null;
	flowScheme = null;
	inputWidth = null;
	inputHeight = null;
	flowRoute = null;
	currentWindow = -1;
    }

    /**
     * Method that performs the event handling of dialog's components.
     * @param e Event that has occurred.
     */
    public void actionPerformed(ActionEvent e) {
	Object cause = e.getSource();
	int inputX;
	int inputY;
	if (cause == okButton) {
	    if (currentWindow == SERVER_PROPERTIES) {
		/* 
		   When 'OK' button has been pressed in the server
		   properties dialog, modified values must be passed to
		   the server. If server's location is changed, it is
		   checked that it wouldn't overlap other objects.
		*/
		int length;
		
		serverQueueLength.setBackground(Color.white);
		/* If queue length input is invalid number or 
		   negative, the input field background is 
		   changed to red. */
		try {
		    length = Integer.parseInt(serverQueueLength.getText());
		} catch (NumberFormatException exc) {
		    serverQueueLength.setBackground(Color.red);
		    return;
		}
		if (length < 0) {
		    serverQueueLength.setBackground(Color.red);
		    return;
		}
		/* If location x-coordinate is invalid number  
		   or outside draw area, the input field background is 
		   changed to red. */
		try {
		    inputX = Integer.parseInt(inputWidth.getText());
		} catch (NumberFormatException exc) {
		    inputWidth.setBackground(Color.red);
		    return;
		}
		if (gui.stateHandler.getActivatedServer().getLocation().x - 
		      inputX > gui.stateHandler.getActivatedServer().
			getServerMinMax((double)1)[0]) {
		    inputWidth.setBackground(Color.red);
		    return;
		}
		
		/* If location y-coordinate is invalid number or 
		   or outside draw area, the input field background is 
		   changed to red. */
		try {
		    inputY = Integer.parseInt(inputHeight.getText());
		} catch (NumberFormatException exc) {
		    inputHeight.setBackground(Color.red);
		    return;
		}
		if (gui.stateHandler.getActivatedServer().getLocation().y - 
		      inputY > gui.stateHandler.getActivatedServer().
			getServerMinMax((double)1)[1]) {
		    inputHeight.setBackground(Color.red);
		    return;
		}
	       	boolean serverTooClose = false;
		boolean tFlowTooClose  = false;
		if (gui.graph.getClosestServer(new Point (inputX, inputY), 
					       1.0 * 
				       gui.getMinServerAddDistance(),
				       gui.stateHandler.getActivatedServer()
				       ) != null) {
		    serverTooClose = true;
		}
		if (!gui.graph.locationOk(new Point (inputX, inputY), 
			gui.stateHandler.getActivatedServer())) { 
		    tFlowTooClose = true;
		}
		
		if (serverTooClose || tFlowTooClose) {
		    inputWidth.setBackground(Color.red);
		    inputHeight.setBackground(Color.red);
		return;    
		}
		

		String tmp = objectNameField.getText();
		gui.stateHandler.editServerProperties(tmp, 
				length, new Point (inputX, inputY));
	    } else if (currentWindow == FLOW_PROPERTIES) {
		/* 
		   When OK button has been clicked in flow properties
		   dialog, changed values must be passed to the TFlow
		   object that is currently selected. Invalid selections
		   in the fields cause the field to turn to red.
		*/
		gui.stateHandler.editTFlowProperties(
		       objectNameField.getText(),
		       flowScheme.getText());
	    } else if (currentWindow == DRAWING_AREA_SIZE) {
		/* 
		   When OK button has been clicked in the dialog that
		   prompts the user for new drawing area size, given
		   values are checked and either new size is set or
		   invalid fields are set to red.
		*/
		/* get current display resolution */
		Toolkit tk = gui.getToolkit();
		Dimension displayResolution = tk.getScreenSize();
		int displayHt = (int) displayResolution.getHeight();
		int displayWt = (int) displayResolution.getWidth();
		int requestedHeight = 0;
		int requestedWidth = 0;
		
		/* Fields may be red already. Before testing their
		   values, one cannot be sure. */
		inputWidth.setBackground(Color.white);
		inputHeight.setBackground(Color.white);
		/* Boolean used in case both fields are incorrect.
		   Only after checking both fields decision is made. */
		boolean valid = true; 
		/* Get values from input fields and change size of drawArea. */
		try {
		    requestedWidth = Integer.parseInt(inputWidth.getText());
		} catch (NumberFormatException exc) {
		    inputWidth.setBackground(Color.red);
		    valid = false;
		}
		
		try {
		    requestedHeight = Integer.parseInt(inputHeight.getText());
		} catch (NumberFormatException ex) {
		    inputHeight.setBackground(Color.red);
		    valid = false;
		}
		/* If user has entered illegal values, it's up to
		   user to correct the mistake. Dialog will not be
		   closed if nothing is done. */
		if (!valid || requestedHeight < gui.HEIGHT_SMALL || 
		    requestedWidth < gui.WIDTH_SMALL) {
		    if (requestedWidth < gui.WIDTH_SMALL) {
			inputWidth.setBackground(Color.red);
		    } 
		    if (requestedHeight < gui.HEIGHT_SMALL) {
			inputHeight.setBackground(Color.red);
		    }
		    return;
		}
		/* Check that new size wouldn't be too small to display all
		   objects. */
		if (!gui.graph.isSizeChangeRecommended(requestedWidth, 
						   requestedHeight)) {
		    message("All objects wouldn't fit in.");
		    return;
		}
		/* maximum window size is display resolution */
		int windowHt = Math.min(requestedHeight, displayHt);
		int windowWt = Math.min(requestedWidth, displayWt);
		gui.setSize(windowWt, windowHt);
		gui.drawArea.changeSize(requestedWidth, requestedHeight);


		/* remove action listener just in case */
		okButton.removeActionListener(this);
		taviDialog.dispose();   
		taviDialog.getContentPane().remove(okButton);
	    
	    } else if (currentWindow == TRANSACTION_NAME) {
		/* 
		   Accepting new transaction's name. If given name is an
		   empty String, it will not be passed onward. Otherwise
		   transaction is created and set active. 
		*/
		if (objectNameField.getText().equals("")) {
		    objectNameField.setBackground(Color.red);
		    return;
		}

		/* Ask transaction manager to create a new transaction. */
		gui.manager.addTransaction(objectNameField.getText(), 
					   identifier);

		gui.graph.changeTransaction(gui.manager.getCurrent());
		gui.stateHandler.setFlowIdentifier(gui.manager.getCurrent()
						   .getFlowIdentifier());
		gui.setActiveLabel(gui.manager.getCurrent().getIdentifier());
		variables = gui.manager.getCurrent().getVariables();
		identifier = null;
	    } else if (currentWindow == CHANGE_CURRENT) {
		/*
		  When OK button has been clicked in a dialog where it is
		  possible to change currently active transaction, the 
		  selected transaction is set as active transaction.
		*/
		gui.manager.setCurrent((Transaction)
				       selectTransaction.getSelectedItem());
		gui.graph.changeTransaction((Transaction)
				       selectTransaction.getSelectedItem());
		gui.stateHandler.setFlowIdentifier(((Transaction)
		    selectTransaction.getSelectedItem()).getFlowIdentifier());
		gui.setActiveLabel(gui.manager.getCurrent().getIdentifier());
		variables = gui.manager.getCurrent().getVariables();
	    } else if (currentWindow == PREFERENCES) {
		/*
		  When accepting preferences, all fields are checked and then 
		  all changes are made. Fields containing invalid values are
		  marked red.
		*/

		/* Fields may be red already. Before testing their
		   values, one cannot be sure. */
		inputRecursive.setBackground(Color.white);
		inputRadius.setBackground(Color.white);
		inputTimeOut.setBackground(Color.white);
		/* Boolean used in case some fields are incorrect.
		   Only after checking all fields decision is made. */
		boolean valid = true; 
		int recursiveTemp = 0;
		int radiusTemp = 0;
		int timeTemp = 0;
		
		/* Get values from input fields and change variables. */
		try {
		    recursiveTemp = Integer.parseInt(inputRecursive.getText());
		} catch (NumberFormatException exc) {
		    inputRecursive.setBackground(Color.red);
		    valid = false;
		}
		
		try {
		    radiusTemp = Integer.parseInt(inputRadius.getText());
		} catch (NumberFormatException ex) {
		    inputRadius.setBackground(Color.red);
		    valid = false;
		}
		
		try {
		    timeTemp = Integer.parseInt(inputTimeOut.getText());
		} catch (NumberFormatException ex) {
		    inputTimeOut.setBackground(Color.red);
		    valid = false;
		}
		
		/* If user has entered illegal values, it's up to
		   user to correct the mistake. Dialog will not be
		   closed if nothing is done. */
		if (!valid || 
		    recursiveTemp > Server.MAX_RECURSIVE_TFLOWS_PER_DIRECTION || 
		    recursiveTemp < gui.graph.getRecMax() || radiusTemp < 20 
		    || radiusTemp > 50 || timeTemp < 1 || timeTemp > 10) {
		    if (recursiveTemp < gui.graph.getRecMax() ||
			recursiveTemp > 
			Server.MAX_RECURSIVE_TFLOWS_PER_DIRECTION) { 
		  
			inputRecursive.setBackground(Color.red);
		    } 
		    if (radiusTemp < 20 || radiusTemp > 50) {
			inputRadius.setBackground(Color.red);
		    }
		    if (timeTemp < 1 || timeTemp > 10) {
			inputTimeOut.setBackground(Color.red);
		    }
		    return;
		}
	    
		gui.setRecursiveMax(recursiveTemp);
		gui.setTimeOut(timeTemp * 1000);
		
		/*scale server coordinates according to new radius*/
		double multipl = (double)radiusTemp / 
		(double)(Server.getRadius());
 
		/*set new radius for general use*/
		Server.setRadius(radiusTemp);

		/*scale servers' coordinates according to new radius
		index 2 = max X, index 3 = max Y*/
		int [] graphMaxXMaxY = gui.graph.scaleServers
			(multipl);
		
		/*set edges' selection areas' visibility*/
		gui.selectionArea = inputShowSelection.isSelected();
		gui.graph.setSelection(gui.getShowSelectionArea());
				
	    } else if (currentWindow == CREATE_VARIABLE) {
		/*
		  When OK button is clicked in a dialog that asks the
		  user to input a name and a value for new variable,
		  given values are checked first. Variables cannot have
		  duplicate names in their scope.
		  ---
		  It should be noted that variables SHOULD NOT HAVE the same
		  name as global variables and vice versa. This is not yet 
		  checked.
		*/
		String nameTemp = objectNameField.getText();
		float valueTemp;
		try {
		    valueTemp = Float.parseFloat(initValue.getText());
		} catch (Exception exc) {
		    initValue.setBackground(Color.red);
		    return;
		}
		Variable variableTemp = new Variable(nameTemp, valueTemp);
		if (!gui.manager.getCurrent().addVariable(variableTemp)) {
		    objectNameField.setBackground(Color.red);
		    return;
		}
	    } else if (currentWindow == CREATE_GLOBAL_VARIABLE) {
		/*
		  When creating global variable, the new variable's
		  name must be unique and its value numeric. Fields
		  containing invalid selections are marked by setting
		  their background red.
		*/
		String nameTemp = objectNameField.getText();
		float valueTemp;
		try {
		    valueTemp = Float.parseFloat(initValue.getText());
		} catch (Exception exc) {
		    initValue.setBackground(Color.red);
		    return;
		}
		if (nameTemp.equals("") || nameTemp == null) {
		    nameTemp = "default";
		}
		if (!gui.manager.addGlobalVariable(nameTemp, valueTemp)) {
		    objectNameField.setBackground(Color.red);
		    return;
		}
	    }
	    /* Close the dialog and make it reusable. */
	    if (changeFlowDirection != null) {
		changeFlowDirection.removeActionListener(this);
	    }
	    okButton.removeActionListener(this);
	    if (cancelButton != null) {
		cancelButton.removeActionListener(this);
	    }
	    taviDialog.dispose();
	    resetAll();
	} else if (cause == cancelButton) {
	    /*
	      CancelButton cancels all selections and closes the
	      dialog that is on the top. Usually taviDialog, but
	      when there are multiple dialogs, it is messageDialog.
	    */
	    if (currentWindow == TRANSACTION_NAME) {
		gui.identifier.releaseTransactionId(identifier);
		identifier = null;
	    } 
	    cancelButton.removeActionListener(this);
	    if (messageDialog != null) {
		messageDialog.dispose();
		messageDialog = null;
	    } else {
		taviDialog.dispose();
		resetAll();
	    }
	} else if (cause == changeFlowDirection) {
	    /* Change the flow's direction and make it 
	       immediately, no matter if cancel button is clicked after
	       selecting this action. 
	    */
	    gui.stateHandler.changeTFlowDirection((JButton)cause);
	} else if (cause == editButton) {
	    /* 
	       Edit button allows the user to edit selected object.
	       New window will open atop the current dialog. It should
	       be noted that this dialog window cannot be created using the
	       same taviDialog as earlier.
	    */
	    if (currentWindow == VIEW_TRANSACTIONS) {
		/*
		  Transactions' name can be edited. It doesn't make 
		  difference, if there are duplicate names. Identifier
		  value makes all transactions unique.
		*/
		Transaction temp = (Transaction)objectList.getSelectedValue();
		statusDialog(temp);
	    } else if (currentWindow == VIEW_VARIABLES) {
		/*
		  Variables' value can be edited. If user wants to rename
		  a variable, it is done by removing the 'old' variable and
		  adding new. These variables are local to transactions.
		*/
		statusDialog((Variable)objectList.getSelectedValue());
	    } else if (currentWindow == VIEW_GLOBAL_VARIABLES) {
		/*
		  As with local variables, global variables' name cannot
		  be edited. Only their value. 
		*/
		variableStatusDialog((Variable)objectList.getSelectedValue());
	    }
	} else if (cause == deleteButton) {
	    if (currentWindow == VIEW_TRANSACTIONS) {
		/*
		  In transaction list, the selected transactions can be
		  deleted, if all transactions have not been selected.
		  There must always be one transaction left. This checking
		  is done mainly elsewhere. If currently active transaction
		  is deleted, some transaction in the structure must be set
		  as active. It is the transaction that is in the index 0 in
		  the structure.
		*/
		Object[] removeThese = objectList.getSelectedValues();
		boolean activeDeleted = false;
		for (int i = 0; i < removeThese.length; ++i) {
		    if (removeThese[i] == gui.manager.getCurrent()) {
			activeDeleted = true;
		    }
		    /* Identifier must be released when 
		       transaction is deleted. */
		    gui.identifier.releaseTransactionId(
		      ((Transaction)removeThese[i]).getIdentifier());
		    gui.manager.removeTransaction(removeThese[i]);
		}
		/* If active transaction is deleted, some transaction
		   must be set active, or else 'history' transaction
		   will be shown. */
		if (activeDeleted) {
		    Transaction candidate = (Transaction)gui.manager
			.getTransactions().elementAt(0);
		    gui.manager.setCurrent(candidate);
		    gui.graph.changeTransaction(candidate);
		    gui.stateHandler.setFlowIdentifier(candidate
						       .getFlowIdentifier());
		    gui.setActiveLabel(candidate.getIdentifier());
		    variables = candidate.getVariables();
		}
		/* List must be updated, so that deleted values are not
		   displayed. */
	        objectList.setListData(transactions);
		taviDialog.pack();
		gui.setActiveLabel(gui.manager.getCurrent().getIdentifier());
	    } else if (currentWindow == VIEW_VARIABLES) {
		/*
		  Variables can be deleted always. There are no limitations.
		  Multiple variables can be deleted at once.
		*/
		Object[] removeThese = 
		    objectList.getSelectedValues(); 
		for (int i = 0; i < removeThese.length; ++i) {
		    gui.manager.getCurrent().removeVariable(
				   (Variable)removeThese[i]);
		}
		objectList.setListData(variables);
	    }  else if (currentWindow == VIEW_GLOBAL_VARIABLES) {
		/*
		  As with local variables, global variables can be 
		  deleted without limitations.
		*/
		Object[] removeThese = 
		    objectList.getSelectedValues(); 
		for (int i = 0; i < removeThese.length; ++i) {
		    gui.manager.removeGlobalVariable(
				   (Variable)removeThese[i]);
		}
		objectList.setListData(globalVariables);
	    }
	} else if (cause == applyButton) {
	    /*
	      ApplyButton is an equivalent of okButton, but is used when
	      okButton is reserved for use elsewhere (in other window).
	    */
	    if (currentWindow == VIEW_VARIABLES) {
		/*
		  Variables' value can be changed. It must be checked that
		  the given value is valid float.
		*/
		float valueTemp = 0;
		try {
		    valueTemp = Float.parseFloat(initValue.getText());
		} catch (Exception ex) {
		    initValue.setBackground(Color.red);
		    return;
		}
		Variable temp = (Variable)objectList.getSelectedValue();
		temp.setValue(valueTemp);
		messageDialog.dispose();
		messageDialog = null;
		objectList.setListData(variables);
		taviDialog.pack();
	    } else if (currentWindow == VIEW_TRANSACTIONS) {
		/*
		  Transaction's name cannot be empty. There is no need
		  for other checks.
		*/
		Transaction temp = (Transaction)objectList.getSelectedValue();
		if ((objectNameField.getText()).equals("")) {
		    objectNameField.setBackground(Color.red);
		    return;
		} 
		temp.setName(objectNameField.getText());
		messageDialog.dispose();
		messageDialog = null;
	    } else if (currentWindow == VIEW_GLOBAL_VARIABLES) {
		/*
		  Same functionality as in local variables.
		*/
		float valueTemp = 0;
		try {
		    valueTemp = Float.parseFloat(initValue.getText());
		} catch (Exception ex) {
		    initValue.setBackground(Color.red);
		    return;
		}
		Variable temp = (Variable)objectList.getSelectedValue();
		temp.setValue(valueTemp);
		messageDialog.dispose();
		messageDialog = null;
		objectList.setListData(globalVariables);
		taviDialog.pack();
	    }
	} 
	
	/*scale servers according to new contents*/
	int [] graphMaxXMaxY = gui.graph.scaleServers
			((double)1);
	/*resize canvas if scaling produced border exceedings*/
	if (graphMaxXMaxY[2] < gui.getWidth()) {
	    graphMaxXMaxY[2] = gui.getWidth();
	}
	if (graphMaxXMaxY[3] < gui.getHeight()){ 
	    graphMaxXMaxY[3] = gui.getHeight();
	}
	gui.drawArea.windowResized(new Dimension(graphMaxXMaxY[2], graphMaxXMaxY[3]));
	gui.drawArea.updateCanvas();
    }

    /**
     * Method required in ListSelection listener.
     * @param e Event to be handled.
     */
    public void valueChanged(ListSelectionEvent e) {
	Object cause = e.getSource();
	/*
	  If variables are displayed in the JList object, there are
	  slightly different rules for deleting as with transactions.
	  All variables can be deleted, but one transaction must remain
	  at all times. Delete button is not enabled, when nothing is
	  selected or all transactions are selected. Editing of multiple
	  objects isn't allowed. */
	if (currentWindow == VIEW_VARIABLES || 
	    currentWindow == VIEW_GLOBAL_VARIABLES) {
	    if (objectList.getSelectedValues().length == 0) {
		editButton.setEnabled(false);
		deleteButton.setEnabled(false);
	    } else {
		if (objectList.getSelectedValues().length > 1) {
		    editButton.setEnabled(false);
		    return;
		}
		editButton.setEnabled(true);
		deleteButton.setEnabled(true);
	    }
	    return;
	}

	if (cause == objectList) {
	    /* If nothing has been selected, nothing can be
	       edited or deleted. */
	    if (objectList.getSelectedValue() == null) {
		editButton.setEnabled(false);
		deleteButton.setEnabled(false);
	    } else {
		/* User can delete more than one transaction at a 
		   time, but cannot edit more than one. */
		deleteButton.setEnabled(true);
		if (objectList.getSelectedValues().length == 1) {
		    editButton.setEnabled(true);
		} else {
		    editButton.setEnabled(false);
		}
		/* User cannot delete all transactions. One must remain at 
		   all times. */
		if (objectList.getSelectedValues().length <
		    transactions.size()) {
		    deleteButton.setEnabled(true);
		} else {
		    deleteButton.setEnabled(false);
		}
	    }
	}
    }
}
