/* 
 * @(#)Graph 27.2.2002 version 0.1.6 
 * 
 * Copyright University of Helsinki / Department of computer science 
 * 
 */ 

package tavi.graph;

import java.awt.*;  
import java.util.*;
import java.io.*;

/** 
 * Class for storing the representations of the servers and
 * transaction flows ie. a graph.
 * 
 * Graph is implemented as a linked list where objects in the list are 
 * servers. Therefore this class contains mainly regular list operations
 * such as adding and removing a server. More difficult operations are needed
 * when checking that the object to be added doesn't cause unpleasant 
 * behaviour in the drawing area. Because user can scale server's radius and
 * drawing area size there is a need to check that all objects can still
 * be viewed and edited.   
 * 
 * @version     0.1.6 26 Feb 2002 
 * @author      Tuominen Heikki
 * @author      Ukkonen Johannes
 * @author      Kupiainen Esko
 * @author      Petj Janne
 * @author      Airamaa Kimmo
 */ 
public class Graph implements Serializable {
    
    /**
     * List of vertices in the graph.
     */
    protected LinkedList vertices;  
    /**
     * Half-finished TFlow. Used when constructing client routed TFlow.
     */
    protected TFlow routeFlow;

    /**
     * Indicates whether or not begin state is allready in graph.
     */
    private boolean beginAllready;

    /**
     * Indicates whether or not end state is allready in graph.
     */
    private boolean endAllready;

    /**
     * Indicates the current transaction.
     */
    private Transaction active;

    /**
     * Constructor has nothing to do
     * since the graph is created online.
     */
    public Graph() {
	vertices = new LinkedList();
	beginAllready = false;
	endAllready = false;
    }

    /**
     * Sets the parameter transaction as active transaction and
     * asks each server to change it's list of tflows.
     * @param current Transaction that is to be set active. Cannot
     * be null.
     */
    public void changeTransaction(Transaction current) {
	if (current == null) {
	    return;
	}
	active = current;
	if (vertices.isEmpty()) {
	    return;
	}
	Server s;
	ListIterator i;
	
	/* Updates all vertices excluding the last vertex. */
	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	     s != vertices.getLast(); s = (Server) i.next()) {
 
		s.changeTransaction(active);
	}

	/* It's necessary to update last vertex separately. */
	s.changeTransaction(active);
    }
    
    /**
     * Returns the active transaction.
     * @return Currently active transaction.
     */
    public Transaction getActive() {
	return active;
    }

    /**
     * Method addServer adds a new vertex to the graph.
     * @param s Server to be added.
     */
    public void addServer(Server s) {
	vertices.add(s);
    }
    
    /**
     * Returns the i:th server of the LinkedList vertices,
     * where i is supplied as a parameter.
     * @param i The index of the desired server.
     * @return The requested server. Null if none found.
     */
    public Server getServer(int i) {
	if (i < 0 || i >= vertices.size()) {
	    return null;
	}
	return (Server) vertices.get(i);
    }

    /** 
     * Set receieved parameter as half-finished TFlow.
     * @param t TFlow.
     */
    public void setRouteFlow(TFlow t) {
	routeFlow = t;
    }

    /**
     * Add new TFlow between servers a and b. Returns null if cannot create 
     * one (i.e. without a starting point).  
     * @param a Source of TFlow.
     * @param b Destination of TFlow.
     * @param p Point of selection.
     * @param id Unique identifier of a tflow.
     * @param ord Amount of tflows already between these two servers (a and b).
     * @return Added TFlow.
     */
    public TFlow addTFlow(Server a, Server b, Point p, int id, int ord) {
	/* TFlows must have valid starting point. */ 
	if (a == null) {
	    return null;
	}
	TFlow t = new TFlow(a, b, p, id, ord, active);
	t.setShowSelectionArea(tavi.Tavi.getShowSelectionArea());
	a.addTFlow(t);
	return t;
    }

    /**
     * Remove a flow.
     * @param target TFlow that is supposed to be removed.
     * @param id Identifier object where removed identifier values are 
     * stored to.
     * @return True if operation successful.
     */
    public boolean removeTFlow(TFlow target, Identifier id) {
	if (vertices.isEmpty()) {
	    return false;
	}
	Server s;
	ListIterator i;
	
	/* Search for TFlow and remove it. */
	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	     s != vertices.getLast(); s = (Server) i.next()) {
	    if (s.removeTFlow(target, id)) {
		return true;
	    }
	}
	if (s.removeTFlow(target, id)) {
	    return true;
	}
	return false;
    }

    /**
     * Remove a server and all flows connected to this server.
     * @param target Server to be removed.
     * @param id Identifier object where removed identifier values 
     * are stored to.
     * @return True if found and removed server. False otherwise.
     */
    public boolean removeServer(Server target, Identifier id) {
	if (vertices.isEmpty()) {
	    return false;
	}
	Server s;
	ListIterator i;
	
	/* Search for TFlow and remove it. */
	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	     s != vertices.getLast(); s = (Server) i.next()) {
	    s.removeConnectedTFlows(target, id);
	}
	s.removeConnectedTFlows(target, id);
	if (vertices.remove(target)) {
	    id.releaseIdentifier(target.getIdentifier());
	    return true;
	}
	return false;
    }

    /**
     * Returns the number of vertices in the Graph
     * @return The number of vertices in the Graph.
     */
    public int verticeCount() {
	return vertices.size();
    }

    /**
     * Clears the LinkedList.
     */
    public void removeAll() {
	vertices.clear();
    }

    /**
     * Returns minimum allowed distance between two servers.
     * @return Minimum allowed distance between two servers.
     */
    private static int getMinSelectionDistance() {
	return Server.getRadius() * 7 / 5;
    }

    /**
     * Checks whether there are objects outside requested boundaries.
     * Returns true if there aren't.
     * @param newWidth Requested width of drawing area.
     * @param newHeight Requested height of drawing area.
     * @return True if objects wouldn't be left outside the new boundaries.
     * False otherwise.
     */
    public boolean isSizeChangeRecommended(int newWidth, int newHeight) {
	/* It's sad that LinkedList does not provide sentinel element.
	   Thus we have to check emptiness separately. */
	if (vertices.isEmpty()) {
	    return true;
	}
	Server s;
	ListIterator i;

	/* Test all vertices excluding the last vertex. */
	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	     s != vertices.getLast(); s = (Server) i.next()) {
	    Point p = s.getLocation();
	    if ((p.x + Server.getRadius()) > newWidth || (p.y + Server.getRadius())
		> newHeight) {
		return false;
	    }
	    if (!s.isSizeChangeOk(newWidth, newHeight)) {
	       return false;
	    }
	}
	Point p = s.getLocation();
	if ((p.x + Server.getRadius()) > newWidth || (p.y + Server.getRadius())
	    > newHeight) {
	    return false;
	}
	if (!s.isSizeChangeOk(newWidth, newHeight)) {
	   return false;
	}

	/* test half-finished routed flow */
	if (routeFlow != null) {
	    int[] xPoints = routeFlow.getXTurnPoints();
	    int[] yPoints = routeFlow.getYTurnPoints();
	    for (int k = 0; k < xPoints.length; ++k) {
		if (xPoints[k] > newWidth || yPoints[k] > newHeight) {
		    return false;
		}
	    }
	}
	return true;
    }

    /**
     * Draws a representation of the graph. Calls drawSelf method of all the
     * vertices in the graph.
     */
    public void drawSelf(Graphics g) {
	/* It's sad that LinkedList does not provide sentinel element.
	   Thus we have to check emptiness separately. */
	if (vertices.isEmpty()) {
	    return;
	}
	Server s;
	ListIterator i;

	/* Draws all vertices excluding the last vertex. */
	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	     s != vertices.getLast(); s = (Server) i.next()) {
	    s.drawSelf(g, active);
 	}

	/* It's necessary to draw last vertex separately. */
	s.drawSelf(g, active);
	
	if (routeFlow != null) {
	    routeFlow.drawSelf(g);
	}
    }

    /**
     * Returns the server closest to the suplied point. Null if no server
     * is closer than specified getMinSelectionDistance().
     * @param p Point supplied.
     * @return Server closest to suplied point. Null if none
     *         closer than getMinSelectionDistance()
     */
    public Server getClosestServer(Point p) {
	return getClosestServer(p, getMinSelectionDistance());
    }

    /**
     * Returns the server closest to the suplied point. Null if no server
     * is closer than specified maximum retValDistance.
     * @param p Point supplied.
     * @param retValDistance Maximum distance to select any TFlow. 
     * @return TFlow closest to suplied point. Null if none
     *         closer than retValDistance.
     */
    public Server getClosestServer(Point p, double retValDistance) {

	/* It's sad that LinkedList does not provide sentinel element.
	   Thus we have to check emptiness separately. */
	if (vertices.isEmpty()) {
	    return null;
	}
	Server retVal = null;
	Server s;
	ListIterator i;

	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	     s != vertices.getLast(); s = (Server) i.next()) {
	    double d = Math.sqrt(
				 Math.pow((p.x - s.getLocation().x), 2) 
				 +
				 Math.pow((p.y - s.getLocation().y), 2)
				 );

	    if(d < retValDistance) {
		retVal = s;
		retValDistance = d;
	    }
	    
	    
	}
	double d = Math.sqrt(
			     Math.pow((p.x - s.getLocation().x), 2) 
			     +
			     Math.pow((p.y - s.getLocation().y), 2)
			     );
	if(d < retValDistance) {
	    retVal = s;
	    retValDistance = d;
	}
	return retVal;
    }
    
    /**
     * Returns the server closest to the suplied point. Null if no server
     * is closer than specified maximum retValDistance. This method is
     * used when it isn't possible to check where point is in higher level in
     * the properties check.
     * (Other getClosestServer could return itself.)
     * @param p Point supplied.
     * @param retValDistance Maximum distance to select any TFlow. 
     * @param server Server that has called this method.
     * @return TFlow closest to suplied point. Null if none
     *         closer than retValDistance.
     */
    public Server getClosestServer(Point p, double retValDistance, Server server) {

	/* It's sad that LinkedList does not provide sentinel element.
	   Thus we have to check emptiness separately. */
	if (vertices.isEmpty()) {
	    return null;
	}
	Server retVal = null;
	Server s;
	ListIterator i;

	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	     s != vertices.getLast(); s = (Server) i.next()) {
	    double d = Math.sqrt(
				 Math.pow((p.x - s.getLocation().x), 2) 
				 +
				 Math.pow((p.y - s.getLocation().y), 2)
				 );

	    if(d < retValDistance && s.getIdentifier() != server.getIdentifier()) {
		retVal = s;
		retValDistance = d;
	    }
	    
	    
	}
	double d = Math.sqrt(
			     Math.pow((p.x - s.getLocation().x), 2) 
			     +
			     Math.pow((p.y - s.getLocation().y), 2)
			     );
	if(d < retValDistance && s.getIdentifier() != server.getIdentifier()) {
	    retVal = s;
	    retValDistance = d;
	}
	return retVal;
    }

    /**
     * Returns the TFlow closest to the suplied point. Null if no TFlow
     * is closer than specified maximum retValDistance.
     * @param p Point supplied.
     * @return TFlow closest to supplied point. Null if none.
     */
    public TFlow getClosestTFlow(Point p) {

	/* It's sad that LinkedList does not provide sentinel element.
	   Thus we have to check emptiness separately. */
	if (vertices.isEmpty()) {
	    return null;
	}
	TFlow retVal = null;
	Server s;
	ListIterator i;

	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	     s != vertices.getLast(); s = (Server) i.next()) {
	    retVal = s.getClickedEdge(p);
	    if (retVal != null) {
		return retVal;
	    }
	    
	}
	retVal = s.getClickedEdge(p);
	if (retVal != null) {
	    return retVal;
	}
	return retVal;
    }
    
    /**
     * Tests if the servers supplied as parameters are connected in the graph.
     * @param a Source of connection.
     * @param b Destination of connection.
     * @return True if servers are connected with TFlow, false otherwise.
     * @deprecated After version 0.3. Checking is now made
     * in edgesTo() in Server class. 
     */
    public boolean areConnected(Server a, Server b) {
	/* It's sad that LinkedList does not provide sentinel element.
	   Thus we have to check emptiness separately. */
	if (vertices.isEmpty()) {
	    return false;
	}
	Server s;
	ListIterator i;
	boolean aFound = false;
	boolean bFound = false;
	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	     s != vertices.getLast(); s = (Server) i.next()) {
	    if (s.equals(a)) {
		aFound = true;
	    }
	    if (s.equals(b)) {
		bFound = true;
	    }
	}
	if (s.equals(a)) {
	    aFound = true;
	}
	if (s.equals(b)) {
	    bFound = true;
	}

	if (!(aFound && bFound)) {
	    return false;
	}

	if (a.isConnected(b) || b.isConnected(a)) {
	    return true;
	}
	return false;
    }

    /**
     * Tests if the point proposed is far enough from existing edges.
     * @param Point p Point to be verified.
     * @return True if given location is away enough from all edges.
     */
    public boolean locationOk(Point p) {
        if (vertices.isEmpty()) {
	    return true;
	}
	Server s;
	ListIterator i;
	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	     s != vertices.getLast(); s = (Server) i.next()) {
	     if (!s.checkLocation(p)) {
	         return false;
	     }	 
	}
	if (s != null) {
	    if (!s.checkLocation(p)) {
	         return false;
	     }	 
	}
	return true;
    }

    /**
     * Tests if the point proposed is far enough from existing edges.
     * Edges excluded where prameter server is either source or destination.
     * @param Point p Point to be verified.
     * @param Server Requesting server.
     * @return True if given location is away enough from all edges.
     */
    public boolean locationOk(Point p, Server requestingServer) {
        if (vertices.isEmpty()) {
	    return true;
	}
	Server s;
	ListIterator i;
	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	     s != vertices.getLast(); s = (Server) i.next()) {
	    if (s != requestingServer) {
	        if (!s.checkLocation(p, requestingServer)) {
	            return false;
	        }
	    }	 
	}
	if (s != null) {
	    if (s != requestingServer) {
	        if (!s.checkLocation(p, requestingServer)) {
	            return false;
	        }
	    }	 
	}
	return true;
    }

    /**
     * Tests if the edge proposed is far enough from existing servers.
     * @param t Edge to be verified.
     * @return True if given edge is away enough from all servers.
     */
     public boolean lineOk(TFlow t) {
        if (vertices.isEmpty()) {
	    return true;
	}
	Server s;
	ListIterator i;
	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	     s != vertices.getLast(); s = (Server) i.next()) {
	     if (!t.isRouteClear(s.getLocation())) {
	         return false;
	     } 
	}
	if (s != null) {
	    if (!t.isRouteClear(s.getLocation())) {
	         return false;
	     }	 
	}
	return true;
    }

    /**
     * Multiplies the coordinates of the vertices by given parameter.
     * Returns graphs minimum and maximum coordinates.
     * @param m Multiplier.
     * @return Graphs minimum x-coordinate index 0, 
     * minimum y-coordinate index 1, maximum x-coordinate index 2,
     * maximum y-coordinate index 3.
     */
     public int[] scaleServers(double m) {
	int [] ret = new int [4];
        if (vertices.isEmpty()) {
	    return ret;
	}
	/*graph's maximum coordinates*/
	int xMax = 0;
	int yMax = 0;

 	/*graph's minimum coordinates. canvas border exceedingwill be
  	 expressed in negative number*/
	int xMin = ((Server)vertices.getFirst()).getLocation().x;
	int yMin = ((Server)vertices.getFirst()).getLocation().y;

	Server s;

	/*memory to hold which servers had minimum coordinates*/
	Server sMinX = null;
	Server sMinY = null;
	ListIterator i;

	/*store for server's min max coordinates.
	index 0 = xmin, index 1 = ymin, index 2 = xmax, index 3 = ymax*/
	int[] minMax = new int[4]; 

	/*scale the server's location coordinates and then
	 find coordinate minimums and maximums on each server*/

	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	    s != vertices.getLast(); s = (Server) i.next()) {
	    if (s != null) {
	        int x = (int)Math.round(m * s.getLocation().getX());
	     	int y = (int)Math.round(m * s.getLocation().getY());
	     	s.setLocation(new Point(x, y));
	    }
	}
	if (s != null) {
	    int x = (int)Math.round(m * s.getLocation().getX());
	    int y = (int)Math.round(m * s.getLocation().getY());
	    s.setLocation(new Point(x, y));
	}

        for (i = vertices.listIterator(0), s = (Server) i.next(); 
	    s != vertices.getLast(); s = (Server) i.next()) {
	    if (s != null) {
		minMax = s.getServerMinMax(m);
	      	/* check if server's x is x minimum */
		if (minMax[0] < xMin) {  
		    xMin = minMax[0];
		    sMinX = s;
		}
		/* check if server's x is x maximum*/
		if (minMax[2] > xMax) {  
		    xMax = minMax[2];
		}

		/* check if server's y is y minimum*/
		if (minMax[1] < yMin) {  
		    yMin = minMax[1];
		    sMinY = s;
		}

		/* check server's y is maximum*/
		if (minMax[3] > yMax) {  
		    yMax = minMax[3];
		}
	    } 
	}
	
	/*last edge handled separately*/
	if (s != null) {
	    minMax = s.getServerMinMax(m);
	    /* check if server's x is x minimum */
		if (minMax[0] < xMin) {  
		    xMin = minMax[0];
		    sMinX = s;
		}
		/* check server's x is x maximum*/
		if (minMax[2] > xMax) {  
		    xMax = minMax[2];
		}

		/* check server's y is y minimum*/
		if (minMax[1] < yMin) {  
		    yMin = minMax[1];
		    sMinY = s;
		}
		/* check server's y is y maximum*/
		if (minMax[3] > yMax) {  
		    yMax = minMax[3];
		}
	}

	/*check unfinished customer routed flow*/
	if (routeFlow != null) {
	    int[] xTp = routeFlow.getXTurnPoints();
	    int[] yTp = routeFlow.getYTurnPoints();
	 	for (int j = 0; j < xTp.length; ++j) {
		    xTp[j] = (int)Math.round(m * (double)xTp[j]);
		    yTp[j] = (int)Math.round(m * (double)yTp[j]);
		    if (xTp[j] < xMin) {
			xMin = xTp[j] - 2 * TFlow.getSelectionAreaRadius();
		    }
		    else if (xTp[j] > xMax) {
			xMax = xTp[j] + 2 * TFlow.getSelectionAreaRadius();
		    }
		    if (yTp[j] < yMin) {
			yMin = yTp[j] - 2 * TFlow.getSelectionAreaRadius();
		     }
		    else if (yTp[j] > yMax) {
			yMax = yTp[j] + 2 * TFlow.getSelectionAreaRadius();
		    }
		}
	    routeFlow.setXTurnPoints(xTp);
	    routeFlow.setYTurnPoints(yTp);
	}

	int shiftX = 0;
	if (xMin < 0) {
	    shiftX = Math.abs(xMin);
	}
	int shiftY = 0;
	if (yMin < 0) {
	    shiftY = Math.abs(yMin);
	}

	/*add shiftX and shiftY to all servers' location coordinates and
	to all routed edge coordinates*/
	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	    s != vertices.getLast(); s = (Server) i.next()) {
	    s.setLocation(new Point (s.getLocation().x + 
	     shiftX, s.getLocation().y + shiftY));
	    if (s.queueLength() > 0) {
		s.calculateQueuePoints();
	    }
	    s.adjustRouted(shiftX, shiftY);
	}
		    

	/*last edge handled separately*/
	s.setLocation(new Point(s.getLocation().x + 
	       shiftX, s.getLocation().y + shiftY));
	if (s.queueLength() > 0) {
	    s.calculateQueuePoints();
	}
	/*shift routed edges according to x/y deviation*/
	s.adjustRouted(shiftX, shiftY);
	  
	ret[0] = xMin + shiftX;
	ret[1] = yMin + shiftY;
	ret[2] = xMax + shiftX;
	ret[3] = yMax + shiftY;
	return ret;	  
    }

    /**
     * Set visible or unvisible all edges' selection areas.
     * @param show True if selection area is to be shown, false otherwise. 
     */
    public void setSelection(boolean show) {
        if (vertices.isEmpty()) {
	    return;
	}
	Server s;
	ListIterator i;
	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	      s != vertices.getLast(); s = (Server) i.next()) {
	    if (s != null) {
	        s.setEdgesVisible(show);
	    } 
	}
	if (s != null) {
	    s.setEdgesVisible(show);
        }	 
    }

    /**
     * Count the max nr of recursive edges within this graph.
     * @return Max number of recursive edges in one direction. 
     */
    public int getRecMax() {
        if (vertices.isEmpty()) {
	    return 0;
	}
	  
	int max = 0;
	Server s;
	ListIterator i;
	for (i = vertices.listIterator(0), s = (Server) i.next(); 
	   s != vertices.getLast(); s = (Server) i.next()) {
	    if (s != null) {
	        if (s.getRecursiveMax() > max) {
		    max = s.getRecursiveMax();
		}
	    } 
	}
	if (s != null) {
	    if (s.getRecursiveMax() > max) {
		max = s.getRecursiveMax();
	    }
	} 
	return max;
    }


    /**
     * Checks if Begin state is allready in graph
     * @return True if Begin state allready in graph, false otherwise.
     */
    public boolean isBeginAllready() {
	return beginAllready;
    }

    /**
     * Checks if End state is allready in graph
     * @return True if End state allready in graph, false otherwise.
     */
    public boolean isEndAllready() {
	return endAllready;
    }

    /**
     * Sets the value of beginAllready field.
     * @param b value to set beginAlready to.
     */
    public void setBeginAllready(boolean b) {
	beginAllready = b;
    }  

    /**
     * Sets the value of endAllready field.
     * @param b value to set endAlready to.
     */
    public void setEndAllready(boolean b) {
	endAllready = b;
    }

}  

