/* 
 * @(#)GraphTest 18.3.2002 version 0.1.2 
 * 
 * Copyright University of Helsinki / Department of computer science 
 * 
 */ 

package tavi.graph;

import junit.framework.*;
import java.awt.*;

/** 
 * JUnit test-class for Graph.java. 
 * 
 * @version     0.1.2 18 Mar 2002 
 * @author      Heikki Tuominen
 * @author      Johannes Ukkonen
 */ 
public class GraphTest extends TestCase {
    /**
     * Graph that includes some servers and flows.
     */
    Graph g; 
 
   /**
     * Some tests call methods that require calculations which are
     * done in the drawSelf method of TFlow. This method does on screen
     * drawing and thus a reference to Graphics object is needed.
     */
    private Graphics testGraphics;

    /**
     * Standard JUnit testfile constructor.
     * @param name Name of the test (should always supply the name of the
     * class as per JUnit doctrine).
     */
    public GraphTest(String name) {
        super(name);
    }

    /**
     * Standard JUnit command line launcher. Allows GraphTest to be 
     * executed independently from command line, type 'java GraphTest'.
     * @param args[] Required in main method. Not used.
     */
    public static void main(String[] args) {
        junit.swingui.TestRunner.run(GraphTest.class);
    }
    /**
     * Automatically executed before every test. Common testcase 
     * initialization should be put here.
     */
    protected void setUp() {
	g = new Graph();
    }
    /**
     * Complement of setUp(). Common testcase zeroeing should be 
     * put here 
     */
    protected void tearDown() {
        ;
    }
    /**
     * Tests method Graph() (constructor). Checks that graph is
     * generated correctly.
     */
    public void testGraph() {
	assertEquals(g.verticeCount(), 0);
    }

    /**
     * Tests getClosestServer method in Graph class.
     */
    public void testGetClosestServer() {
	Server[] sTable = new Server[3];

	for(int i = 0; i < 3; ++i) {
	    sTable[i] = new Server(new Point(5 * i, 5 * i), 0,"ab");
	    g.addServer(sTable[i]);
	}
	assertTrue(g.getClosestServer(new Point(100, 100)) == null);
	assertTrue(!sTable[2].equals(g.getClosestServer(new Point(6, 6))));
	assertTrue(sTable[1].equals(g.getClosestServer(new Point(6, 6))));
	assertTrue(!sTable[0].equals(g.getClosestServer(new Point(6, 6))));    
    }

    /**
     * Tests that the servers really get added to Graph.
     */
    public void testAddServer() {
	Server[] sTable = new Server[3];

	/* Test that added server goes to last in the list of Servers. */
	for(int i = 0; i < 3; ++i) {
	    sTable[i] = new Server(new Point(5 * i, 5 * i), 0, "ab");
	    g.addServer(sTable[i]);
	    assertTrue(sTable[i].equals(g.getServer(i)));
	}
	/* Test that all Servers are in the list. */
	for (int i = 0; i < 3; ++i) {
	    assertTrue(sTable[i].equals(g.getServer(i)));
	}
    }
    
    /**
     * Tests that the servers are correctly indexed in Graph.
     */
    public void testGetServer() {
	assertTrue(g.getServer(0) == null);
	Server[] sTable = new Server[3];
	/* Test that added server goes to last in the list of Servers. */
	for(int i = 0; i < 3; ++i) {
	    sTable[i] = new Server(new Point(5 * i, 5 * i), 0, "ab");
	    g.addServer(sTable[i]);
	}
	/* Test that all Servers are in the list. */
	for (int i = 0; i < 3; ++i) {
	    assertTrue(sTable[i].equals(g.getServer(i)));
	}
	assertTrue(g.getServer(3) == null);
	assertTrue(g.getServer(-1) == null);
    }
    
    /**
     * Tests that the TFlow is correctly added between servers (one or two).
     */
    public void testAddTFlow() {
	Server a = new Server(new Point(100, 100), 0, "as");
	Server b = new Server(new Point(200, 200), 0, "ae");
	g.addServer(a);
	g.addServer(b);
	/* Tests a non-recursive TFlow. */
	TFlow t = g.addTFlow(a, b, new Point(2, 2), 4, 1);
	assertTrue(t.getSource().equals(a));
	assertTrue(t.getDestination().equals(b));
	assertTrue(t.getIdentifier() == 4);
	assertTrue(t.getOrder() == 1);

	/* Tests a recursive TFlow. */
	TFlow t1 = g.addTFlow(a, a, new Point(2, 2), 2, 2);
	assertTrue(t1.getSource().equals(a));
	assertTrue(t1.getDestination().equals(a));
	assertTrue(t1.getIdentifier() == 2);
	assertTrue(t1.getOrder() == 2);
	
	/* Tests an invalid parametred TFlow. */
	TFlow tInv = g.addTFlow(null, b, new Point(2, 2), 2, 2);
	assertTrue(tInv == null);
	
	/* Tests an null-pointed TFlow. */
	TFlow tInv2 = g.addTFlow(a, b, null, 2, 2);
	assertTrue(tInv2.getSource().equals(a));
    }
    
    /**
     * Tests that the TFlow is correctly removed.
     */
    public void testRemoveTFlow() {
	Identifier id = new Identifier();
	Server a = new Server(new Point(100, 100), 0, "as");
	Server b = new Server(new Point(200, 200), 0, "ae");
	g.addServer(a);
	g.addServer(b);
	/* Tests a non-recursive TFlow. */
	TFlow t = g.addTFlow(a, b, new Point(2, 2), 4, 1);
	TFlow t1 = new TFlow(a, b, null, 200, 3, null);
	assertTrue(!g.removeTFlow(t1, id));
	assertTrue(g.removeTFlow(t, id));
        assertTrue(!g.removeTFlow(t, id));
	
    }
    
    /**
     * Tests that the Server is correctly removed.
     */
    public void testRemoveServer() {
	Identifier id = new Identifier();
	Server a = new Server(new Point(100, 100), 0, "as");
	Server b = new Server(new Point(200, 200), 0, "ae");
	Server c = new Server(new Point(52, 55), 2, "fu");
	Server notInGraph = new Server(new Point(0, 0), 0, "hh");
	g.addServer(a);
	g.addServer(b);
	g.addServer(c);
	/* Tests a non-recursive TFlow. */
	TFlow t = g.addTFlow(a, b, new Point(2, 2), 4, 1);
	TFlow t1 = g.addTFlow(a, c, null, 200, 3);
	TFlow t2 = g.addTFlow(a, a, new Point(2, 2), 33, 3);
	assertTrue(!g.removeServer(notInGraph, id));
	assertTrue(g.removeServer(a, id));
	assertTrue(!g.removeTFlow(t1, id));
	assertTrue(!g.removeTFlow(t, id));
        assertTrue(!g.removeTFlow(t2, id));
	assertTrue(!g.removeServer(a, id));	
    }
    
    /**
     * Tests that the method that counts the servers is working correctly.
     */
    public void testVerticeCount() {
	assertTrue(g.verticeCount() == 0);
	Server[] sTable = new Server[3];
	/* Test that added server is counted right. */
	for(int i = 0; i < 3; ++i) {
	    sTable[i] = new Server(new Point(5 * i, 5 * i), 0, "ab");
	    g.addServer(sTable[i]);
	    assertTrue((i+1) == g.verticeCount());
	}
	for(int i = 2; i > -1 ; --i) {
	    g.removeServer(sTable[i], new Identifier());
	    assertTrue(i == g.verticeCount());
	}
    }
   
     /**
     * Tests that removeAll removes all the servers.
     */
    public void testRemoveAll() {
	Server[] sTable = new Server[3];
	/* Some servers are added to the list. */
	for(int i = 0; i < 3; ++i) {
	    sTable[i] = new Server(new Point(5 * i, 5 * i), 0, "ab");
	    g.addServer(sTable[i]);
	}
	assertTrue(3 == g.verticeCount());
	g.removeAll();
        assertTrue(0 == g.verticeCount());
	g.addServer(new Server(new Point(5, 5), 0, "ab"));
	assertTrue(1 == g.verticeCount());
    }
    
    /**
     * Tests method isSizeChangeRecommended.
     */
    public void testIsSizeChangeRecommended() {
	Graph g = new Graph();
	assertTrue(g.isSizeChangeRecommended(1, 1));
	Server[] sTable = new Server[3];	
	/* Some servers are added to the list. */
	for(int i = 0; i < 3; ++i) {
	    sTable[i] = new Server(new Point(100 * i, 100 * i), 0, "ab");
	    g.addServer(sTable[i]);
	    assertTrue(!g.isSizeChangeRecommended(100 * i + 24, 100 * i + 24));
	    assertTrue(g.isSizeChangeRecommended(100 * i + 25, 100 * i + 25));
	}
	assertTrue(g.isSizeChangeRecommended(301, 301));
    }  
    
    /**
     * Tests that the closest TFlow is found.
     */
    public void testGetClosestTFlow() {
	Server a = new Server(new Point(100, 100), 0, "as");
	Server b = new Server(new Point(200, 200), 0, "ae");
	g.addServer(a);
	g.addServer(b);
	/* Tests a non-recursive TFlow. */
	TFlow t = g.addTFlow(a, b, new Point(2, 2), 4, 1);
	t.drawSelf(null);
	assertTrue(g.getClosestTFlow(t.getSelectionCenter()) == t);
	assertTrue(g.getClosestTFlow(new Point(150, 150)) == t);
	assertTrue(!(g.getClosestTFlow(new Point(-20,-20)) == t));
	assertTrue(g.getClosestTFlow(new Point(149, 149)) == t);
	assertTrue(g.getClosestTFlow(new Point(151, 151)) == t);
	TFlow t2 = g.addTFlow(a, b, new Point(10, 10), 3, 3);
	t2.drawSelf(null);
	int t2SelX = t2.getSelectionCenter().x;
	int t2SelY = t2.getSelectionCenter().y;
	assertTrue(g.getClosestTFlow(new Point(t2SelX, t2SelY)) == t2);
	assertTrue(g.getClosestTFlow(new Point(t2SelX - 2, t2SelY - 2)) == t2);
	assertTrue(!(g.getClosestTFlow(new Point(t2SelX - 3, 
						 t2SelY - 3)) == t2));
	assertTrue(!(g.getClosestTFlow(new Point(6, 6)) == t)); 

    }
    /**
     * Tests that the location check is working correctly.
     */
    public void testLocationOk() {
	assertTrue(g.locationOk(new Point(3, 3)));
	Server[] sTable = new Server[3];
	/* Add few servers to graph. */
	for(int i = 0; i < 3; ++i) {
	    assertTrue(g.locationOk(new Point(200 * i, 100)));
	    sTable[i] = new Server(new Point(200 * i, 100), 0, "ab");
	    g.addServer(sTable[i]);
	}
	TFlow t = g.addTFlow(sTable[0], sTable[1], new Point (100, 100), 4, 1);
	TFlow t1 = g.addTFlow(sTable[1], sTable[2], new Point(100, 100), 4, 1);
	/* It seems that values smaller than 25 could asserted true. */
	assertTrue(!g.locationOk(new Point(25, 100)));
	//assertTrue(!g.locationOk(new Point(225, 100)));
	assertTrue(!g.locationOk(new Point(226, 100)));
	assertTrue(!g.locationOk(new Point(100, 100)));
	assertTrue(g.locationOk(new Point(3000, 3000)));
    }
   
    
//     /**
//      * Tests method areConnected().
//      * @deprecated Method is deprecated.
//      */
//     public void testAreConnected() {

// 	Server[] sTable = new Server[5];

// 	/* create test servers and populate graph with them */
// 	for(int i = 0; i < 5; ++i) {
// 	    sTable[i] = new Server(new Point(5 * i, 5 * i), 0, "ad");
// 	    g.addServer(sTable[i]);
// 	}	
// 	/* initially, none of the servers should be connected */
// 	for(int i = 0; i < 5; ++i) {
// 	    for(int j = 0; j < 5; ++j) {
// 		assertTrue(!g.areConnected(sTable[i], sTable[j]));
// 	    }
// 	}
// 	/* connect two servers and assert that they, and only they are
// 	   connected */
// 	g.addTFlow(sTable[2], sTable[3], new Point(), 1, 345);
// 	assertTrue(g.areConnected(sTable[2], sTable[3]));
// 	assertTrue(g.areConnected(sTable[3], sTable[2]));
// 	for(int i = 0; i < 5; ++i) {
// 	    for(int j = 0; j < 5; ++j) {
// 		if (!((i == 2 && j == 3) || (i == 3 && j == 2))) { 
// 		    assertTrue(!g.areConnected(sTable[i], sTable[j]));
// 		}
// 	    }
// 	}
// 	/* test recursive connection */
// 	g.addTFlow(sTable[1], sTable[1], new Point(), 1, 67);
// 	assertTrue(g.areConnected(sTable[1], sTable[1]));

// 	/* test first and last in list */
// 	g.addTFlow(sTable[0], sTable[3], new Point(), 1, 79);
// 	assertTrue(g.areConnected(sTable[3], sTable[0]));
// 	g.addTFlow(sTable[4], sTable[3], new Point(),1, 89);
// 	assertTrue(g.areConnected(sTable[4], sTable[3]));
//     }

    /**
     * Tests method lineOK(). To get a reference to
     * a Graphics object (Graphics being an abstract class) it is 
     * necessary to temporarily create an object (in this case, a
     * dummy Frame) from which a valid Graphics object can be 
     * extracted.
     */
    public void testLineOK() {
	/* Fetch a Graphics instance from a temporary frame.*/
	Frame fr = new Frame();
	fr.pack();
	fr.setVisible(true);
	testGraphics = fr.getGraphics();
	fr.dispose();
	fr = null;

	Graph g = new Graph();
	Server a = new Server(new Point(100, 100), 0, "aa");
	Server b = new Server(new Point(250, 75), 0, "ab");
	Server c = new Server(new Point(500, 100), 0, "ac");
	Server d = new Server(new Point(250, 250), 0, "ad");
	Server e = new Server(new Point(500, 500), 0, "ae");
	g.addServer(a);
	g.addServer(b);
	TFlow t = new TFlow(a, b, new Point(99, 99), 1, 1, null);
	a.addTFlow(t);
	g.drawSelf(testGraphics);
	assertTrue(g.lineOk(t));
	g.addServer(c);
	TFlow t1 = new TFlow(a, c, new Point(99, 99), 1, 1, null);
	a.addTFlow(t1);
	g.drawSelf(testGraphics);
	assertTrue(!g.lineOk(t1));
	TFlow t2 = new TFlow(a, b, new Point(99, 99), 1, 2, null);
	a.addTFlow(t2);
	g.drawSelf(testGraphics);
	assertTrue(g.lineOk(t2));
	g.addServer(d);
	g.addServer(e);
	TFlow t3 = new TFlow(a, d, new Point(99, 99), 1, 1, null);
	a.addTFlow(t3);
	g.drawSelf(testGraphics);
	assertTrue(g.lineOk(t3));
	TFlow t4 = new TFlow(d, e, new Point(99, 99), 1, 1, null);
	d.addTFlow(t4);
	g.drawSelf(testGraphics);
	assertTrue(g.lineOk(t4));
	TFlow t5 = new TFlow(a, e, new Point(99, 99), 1, 1, null);
	a.addTFlow(t5);
	g.drawSelf(testGraphics);
	assertTrue(!g.lineOk(t5));
	/* Checks that it isn't possible to draw recursive 
	   arcs that overlap another servers. */
	Server f = new Server(new Point(350, 250), 0, "kk");
	g.addServer(f);
	TFlow t6 = new TFlow(d, d, new Point(265, 250), 1, 1, null);
	d.addTFlow(t6);
	g.drawSelf(testGraphics);
	assertTrue(g.lineOk(t6));
	TFlow t7 = new TFlow(d, d, new Point(265, 250), 1, 2, null);
	d.addTFlow(t7);
	g.drawSelf(testGraphics);
	assertTrue(!g.lineOk(t7));
	TFlow t8 = new TFlow(d, d, new Point(265, 250), 1, 3, null);
	d.addTFlow(t8);
	g.drawSelf(testGraphics);
	assertTrue(!g.lineOk(t8));
	TFlow t9 = new TFlow(d, d, new Point(265, 250), 1, 4, null);
	d.addTFlow(t9);
	g.drawSelf(testGraphics);
	assertTrue(!g.lineOk(t9));
	TFlow t10 = new TFlow(d, d, new Point(265, 250), 1, 5, null);
	d.addTFlow(t10);
	g.drawSelf(testGraphics);
	assertTrue(!g.lineOk(t10));
	Server x = new Server(new Point(100, 500), 0, "ss");
	Server y = new Server(new Point(49, 250), 0, "sf");
	g.addServer(x);
	g.addServer(y);
	TFlow t11 = new TFlow(a, x, new Point(265, 250), 1, 1, null);
	a.addTFlow(t11);
	g.drawSelf(testGraphics);
	assertTrue(g.lineOk(t11));
	TFlow t12 = new TFlow(a, x, new Point(265, 250), 1, 2, null);
	d.addTFlow(t12);
	g.drawSelf(testGraphics);
	assertTrue(g.lineOk(t12));
    }

    /**
     * Tests method isBeginAllready.
     */
    public void testIsBeginAllready(){
	assertTrue(g.isBeginAllready() == false);
	g.setBeginAllready(true);
	assertTrue(g.isBeginAllready() == true);	
    }

    /**
     * Tests method isEndAllready.
     */
    public void testIsEndAllready(){
	assertTrue(g.isEndAllready() == false);
	g.setEndAllready(true);
	assertTrue(g.isEndAllready() == true);
    }

    /**
     * Tests method setBeginAllready.
     */
    public void testSetBeginAllready(){
	assertTrue(g.isBeginAllready() == false);
	g.setBeginAllready(false);
	assertTrue(g.isBeginAllready() == false);
	g.setBeginAllready(true);
	assertTrue(g.isBeginAllready() == true);
	g.setBeginAllready(true);
	assertTrue(g.isBeginAllready() == true);	
	g.setBeginAllready(false);
	assertTrue(g.isBeginAllready() == false);
    }

    /**
     * Tests method setEndAllready.
     */
    public void testSetEndAllready(){
	assertTrue(g.isEndAllready() == false);
	g.setEndAllready(false);
	assertTrue(g.isEndAllready() == false);
	g.setEndAllready(true);
	assertTrue(g.isEndAllready() == true);
	g.setEndAllready(true);
	assertTrue(g.isEndAllready() == true);	
	g.setEndAllready(false);
	assertTrue(g.isEndAllready() == false);
    }

}
