/* 
 * @(#)Identifier 28.2.2002 version 0.1.1 
 * 
 * Copyright University of Helsinki / Department of computer science
 * 
 */ 

package tavi.graph;

import java.util.Stack;

import java.io.*;

/** 
 * Identifier generator for servers and tflows. Get operations return
 * unique two-character String or number, starting from "aa" and 0 and
 * strings go from "aa" to "zz" in lexicographical order, unless there
 * are released identifiers in stack in which case no new identifier is
 * created but instead the topmost identifier in stack is returned. 
 * Does not enforce Identifier's uniqueness on release operations. 
 * I.e. you can say 'releaseIdentifier("aa")' twice in succession and 
 * "aa" is pushed twice to the stack.
 * 
 * @version     0.1.1 28 Feb 2002 
 * @author      Ukkonen Johannes
 */ 
public class Identifier implements Serializable {
    
    /**
     * Integer value of next server identifier.
     */
    private int serverId;

    /**
     * Stack of String server identifiers.
     */
    private Stack serverStack;
    
    /**
     * Integer value of next TFlow identifier.
     */
    private int tFlowId;

    /**
     * Stack of integer TFlow identifiers.
     */
    private Stack tFlowStack;

    /**
     * Integer value of next Transaction identifier.
     */
    private int transactionId;

    /**
     * Stack of String transaction identifiers.
     */
    private Stack transactionStack;


    /**
     * Initialize stacks.
     * Sets identifiers' inital values.
     */
    public Identifier() {
	serverId = 0;
	serverStack = new Stack();
	tFlowId = 0;
	tFlowStack = new Stack();
	transactionId = 1;
	transactionStack = new Stack();
    }

    /**
     * Gives identification String for server.
     * @return Identifier of server.
     */
    public String getIdentifier() {
	if (!serverStack.empty()) {
	    return (String) serverStack.pop();
	} else {
	    String tmp = "";
	    /* Only two characters are allowed. */ 
	    if (serverId < 676) {
		tmp += (char) (serverId / 26 + 'a');
		tmp += (char) (serverId % 26 + 'a');
		++serverId;
		return tmp;
	    } else {
		return "";
	    }
	}
    }
    
    /**
     * Adds deleted server's identifier to stack. Begin and End states'
     * identifiers cannot be inserted into stack. 
     * @param s Server's identifier.
     */
    public void releaseIdentifier(String s) {
	if (s.equals("") || s.equals("B") || s.equals("E")) {
	    return;
	}
	serverStack.push(s);
    }
    
    /**
     * Gives identifier for TFlow.
     * @return Identifier of TFlow.
     */
    public int getNumber() {
	if (!tFlowStack.empty()) {
	     return ((Integer) tFlowStack.pop()).intValue();
	}
	return tFlowId++;
    }
    
    /**
     * Adds deleted TFlow's identifier to stack.
     * @param id TFlow's identifier.
     */
    public void releaseNumber(int id) {
	tFlowStack.push(new Integer(id));
    }
      
    /**
     * Gives identifier for Transaction.
     * @return Identifier of Transaction.
     */
    public String getTransactionId() {
	if (!transactionStack.empty()) {
	     return (String) transactionStack.pop();
	}
	
	return "Tr" + transactionId++;
    }
    
    /**
     * Adds deleted Transaction's identifier to stack. Default id cannot be
     * pushed to stack.
     * @param id Transaction's identifier.
     */
    public void releaseTransactionId(String id) {
	transactionStack.push(id);
    }

}
