/* 
 * @(#)Server 6.5.2002 0.1.8 
 * 
 * Copyright University of Helsinki / Department of computer science
 * 
 */ 

package tavi.graph;

import java.awt.*;
import java.util.*;
import java.io.Serializable;


/** 
 * Class which represents a server. 
 * Knows where it its located and how to draw itself.
 * Server contains two linked lists of it's transaction flows i.e. flows 
 * that have this server as source. There are two linked lists because only 
 * current transaction's tflows are displayed. Those tflows are stored
 * separately.  
 * Therefore there are regular linked list 
 * operations such as adding and removing.
 * <p> 
 * Each server have also name (initialaized as empty), 
 * unique identifier and location. Server is a circle and it is possible to 
 * change it's radius size. Location is circle's center point. 
 * Server can have a queue (initialaized as one)
 * which is drawn as a 
 * polyline. Also there can be one begin server and one end server in the 
 * graph. Those servers have some minor differences compared to "regular" 
 * servers i.e they can't have queue or can't be named.
 * <p>
 * Drawing of a server is implemented in drawSelf() method where it is checked
 * whether this server contains a queue or not. Server is displayed as a 
 * circle in draw area (with a polyline representing a queue on the left 
 * side) and identifier in the middle.
 * Server is drawn everytime drawSelf is called
 * i.e. it's not dependable on transactions.
 * Every time server's drawSelf is called, each server then calls
 * drawSelf() (in Tflow class) to all its tflows in current transaction. 
 * After drawSelf() has finished the graph can be seen in drawing area.  
 *
 * @version     0.1.8 6 May 2002 
 * @author      Tuominen Heikki
 * @author	Kupiainen Esko 
 * @author      Ukkonen Johannes
 * @author      Airamaa Kimmo
 * @author      Petj Janne
 */ 
public class Server implements Serializable {

    /**
     * Server radius used at program start and at load/save functions.
     */
    public static final int INITIAL_RADIUS = 25;
    /* The radius of the circle representing the server. Used
     * when drawing the server.
     */
    protected static int radius = INITIAL_RADIUS;
    /**
     * For drawing the queue.
     */
    protected static final double Q_ANGLE = 0.33 * Math.PI / 2;
    /**
     * The location of the server on the draw area. Actually, the 
     * upper left corner of the rectangle that encloses the circle
     * representing the server (see Graphics.drawCircle() of Java API).
     */
    protected Point location;

    /**
     * Maximum number of non-routed TFlows between two servers.
     * Before changing this number please check that TFlow
     * is able to handle that number of parallel edges.
     */
    public static final int MAX_PARALLEL_TFLOWS_BETWEEN_TWO_SERVERS = 5;

    /**
     * Maximum number of recursive TFlows in one compass direction.
     */
    public static final int MAX_RECURSIVE_TFLOWS_PER_DIRECTION = 10;

    /**
     * Server's name.
     */
    protected String name;
    /**
     * Reserved symbol representing begin state.
     */
    protected static final char BEGIN_SERVER = 'B';
    /**
     * Reserved symbol representing end state.
     */
    protected static final char END_SERVER = 'E';
    /**
     * Short identifier for the server.
     */
    protected String identifier;
    /**
     * Indicates whether or not this server is selected.
     */
    protected boolean selected;
    /**
     * Length of server's queue.
     */
    protected int queueSize;
    /**
     * List of edges leaving this server in current transaction.
     */
    protected LinkedList edges;
    /**
     * List of all edges leaving this server.
     */
    protected LinkedList allEdges;
    /**
     * X-coordinates of the queue.
     */
    protected int[] xPoints = new int[4];
    /**
     * Y-coordinates of the queue.
     */
    protected int[] yPoints = new int[4];
    /**
     * Creates a server object centered on Point p. The length of 
     * the server's queue is also introduced as is server's unique
     * identification id.
     * @param p Location of server.
     * @param length Length of the queue. 
     * @param s 2-digit identifier of server.
     */
    public Server(Point p, int length, String s) {
	location = new Point(p);
	name = "";
	if (length < 0) {
	    length = 0;
	}
	queueSize = length;
	edges = new LinkedList();
	allEdges = new LinkedList();
	identifier = s;
	if (length > 0) {
	    calculateQueuePoints();
	}
    }    

    /**
     * Creates a server that is either Begin or End state.
     * @param p Location of server.
     * @param c Either 'B' (Begin) or 'E' (End)
     */
    public Server(Point p, char c) {
	location = new Point(p);
	name = "";
	queueSize = 0;
	edges = new LinkedList();
	allEdges = new LinkedList();
	identifier = "" + c;
    }
    
    /**
     * Adds new TFlow orginating from this server.
     * @param t TFlow to be added.
     */
    public void addTFlow(TFlow t) {
	edges.add(t);
    }

    /**
     * Number of recursion arcs of this server at given
     * main direction: north, east, south or west.
     * @param dir Main direction.
     * @return Number of recursion arcs.
     */
    public int arcsInThisDirection(int dir) {
	int ret = 0;
	if (!edges.isEmpty()) {
	    TFlow t;
	    ListIterator i;
   
	    for (i = edges.listIterator(0), t = (TFlow) i.next(); 
		 t != edges.getLast() ;
		 t = (TFlow) i.next()) {

		if (t.isRecursion() && t.giveArcDirection() == dir) {
		    ++ret;
		}
	    }

	    /* Last edge handled separately. */
	    if (t.isRecursion() && t.giveArcDirection() == dir) {
		++ret;
	    }
	}
	return ret;
    }

    /**
     * Writes all tflows in current transaction to global tflow list
     * and moves the tflows of the requested transaction to local tflow
     * list.
     * @param requestedTransaction Reference to the transaction whose tflows 
     * are to be selected.
     * @return True if operation succesful, false otherwise.
     */
    public boolean changeTransaction(Transaction requestedTransaction) {
	Stack flowStack = new Stack();
	allEdges.addAll(edges);
	edges.clear();
	if (allEdges.isEmpty()) {
	    return true;
	}
	TFlow t;
	ListIterator i;
	for (i = allEdges.listIterator(0), t = (TFlow) i.next(); 
	     t != allEdges.getLast(); t = (TFlow) i.next()) {
	    if (t.getTransaction() == requestedTransaction) {
		flowStack.push(t);
		edges.add(t);
	    }
	}
	if (t.getTransaction() == requestedTransaction) {
	    flowStack.push(t);
	    edges.add(t);
	}
	while (!flowStack.isEmpty()) {
	    allEdges.remove(flowStack.pop());
	}
	return true;
    }

    /**
     * Returns info if the given point is away at least minimum distance 
     * from all edges of this server.
     * @param Point p.
     * @return True if minimum distance is guaranteed on all edges.
     */
    public boolean checkLocation (Point p) {
	if (!edges.isEmpty()) {
	    TFlow t;
	    ListIterator i;
	    for (i = edges.listIterator(0), t = (TFlow) i.next(); 
		 t != edges.getLast(); t = (TFlow) i.next()) {
		
		if (!t.isRouteClear(p)) {
		    return false;
		}
	    }
	    /* Last edge handled separately. */
	    if (!t.isRouteClear(p)) {
		return false;
	    }
	}
	return true;
    }	

     /**
     * Returns info if the given point is away at least minimum distance 
     * from all edges of this server excluding edges where parameter
     * server is either source or destination. This method is usable when
     * an existing server would like to check permission before moving to 
     * another location.
     * @param Point p.
     * @param Server requestingServer
     * @return True if minimum distance is guaranteed on all edges.
     */
    public boolean checkLocation (Point p, Server requestingServer) {
	Point rSl = requestingServer.getLocation();
	if (!edges.isEmpty()) {
	    TFlow t;
	    ListIterator i;
	    

	    /*set requesting servers location to the point to be verified*/
	    requestingServer.setLocation(p);
	    for (i = edges.listIterator(0), t = (TFlow) i.next(); 
		 t != edges.getLast(); t = (TFlow) i.next()) {

	        /*make sure that edge has valid selection center point*/
		t.drawSelf(null);
		 
		if (!t.isRouteClear(p)) {
 		    requestingServer.setLocation(rSl);	
		    return false;
		}
    	    }
	    /* Last edge handled separately. */
	    t.drawSelf(null);
	    if (!t.isRouteClear(p)) {
	        requestingServer.setLocation(rSl);
	        return false;
	    }
  	}
        requestingServer.setLocation(rSl);
	return true;
    }  	

    /**
     * Checks whether there are some flows that would be left outside
     * the requested boundaries.
     * @param newWidth Requested width of drawing area.
     * @param newHeight Requested height of drawing area.
     * @return True if no transaction flows would be left outside the
     * requested boundaries. False otherwise.
     */
    public boolean isSizeChangeOk(int newWidth, int newHeight) {
	/* Checks all flows' locations. */
	/* It's sad that LinkedList does not provide sentinel element.
	   Thus we have to check emptiness separately. */
	if (edges.isEmpty()) {
	    return true;
	}
	TFlow t;
	ListIterator i;
	for (i = edges.listIterator(0), t = (TFlow) i.next(); 
	     t != edges.getLast(); t = (TFlow) i.next()) {
	    if (t.isRecursion()) {
		Point p = t.getSelectionCenter();
		if ((p.x + 5) > newWidth || (p.y + 5) > newHeight) {
		    return false;
		}
	    } else if (t.isRouted()) {
		int[] xPoints = t.getXTurnPoints();
		int[] yPoints = t.getYTurnPoints();
		for (int k = 0; k < xPoints.length; ++k) {
		    if (xPoints[k] > newWidth || yPoints[k] > newHeight) {
			return false;
		    }
		}
	    }
	}
	/* It's necessary to check last vertex separately. */
	if (t.isRecursion()) {
	    Point p = t.getSelectionCenter();
	    if ((p.x + 5) > newWidth || (p.y + 5) > newHeight) {
		return false;
	    }
	} else if (t.isRouted()) {
	    int[] xPoints = t.getXTurnPoints();
	    int[] yPoints = t.getYTurnPoints();
	    for (int j = 0; j < xPoints.length; ++j) {
		if (xPoints[j] > newWidth || yPoints[j] > newHeight) {
		    return false;
		}
	    }
	}	
	return true;
    }

     /**
     * Draw a server centered on location.
     * @param g Standard Graphics object required for painting.
     * @param current Currently active transaction.
     */
    public void drawSelf(Graphics g, Transaction current) {
	if (selected) {
	    g.setColor(Color.red);
	} else {
	    g.setColor(Color.black);
	}
	g.drawOval(location.x - radius, location.y - radius, 
		   radius * 2, radius * 2);
	g.setFont(new Font(null, Font.BOLD, 15));
	if (identifier.length() >= 2) {
	    g.drawString(identifier, location.x - 8, location.y + 5);
	} else {
	    g.drawString(identifier, location.x - 4, location.y + 5);
	}

	if (queueSize > 0) {
	    calculateQueuePoints();
	    /* Draw queue to left side of server.*/
	    g.drawPolyline(xPoints, yPoints, xPoints.length);
	    g.drawRect(xPoints[2] + (2 * radius / 9),
		       yPoints[2],
		       2 * radius / 9,
		       yPoints[1] - yPoints[2]);
	}

	/* Draw all TFlows leaving this Server. */
	/* It's sad that LinkedList does not provide sentinel element.
	   Thus we have to check emptiness separately. */
	if (edges.isEmpty()) {
	    return;
	}
	TFlow t;
	ListIterator i;
	for (i = edges.listIterator(0), t = (TFlow) i.next(); 
	     t != edges.getLast(); t = (TFlow) i.next()) {
	    if (t.getTransaction() == current) {
		t.drawSelf(g);
	    }
	}
	/* It's necessary to draw last vertex separately. */
	if (t.getTransaction() == current) {
	    t.drawSelf(g);
	}
    }

    /**
     * Calculates the corner points of server's queue.
     */
    public void calculateQueuePoints() {
	    /* Determine the points used to draw queue.*/
	    xPoints[0] = location.x -
		(int)(Math.cos(Q_ANGLE) * radius);
	    xPoints[3] = xPoints[0];
	    xPoints[1] = xPoints[0] - (int) (0.66 * radius);
	    xPoints[2] = xPoints[1];

	    yPoints[0] = location.y + 
		(int)(Math.sin(Q_ANGLE) * radius);
	    yPoints[1] = yPoints[0];

	    yPoints[2] = location.y - 
		(int)(Math.sin(Q_ANGLE) * radius);
	    yPoints[3] = yPoints[2];	
    }

    /**
     * Calculates number of non-routed edges whose source is this 
     * server and destination is the server supplied as parameter.
     * @param s The destination server.
     * @return int The number of edges between this server and given server.
     */
    public int edgesTo(Server s) {
	int ret = 0;
	if (!edges.isEmpty()) {
	    TFlow t;
	    ListIterator i;
   
	    for (i = edges.listIterator(0), t = (TFlow) i.next(); 
		 t != edges.getLast() ;
		 t = (TFlow) i.next()) {

		if (!t.isRecursion() && !t.isRouted() &&
		    t.getDestination().equals(s)) {
		    ++ret;
		}
	    }

	    /* Last edge handled separately. */
	    if (!t.isRecursion() && !t.isRouted() &&
		t.getDestination().equals(s)) {
		++ret;
	    }
	}
	return ret;
    }

    /**
     * Returns the identifier of the server.
     * @return Identifier
     */
    public String getIdentifier() {
	return identifier;
    }

    /**
     * Return the location of this server.
     * @return Point representing the location of this server.
     */
    public Point getLocation() {
	return location;
    }

    /**
     * Sets the location of this server.
     * Shifts customer routed edges' coordinates according to server shift.
     * @param Point representing the location of this server.
     */
    public void setLocation(Point p) {
        location = p;
    }

    /**
     * Returns the 'long' name of the server.
     * @return Server's full name.
     */
    public String getName() {
	return name;
    }

    /**
     * Returns a TFlow, if there is one with selection point
     * near the parameter location, null otherwise.
     * @param point Point of mouse click.
     * @return TFlow near clicked position or null.
     */
    public TFlow getClickedEdge(Point point) {
	if (edges.isEmpty()) {
	    return null;
	}
	TFlow t;
	ListIterator i;
	Point temp;
	
	for (i = edges.listIterator(0), t = (TFlow) i.next(); 
	     t != edges.getLast(); t = (TFlow) i.next()) {
	    temp = t.getSelectionCenter();
	    if (temp == null) {
		break;
	    }
	    
	    if (t.isInsideSelArea(point)) {
		return t;
	    }
	}
	temp = t.getSelectionCenter();
	if (temp == null) {
	    return null;
	}
	
	if (t.isInsideSelArea(point)) {
	    return t;
	}
	return null;
    }

    /**
     * Returns true if server is Begin server.
     * @return True if server is begin server, false otherwise.
     */
    public boolean isBegin() {
	return identifier.equals("B");
    }

    /**
     * Returns info whether there exists a TFlow-edge already
     * between this server and given server.
     * @param d destination.
     * @return True if connection exists.
     * @deprecated After version 0.3. Checking is now made
     * in edgesTo().
     */
    public boolean isConnected(Server d) {
	if (!edges.isEmpty()) {
	    TFlow t;
	    ListIterator i;
	    for (i = edges.listIterator(0), t = (TFlow) i.next(); 
		 t != edges.getLast(); t = (TFlow) i.next()) {
		if (t.getDestination().equals(d)) {
		    return true;
		}
	    }
	    /* Last edge handled separately. */
	    if (t.getDestination().equals(d)) {
		return true;
	    }
	}
	if (!d.edges.isEmpty()) {
	    TFlow t;
	    ListIterator i;
	    for (i = d.edges.listIterator(0), t = (TFlow) i.next(); 
		 t != d.edges.getLast(); t = (TFlow) i.next()) {
		if (t.getDestination().equals(this)) {
		    return true;
		}
	    }
	    /* Last edge handled separately */
	    if (t.getDestination().equals(this)) {
		return true;
	    }
	}
	return false;
    }

    /**
     * Returns true if server is End server.
     * @return True if server is End server, false otherwise.
     */
    public boolean isEnd() {
	return identifier.equals("E");
    }

    /**
     * Returns the leftmost x-coordinate of this server.
     * @return Leftmost x-coordinate of this server.
     */
    public int leftMostX() {
	return xPoints[1];
    }

    /**
     * Returns the downmost queue y-coordinate of this server.
     * @return Downmost queue y-coordinate of this server.
     */
    public int qdownMostY() {
	return yPoints[1];
    }
    
    /**
     * Returns the length of the queue of this server.
     * @return Length of the queue.
     */
    public int queueLength(){
	return queueSize;
    }
    
    /**
     * Returns the queue-ring connection x-coordinate of this server.
     * @return queue-ring connection x-coordinate of this server.
     */
    public int queRingConnectionX() {
	return xPoints[0];
    }

    /**
     * Returns the upmost queue y-coordinate of this server.
     * @return Upmost queue y-coordinate of this server.
     */
    public int qupMostY() {
	return yPoints[2];
    }

    /**
     * Number of recursion arcs of this server at parameters
     * main direction before given recursion arc. If arc is null, 
     * zero is returned.
     * @param arc to stop counting.
     * @return Number of recursion arcs.
     */
    public int recArcsBefore(TFlow arc) {
	int ret = 0;
	if (!edges.isEmpty() && arc != null) {
	    TFlow t;
	    ListIterator i;
	    
	    for (i = edges.listIterator(0), t = (TFlow) i.next(); 
		 t != edges.getLast() && !t.equals(arc);
		 t = (TFlow) i.next()) {
		
		if (t.isRecursion() && t.giveArcDirection() == 
		    arc.giveArcDirection()) {
		    ++ret;
		}
	    }
	}
	return ret;
    }

    /**
     * Search for TFlow and remove if found. Release TFlow's identifier 
     * i.e. put the identifer back to the Identifier object. If 
     * parameter TFlow is null false will be returned.
     * @param target TFlow to be removed. 
     * @param id Identifier to store removed TFlow's identifier value to.
     * @return True if found and removed TFlow. False otherwise.
     */
    public boolean removeTFlow(TFlow target, Identifier id) {
	if (target == null) {
	    return false;
	}
	/* Search for TFlow and remove if found. */
	if (edges.remove(target)) {
	    id.releaseNumber(target.getIdentifier());
	    renumberEdges(this, target.getDestination()); 
	    return true;
	}
	return false;
    }

    /**
     * Swaps parameter edge's source and destination server references.
     * Removes parameter edge from source server's linked list and
     * adds it to destination server's linked list. Operation does not
     * check that the diagram is valid after changes. It should be
     * checked elsewhere. Method will return false, if given parameter
     * is null.
     * @param t TFlow whose source and destination references are 
     * to be swapped.
     * @return True if operation was done successfully, false otherwise.
     */
    public boolean swapOwners(TFlow t) {
	if (t == null) {
	    return false;
	}
	Server d = t.getDestination();
	t.setSource(d);
	t.setDestination(this);
        boolean removed = edges.remove(t);
	d.addTFlow(t);
        return removed;
    }

    /**
     * Search for TFlows connected to target server and remove them.
     * Release all identifiers of removed TFlows.
     * @param target Server whose TFlows are to be removed.
     * @param id Identifier object where removed identifiers are stored to.
     */
    public void removeConnectedTFlows(Server target, Identifier id) {
	Stack removedElements = new Stack();

	/* Find and remove all Tflows connected to target server. */
	/* It's sad that LinkedList does not provide sentinel element.
	   Thus we have to check emptiness separately. */
	if (edges.isEmpty()) {
	    return;
	}
	TFlow t;
	ListIterator i;
	for (i = edges.listIterator(0), t = (TFlow) i.next(); 
	     t != edges.getLast(); t = (TFlow) i.next()) {
	    if (t.getSource() == target || t.getDestination() == target) {
		removedElements.push(t);
	    }
	}
	if (t.getSource() == target || t.getDestination() == target) {
	    removedElements.push(t);
	}
	while (!removedElements.empty()) {
	    TFlow flow = (TFlow) removedElements.pop();
	    removeTFlow(flow, id);
	}
    }

    /**
     * Sets new queue length for server.
     * @param qLength Server's new length.
     */
    public void setQueueLength(int qLength) {
	if(qLength < 0) {
	    qLength = 0;
	}
	queueSize = qLength;
    }

    /**
     * Sets the selected field of the server. True if selected, false 
     * otherwise.
     * @param b Boolean which indicates whether the server is selected or not.
     */
    public void setSelected(boolean b) {
	selected = b;
    }

    /**
     * Introduce new name to server.
     * @param newName Server's new name.
     */
    public void setServerName(String newName) {
	if (newName != null) {
	    name = newName;
	}
    }

    /**
     * Returns the radius of the circle representing the server.
     * @return The value oh radius field.
     */
    public static int getRadius() {
	return radius;
    }

    /**
     * Sets the radius of the circle representing the server.
     * @param r The value of radius field.
     */
    public static boolean setRadius(int r) {
	if (r >= 10) {
	    radius = r;
	    return true;
	 }
	return false;
    }
    
    /**
     * Renumbers the edges between parameter servers.
     * No renumbering with null references, with routed edges 
     * nor with recursive edges.
     * @param server1. The other server which connects edges.
     * @param server2. The other server.
     * @return True if renumbering was 
     * done successfully, false otherwise.
     */
    public static boolean renumberEdges(Server server1, 
					Server server2) {
	int order = 1;
	
	/*renumbering avoided with null references */
	if (server1 == null || server2 == null) {
	    return false;
	}
	
	/*renumbering avoided with recursive edges*/
	if (server1 == server2) {
	    return false;
	}
	
	if (!server1.edges.isEmpty()) {
	    TFlow t;
	    ListIterator i;
	    
	    for (i = server1.edges.listIterator(0), t = (TFlow) i.next(); 
		 t != server1.edges.getLast(); t = (TFlow) i.next()) {
		if (t.getDestination().equals(server2)) {
		    /* numerate the edge only if not polyline*/
		    if (!t.isRouted()) {
		        t.setOrder(order);
		        ++order;
		    }
		}
	    }
	    /* Last edge handled separately. */
	    if (t.getDestination().equals(server2)) {
		/* numerate the edge only if not polyline*/
		if (!t.isRouted()) {
		    t.setOrder(order);
		    ++order;
		}
	    }
	}
	if (!server2.edges.isEmpty()) {
	    TFlow t;
	    ListIterator i;
	    for (i = server2.edges.listIterator(0), t = (TFlow) i.next(); 
		 t != server2.edges.getLast(); t = (TFlow) i.next()) {
		if (t.getDestination().equals(server1)) {
		    /* numerate the edge only if not polyline*/
		    if (!t.isRouted()) {
		        t.setOrder(order);
		        ++order;
		    }
		}
	    }
	    /* Last edge handled separately */
	    if (t.getDestination().equals(server1)) {
		/* numerate the edge only if not polyline*/
		if (!t.isRouted()) {
		    t.setOrder(order);
		}    
	    }
	}
	return true;
    }
 
    /**
     * Sets all edges' selection areas visible or unvisible,
     * according to parameter.
     * @param show True if edges' selection area is to be shown, 
     * false otherwise.
     */
    public void setEdgesVisible(boolean show) {
			
	if (!edges.isEmpty()) {
	    TFlow t;
	    ListIterator i;
	    
	    for (i = edges.listIterator(0), t = (TFlow) i.next(); 
		 t != edges.getLast(); t = (TFlow) i.next()) {
	    	if (t != null) {
		   t.setShowSelectionArea(show);
		}
	    }
	    /* Last edge handled separately. */
	    if (t != null) {
		t.setShowSelectionArea(show);
	    }
	}
    }

    /**
     * Counts maximum number of recursive edges in any of main directions.
     * @return Max number of recursive edges in any of the main directions.
     */
    public int getRecursiveMax() {
 	/*index 0 = north, 1 = east, 2 = south, 3 = west*/
	int [] max = new int [4]; 		
	if (!edges.isEmpty()) {
	    TFlow t;
	    ListIterator i;
	    
	    for (i = edges.listIterator(0), t = (TFlow) i.next(); 
		 t != edges.getLast(); t = (TFlow) i.next()) {
	    	if (t != null) {
		    if (t.isRecursion() && 
					t.giveArcDirection() != TFlow.N_DIR) {
		        ++max[t.giveArcDirection()];
		    }
		}
	    }
	    /* Last edge handled separately. */
	    if (t != null) {
	        if (t.isRecursion() && t.giveArcDirection() != TFlow.N_DIR) {
		    ++max[t.giveArcDirection()];
		}
	    }
	    int max1 = Math.max(max[0], max[1]);
	    int max2 = Math.max(max[2], max[3]);
	    return Math.max(max1, max2);
	}
	return 0;
    }


    /**
     * Returns server's minimum/maximum x/y coordinates in an int[4] table.
     * @param m Multiplier for adjusting customer routed edge's coordinates.
     * @return Minimum/maximum x/y -coordinates in int[4] table.
     * [0] xMin, [1] yMin, [2] xMax, [3] yMax.
     */
    public int[] getServerMinMax(double m) {
     	int xMin;
        /*minimum border coordinates*/
	if (queueSize == 0) { 
	    xMin = location.x - radius;
	}
	else {
	    xMin = leftMostX();
	}
	int yMin = location.y - radius;

	/*maximum coordinates at this server*/
	int xMax = location.x + radius;
	int yMax = location.y + radius;
	
	/* space to be substracted from minimum coordinate value and
	added to maximum coordinate value*/
	int space = 0;

	/*check if edges' selection points' coordinates
	  go below or above server's coordinates*/	
	if (!edges.isEmpty()) {
	    TFlow t;
	    ListIterator i;
    
	    for (i = edges.listIterator(0), t = (TFlow) i.next(); 
		 t != edges.getLast(); t = (TFlow) i.next()) {
	    	if (t != null) {
		    /* adjust customer routed edge's coordinates*/
		    if (t.isRouted()) {
			int[] xTp = t.getXTurnPoints();
			int[] yTp = t.getYTurnPoints();
		  	for (int j = 0; j < xTp.length; ++j) {
			    xTp[j] = (int)Math.round(m * (double)xTp[j]);
			    yTp[j] = (int)Math.round(m * (double)yTp[j]);

			    /*compare coordinates with current min/max*/
			    if (xTp[j] < xMin) {
				xMin = xTp[j];
			    }
			    else if (xTp[j] > xMax) {
				xMax = xTp[j];
			    }
			    if (yTp[j] < yMin) {
				yMin = yTp[j];
			    }
			    else if (yTp[j] > yMax) {
				yMax = yTp[j];
			    }
			}
			/*set scaled coordinates back to edge*/
		        t.setXTurnPoints(xTp);
		        t.setYTurnPoints(yTp);
		    }

		    /*make sure that edge's selection area center is valid*/
		    t.drawSelf(null);

		    if (t.getSelectionCenter().x < xMin) {
		        xMin = t.getSelectionCenter().x;
		    }
		    else if (t.getSelectionCenter().x > xMax) {
		        xMax = t.getSelectionCenter().x;
		    }

		    if (t.getSelectionCenter().y < yMin) {
		        yMin = t.getSelectionCenter().y;
		    }	
		    else if (t.getSelectionCenter().y > yMax) {
		        yMax = t.getSelectionCenter().y;
		    }	
		}
	    }

	    /* Last edge handled separately. */
	    if (t != null) {

	        /* adjust customer routed edge's coordinates*/
		if (t.isRouted()) {
		    int[] xTp = t.getXTurnPoints();
		    int[] yTp = t.getYTurnPoints();

		    for (int j = 0; j < xTp.length; ++j) {
			xTp[j] = (int)Math.round(m * (double)xTp[j]);
			yTp[j] = (int)Math.round(m * (double)yTp[j]);
	
			if (xTp[j] < xMin) {
			    xMin = xTp[j];
			}
			else if (xTp[j] > xMax) {
			    xMax = xTp[j];
			}
			if (yTp[j] < yMin) {
			    yMin = yTp[j];
			}
			else if (yTp[j] > yMax) {
			    yMax = yTp[j];
			}
		    }
		    t.setXTurnPoints(xTp);
		    t.setYTurnPoints(yTp);
		}
		/*make sure that edge's selection area center is valid*/
		t.drawSelf(null);
		if (t.getSelectionCenter().x < xMin) {
		    xMin = t.getSelectionCenter().x;
		}
		else if (t.getSelectionCenter().x > xMax) {
		    xMax = t.getSelectionCenter().x;
	  	}
		if (t.getSelectionCenter().y < yMin) {
		    yMin = t.getSelectionCenter().y;
		}	
		else if (t.getSelectionCenter().y > yMax) {
		    yMax = t.getSelectionCenter().y;
	 	}	
	    }
        }

	/*make some space between object and draw area border*/
	space = 2 * TFlow.getSelectionAreaRadius();

	int [] ret = new int[4];
	ret[0] = xMin - space;
	ret[1] = yMin - space;
	ret[2] = xMax + space;
	ret[3] = yMax + space;
	return ret;
    }

    /**
     * Adds given numbers to customer routed edge coordinates.
     * @param xShift Value to be added to routed edge's x-coordinates.
     * @param yShift Value to be added to routed edge's y-coordinates.
     */
    public void adjustRouted(int xShift, int yShift){
        if (!edges.isEmpty()) {
	    TFlow t;
	    ListIterator i;
	    
	    for (i = edges.listIterator(0), t = (TFlow) i.next();  
		t != edges.getLast(); t = (TFlow) i.next()) {
	        if (t != null) {
		    /* adjust customer routed edge's coordinates*/
		    if (t.isRouted()) {
		        int[] xTp = t.getXTurnPoints();
		      	int[] yTp = t.getYTurnPoints();
		        for (int j = 0; j < xTp.length; ++j) {
			    xTp[j] = xTp[j] + xShift;
			    yTp[j] = yTp[j] + yShift;
		    	}
		    }
	    	}
	
	    }
	    /* Last edge handled separately. */
	    if (t != null) {
		/* adjust customer routed edge's coordinates*/
		if (t.isRouted()) {
		    int[] xTp = t.getXTurnPoints();
		    int[] yTp = t.getYTurnPoints();
		    for (int j = 0; j < xTp.length; ++j) {
			xTp[j] = xTp[j] + xShift;
			yTp[j] = yTp[j] + yShift;
		    }
		}
	    }
        }
    }
}
