/* 
 * @(#)TFlow 15.6.2002 1.0 
 * 
 * Copyright University of Helsinki / Department of computer science
 * 
 */ 

package tavi.graph;

import java.awt.*;
import java.util.*;
import java.io.Serializable;

/** 
 * Class which represents a transaction flow (edge of a graph). 
 * It knows where it is located, to which server or servers it is connected, 
 * what is its orientation and how to draw itself. 
 * Three different types of transaction flows are possible:<br>
 * -recursion edge within one server<br> 
 *<dd>		(isRecursion == true)</dd><br>
 *<br>
 * -non-recursive non-routed edge between two servers.<br> 
 *<dd>		(isRecursion == false && isRouted == false)</dd><br>
 *<br>
 * -routed edge between two servers.<br>
 *<dd>		(isRouted == true).</dd><br>
 *<br>
 * In parenthesis are introduced the variables that distinguish
 * a type of an edge from another.
 * Non-recursive flows must have an order number (int order).
 * If order number of the flow is int 1 and the total number of arcs
 * between servers is odd the flow will be drawn as a direct line, otherwise
 * all non-recursive flows will be drawn as arcs.
 * Non recursive flow's source and destination servers must be different.
 * Routed flow is a polyline that user routes by hand.
 * Recursive flow must have the same server as source and as destination.  
 * Recursive flow will be drawn as an arc in one of the main directions,
 * with arrow head. The main directions are: 
 * North, East, South or West of the server. Constructor's parameter Point p
 * indicates the direction.
 * 
 * @version     1.0 15 June 2002 
 * @author      Tuominen Heikki
 * @author	Kupiainen Esko 
 * @author      Ukkonen Johannes
 * @author      Petj Janne
 * @author	Lnk Liisa 
 * @author 	Airamaa Kimmo
 */ 
public class TFlow implements Serializable {

    /**
     * Angle of arrow head in radians.
     */
    protected static final double ARROW_HEAD_ANGLE = 
				Math.toRadians((double)(12));
 
    /**
     * Length of arrow head.
     */
    private int arrowHeadLength; 

    /**
     * Length of recursive edge head.
     */
    private int recArcHeadLength; 
    
    /**
     * The source of the TFlow.
     */
    private Server source;

    /**
     * The destination of the TFlow.
     */
    private Server destination;

    /**
     * x-coordinates of turning points of polyline.
     */
    private int[] xTurnPoints;
    
    /**
     * y-coordinates of turning points of polyline.
     */
    private int[] yTurnPoints;

    /**
     * Public field indicating upward.
     */
    public static final int NORTH = 0;

    /**
     * Public field indicating rightward.
     */
    public static final int EAST = 1;

    /**
     * Public field indicating downward.
     */
    public static final int SOUTH = 2;

    /**
     * Public field indicating leftward.
     */
    public static final int WEST = 3;

    /**
     * Public field indicating no direction.
     */
    public static final int N_DIR = -1; 

    /**
     * Indicator of recursion arc's direction.
     * Directions: north, east, south, west, none.
     */
    private int arcDir = N_DIR;
 
     /**
     * Indicator for telling if this edge is a recursion arc.
     * True when edge's source and destination are the same server.
     */
    private boolean arc = false;

    /**
     * Color for selection area.
     */
    private Color selectionAreaColor = new Color(160, 160, 255);

     /**
     * Color for edge when selected.
     */
    private Color activeColor = Color.red;
 
    /**
     * Color for edge when not selected.
     */
    private Color passiveColor = Color.black;

    /**
     * Radius of the selection area circle.
     */
    private static int selectionAreaRadius;  

    /**
     * Point indicating the center of the selection area.
     */
    private Point selectionCenter;

    /**
     * Indicates whether or not this edge is selected.
     */
    private boolean selected = false;

    /**
     * Indicates whether or not this edge should show its selection point.
     */
    private boolean showSelection = true;

    /**
     * Identifier number of TFlow.
     */
    private int identifier;

    /**
     * Segment circle radius.
     */
    private double segmentRadius;

    /**
     * Segment circle center x-coordinate.
     */
    private double segmentCenterX;

    /**
     * Segment circle y-coordinate.
     */
    private double segmentCenterY;
 
    /**
     * Non-recursive arc's order number.
     */
    private int order; 

    /**
     * Name of the TFLow.
     */
    private String name;

    /**
     * Scheme to model TFlow's load.
     */
    private String scheme;
    
    /**
     * This edge's transcation type.
     */
    private Transaction transaction;
      	
     /**
     * Creates a TFlow object. If begin and end servers are equal
     * and the selection point is in one of the main directions,
     * a recursive edge will be created. In other cases
     * a non-recursive edge will be created.
     * @param begin Beginning the edge.
     * @param end End of the edge.
     * @param p Point of the selection.
     * @param id Edge's identification number.
     * @param ord Edge's order number.
     * @param transact Transaction class of this edge.
     */
    public TFlow(Server begin, Server end, Point p, 
		 int id, int ord, Transaction transact) {
 	source = begin;
	destination = end;
	xTurnPoints = null;
	yTurnPoints = null;
	arcDir = isInMainDirection(source, p);
	
	if (source != null && begin != null) {
	    /* check if recursive edge is requested*/
	    if (source.equals(destination) && arcDir != N_DIR) {
	        arc = true; 
	    }
	    else {
	        arc = false;
	     }
	}
	identifier = id;
	order = ord; 

	name = "";
	scheme = "";
	transaction = transact;
    }    

    /**
     * Creates a TFlow object as non-recursive edge between
     * parameter servers.
     * @param begin Beginning the edge.
     * @param end End of the edge.
     * @param id Edge's identification number.
     * @param ord Edge's order number.
     */
    public TFlow(Server begin, Server end, int id, int ord, Transaction transact) {
	source = begin;
	destination = end;
	xTurnPoints = null;
	yTurnPoints = null;
	identifier = id;
	order = ord;
	name = "";
	scheme = "";
	transaction = transact;
    } 

    /** 
     * Swaps the source and destination of TFlow.
     */
    public void changeDirection() {
	
	/*remove this edge from source server's linked list
	and add it to destination server's linked list*/
	source.swapOwners(this);
	if (xTurnPoints != null) {
	    turnTurnPoints();
	}
    }

    /**
     * Returns the name of the TFlow.
     * @return Name of the TFlow.
     */
    public String getName() {
	return name;
    }

    /**
     * Sets the name of the TFlow.
     * @param newName New name of the TFlow.
     */
    public void setName(String newName) {
	if (newName == null) {
	    return;
	}
	name = newName;
    }

    /**
     * Returns the scheme of the TFlow.
     * @return Scheme of the TFlow.
     */
    public String getScheme() {
	return scheme;
    }

    /**
     * Sets the scheme of the TFlow.
     * @param newScheme New scheme of the TFlow.
     */
    public void setScheme(String newScheme) {
	if (newScheme == null) {
	    return;
	}
	scheme = newScheme;
    }  

    /**
     * Returns the identifier number of this TFlow.
     * @return Identifier of TFlow.
     */
    public int getIdentifier() {
	return identifier;
    }
   
    /**
     * Returns table of x-coordinates of interior turning points of TFlow.
     * @return Table of turnpoints.
     */
    public int[] getXTurnPoints() {
	return xTurnPoints;
    }
    
    /**
     * Returns table of y-coordinates of interior turning points of TFlow.
     * @return Table of turnpoints.
     */
    public int[] getYTurnPoints() {
	return yTurnPoints;
    }

    /**
     * Sets table of x-coordinates of interior turning points of TFlow.
     * @param i Table of turnpoints.
     */
    public void setXTurnPoints(int[] i) {
	xTurnPoints = (int[])i.clone();
    }
    
     /**
     * Sets table of y-coordinates of interior turning points of TFlow.
     * @param i Table of turnpoints.
     */
    public void setYTurnPoints(int[] i) {
	yTurnPoints = (int[])i.clone();
    }

    /**
     * Returns the transaction of this TFlow.
     * @return Transaction of this TFlow.
     */
    public Transaction getTransaction() {
	return transaction;
    }

    /**
     * Sets the transaction of this TFlow.
     * @param Transaction for this TFlow.
     */
    public void setTransaction(Transaction transact) {
	if (transact != null) {
	    transaction = transact;
	}
    }

    /**
     * Turns table of x- and y-coordinates of interior turning points of TFlow.
     */
    public void turnTurnPoints() {
	if (!isRouted()) {
	    return;
	}
	int[] xTemp = new int[xTurnPoints.length];
	int[] yTemp = new int[yTurnPoints.length];
	for (int i = 0; i < xTurnPoints.length; ++i) {
	    xTemp[xTemp.length - i - 1] = xTurnPoints[i];
	    yTemp[yTemp.length - i - 1] = yTurnPoints[i];
	}
	setXTurnPoints(xTemp);
	setYTurnPoints(yTemp);
    }

    /**
     * Returns info whether flow is routed.
     * @return True if flow is routed, false otherwise.
     */
    public boolean isRouted() {
	if (xTurnPoints == null) {
	    return false;
	}
	return true;
    }

    /**
     * Return the source of this TFlow.
     * @return Server The source of this TFlow.
     */
    public Server getSource() {
	return source;
    }


    /**
     * Returns the destination of this TFlow.
     * @return Server The destination of this TFlow.
     */
    public Server getDestination() {
	return destination;
    }


    /**
     * Sets the destination of this TFlow.
     * @param d Destination of this flow.
     */
    public void setDestination(Server d) {
	if (d == null) {
	    return;
	}
	destination = d;
    }


    /**
     * Sets the source of this TFlow.
     * @param Server Source of this flow.
     */
    public void setSource(Server s) {
	source = s;
    }


    /**
     * Returns the segment circle radius.
     * @return Segment circle radius.
     */
    public double getSegmentRadius() {
	return segmentRadius;
    }


    /**
     * Returns the segment circle center x-coordinate.
     * @return Segment circle x-coordinate.
     */
    public double getSegmentCenterX() {
	return segmentCenterX;
    }


    /**
     * Returns the segment circle center y-coordinate.
     * @return Segment circle y-coordinate.
     */
    public double getSegmentCenterY() {
	return segmentCenterY;
    }

    /**
     * Returns the height increment of recursive edge.
     * @return The height increment of recursive edge.
     */
    public static int getHeightIncrement() {
	return Server.getRadius() * 64 / 135;
    }

    /**
     * Returns the distance between the parameter server centers.
     * @param s Server from which the distance is calculated.
     * @param d Server to which the distance is calculated.
     * @return Distance between servers.
     */
    public static int distanceBetween(Server s, Server d) {
	return (int)Math.sqrt(Math.pow((double)(s.getLocation().x  -
	       	d.getLocation().x), 2) + 
		Math.pow((double)(s.getLocation().y - 
 	       	d.getLocation().y), 2));
    }
    

    /**
     * Draw a transaction flow (edge).
     * @param g Standard Graphics object required for painting.
     */
    public void drawSelf(Graphics g) {
	Color showingColor;

	/* select the color for edge*/
	if (selected) {
	    showingColor = activeColor;
	} else {
	    showingColor = passiveColor;
	}

	if (g != null) {
	    g.setColor(showingColor);
	}

	/*calculate variables for this edge*/
	selectionAreaRadius = Server.getRadius() * 4 / 25;
	recArcHeadLength = Server.getRadius() * 2 / 5;
	arrowHeadLength = Server.getRadius() * 3 / 5;
	

	/* In case of drawing a half finished client routed 
  	   TFlow. */
        if (destination == null) {
	    int bX = fitBorder (source, new Point (xTurnPoints[0], 
  	    yTurnPoints[0])).x;
	    int bY = fitBorder (source, new Point (xTurnPoints[0], 
	    yTurnPoints[0])).y;
	    if (g != null) {
		g.drawLine(bX, bY, 
			   xTurnPoints[0], yTurnPoints[0]);
		g.drawPolyline(xTurnPoints, yTurnPoints, xTurnPoints.length);
	    }
	    selectionCenter = new Point(xTurnPoints[0], yTurnPoints[0]);
	    return;          
	}

	/* number of lines between parameter servers*/
	int nrOfLines = source.edgesTo(destination) + 
			  destination.edgesTo(source);

	/* if recursive edge*/
	if (arc) {
	   selectionCenter = drawRecursiveEdge(source, arcDir, g);
	}
	
	/*direct line between two separate servers i.e. line number=1*/
 	else if (!arc && xTurnPoints == null && 
		order == 1 && nrOfLines % 2 != 0)  {
 
	    /* In case of straight line between two servers. */
	    selectionCenter = drawDirect(source, destination, g);
	 }
 	
	/* draw segment arc between given servers*/
	else if (!arc && xTurnPoints == null && 
	    !(nrOfLines % 2 != 0 && order == 1)) {
	
	       selectionCenter = drawArc(source, destination, g, order);
	}

	/* In case of polyline between two servers. */
	else if (xTurnPoints != null) { 
	       selectionCenter = drawMultiLine(source, destination, 
				  xTurnPoints, yTurnPoints, g);
	 }
	else {
	    System.out.println("TFlow: No line to draw.");
	}

	/* draw selection area indicator only upon request*/ 
	if (showSelection && g != null) {
	    /* set selection area color */
	    g.setColor(selectionAreaColor);
	    g.fillOval((int)(selectionCenter.x - selectionAreaRadius), 
	    selectionCenter.y - selectionAreaRadius, 
		   2 * selectionAreaRadius + 1, 2 * selectionAreaRadius + 1);	
	    g.setColor(showingColor);
	    
	    if (identifier >= 0) {
		g.setFont(new Font("Courier", Font.BOLD, 
		   8 * Server.getRadius() / 25));
		int sep = 0;
	    	if (identifier < 10) {
		    sep = selectionAreaRadius / 2;
		}
		g.drawString("" + identifier, 
		   selectionCenter.x - (selectionAreaRadius) + sep, 
			     selectionCenter.y + (selectionAreaRadius / 2 ));
	    }
	}
    }


    /**
     * Returns arrow's and head sides' angles to abscissa in radians 
     * in int[3] table.
     * @param b Starting point of the line
     * @param e Ending point of the line
     * @return Arrow and head side angles to abscissa in radians
     * Values: [0] arrow, [1] head side one, [2] head side two 
     */
    public static double[] calculateAngles(Point b, Point e) {
	double[] angles = new double[3];
	if ((e.x - b.x) != 0) { 
	    /* If not in upright position. */
	    angles[0] = Math.abs(Math.atan((e.getY() - b.getY()) /
				      (e.getX() - b.getX())));
	}
	else {
	    angles[0] = Math.PI / 2;
	}
	angles[1] = angles[0] - Math.toRadians((double) 12); 
	angles[2] = Math.PI - angles[0] - Math.toRadians((double) 12);
	return angles; 
    }


    /**
     * Returns the order nr of non-recursive edge.
     * @return Non-recursive edge's order nr.
     */
    public int getOrder() {
	return order;
    }


    /**
     * Sets the order nr of non-recursive edge.
     * @param o Order nr of this edge among the edges between the servers
     */
    public void setOrder(int o) {
	order = o;
    }


    /**
     * Answers whether given point belongs to one of the principal 
     * directions within the server given. 
     * Values are: north, east, south, west and none.
     * @param s Server that is clicked.
     * @param p Point in the server.
     * @return Direction.
     */
    public static int isInMainDirection(Server s, Point p) {
	/* Check if direction is clear. */
	if (p != null && s != null) {
            if ((p.getX() > (s.getLocation().getX() - Server.getRadius() / 2))
		 && (p.getX() < (s.getLocation().getX() + 
		Server.getRadius() / 2)) && (p.getY() < (s.getLocation().getY()
		- Server.getRadius() / 2))) {
		return NORTH;
            }
            else if ((p.getX() > (s.getLocation().getX() - 
		Server.getRadius() / 2)) && (p.getX() < (s.getLocation().getX()
		      + Server.getRadius() / 2)) && (p.getY() > 
		      (s.getLocation().getY() + Server.getRadius() / 2))) {
                return SOUTH; 
            }
  	    else if ((p.getX() < (s.getLocation().getX() - 
		Server.getRadius() / 2)) && (p.getY() > (s.getLocation().getY()
		      - Server.getRadius() / 2)) && (p.getY() < 
		      (s.getLocation().getY() + Server.getRadius() / 2)))  {
	        return WEST; 
 	    }
	    else if ((p.getX() > (s.getLocation().getX() + 
		Server.getRadius() / 2)) && (p.getY() > (s.getLocation().getY() 
		      - Server.getRadius() / 2)) && (p.getY() < 
		      (s.getLocation().getY() + Server.getRadius() / 2))) {
	        return EAST; /* East = true; */
	    }
	}    
	return N_DIR; /* No main direction was found. */
    }


    /**
     * Returns the main direction of this recursive arc.
     * Main direction values are: NORTH, EAST, SOUTH, WEST, N_DIR
     * @return Direction.
     */
    public int giveArcDirection() {
	return arcDir;
    }

    /**
     * Sets the main direction of this recursive arc.
     * Before invoking this method please check with Server.recArcsBefore()
     * that parameter main direction is capable of taking this arc.
     * Main direction values are: NORTH, EAST, SOUTH, WEST.
     * @param direction Main direction of this recursive arc.
     */
    public void setArcDirection(int direction) {
	if (direction == NORTH || direction == EAST || direction == SOUTH ||
		direction == WEST) {
	    arcDir = direction;
	}
    }


    /**
     * Returns the point indicating the center of the selection area.
     * @return Value of the center of the selection area.
     */
    public Point getSelectionCenter() {
	return selectionCenter;
    }

   
    /**
    * Returns true if this edge is a recursion edge, false otherwise.
    * @return True if this is a recursive edge.
    */
    public boolean isRecursion() {
	return arc;
    }


    /**
     * Sets the selected field of the edge. True if selected, false 
     * otherwise.
     * @param b Indicates whether the edge is selected or not.
     */
    public void setSelected(boolean b) {
	selected = b;
    }


    /**
     * Returns the selected field of the edge. 
     * @return True if this edge is selected, false otherwise.
     */
    public boolean getSelected() {
	return selected;
    }


    /**
     * Returns information whether given point is within the 
     * edge selection area.
     * @param p Point to be checked if it's inside this edge's selection area.
     * @return True if the given point is inside the selection area,
     * false otherwise.
     */
    public boolean isInsideSelArea(Point p) {
	if (Math.sqrt(Math.pow(selectionCenter.x - p.x, 2) + 
	    Math.pow(selectionCenter.y - p.y, 2)) <= selectionAreaRadius) { 
	    return true;
	}
	return false;
    }


    /**
     * Sets the flag that indicates whether or not this edge 
     * should make visible its selection area.
     * @param b If true this edge should show its selection area,
     * if false the selection area will not be shown.
     */
    public void setShowSelectionArea(boolean b) {
	showSelection = b;
    }
    

    /**
     * Returns the flag that indicates whether or not this edge 
     * should make visible its selection area.
     * @return True if this edge if should show its
     * selection area, false otherwise.
     */
    public boolean getShowSelectionArea() {
	return showSelection;
    }


    /**
     * Returns the selection area radius of this edge.
     * @return The selection area radius of this edge.
     */
    public static int getSelectionAreaRadius() {
	return selectionAreaRadius;
    }


    /**
    * Returns the String representation of a direction.
    * @param direction Values are NORTH, EAST, SOUTH, WEST.
    * @return Main direction in String object.
    * Values are: North, East, South, West, N_Dir. 
    */
    public static String directionName(int direction) {
	String[] s = {"North", "East", "South", "West", "N_Dir"};
	if (direction <0 || direction > s.length - 1) {
	    direction = s.length - 1;
	}
	return s[direction];
    }


    /**
    * Returns the coordinates for a line to meet server's border.
    * @param s  Servers whose border is to be referred.
    * @param p  Starting point of the line.
    * @return Point at server's border in the line's direction. 
    */
    public static Point fitBorder(Server s, Point p) {
	/*Eliminate null references*/
	if (s == null || p == null) {
	    return null;
	}
	int sX = s.getLocation().x;
	int sY = s.getLocation().y;
	double[] angle = calculateAngles(p, new Point(sX, sY)); 
	/*x-coordinate that fits the border*/
	int fX = sX; 

	/*y-coordinate that fits the border*/
	int fY = sY; 

 	/* If server is down right. */
	if (sX >= p.x && sY >= p.y ) {
 
	    /*x to ring. */
	    fX-= (int)((double)(Server.getRadius()) * 
	    Math.cos(angle[0]));

 	    /*y to ring. */
	    fY-= (int)((double)(Server.getRadius()) * 
	    Math.sin(angle[0]));

	    /* Stop the line to border when destination 
	    is a queued server. */
	    if (s.queueLength() != 0 && fX < 
		s.queRingConnectionX()) { 
	    	fY = s.qupMostY();

		if (fX == p.x) { 
		/* Eliminate zero divider. */
		} 
		else {  
		    fX = (int)((s.qupMostY() + 
		    p.x * Math.tan(angle[0])  
		    - p.y) / Math.tan(angle[0]));
		    if (fX < s.leftMostX()) {
		        fX = s.leftMostX();
			fY = (int)(s.leftMostX() * 
			Math.tan(angle[0]) - p.x *
			Math.tan(angle[0]) + p.y);
		    }  
		} 
	    }  
        } 

       	/* If server is down left. */
    	else if (sX < p.x && sY > p.y) {
	    /* x to ring. */
	    fX+= (int)((double)(Server.getRadius()) * 
				    Math.cos(angle[0]));

	    /* y to ring. */
	    fY-= (int)((double)(Server.getRadius()) * 
				   Math.sin(angle[0])); 
        } 

    	/* If server is up left. */
    	else if (sX <= p.x && sY <= p.y) {
	    
	    /* End x to ring. */
	    fX+= (int)((double)(Server.getRadius()) * 
				    Math.cos(angle[0]));

  	    /* End y to ring. */
	    fY+= (int)((double)(Server.getRadius()) * 
				    Math.sin(angle[0]));  
	}
    
    	/* Up right. */
    	else {                       
	
 	    /*  x to ring. */
	    fX-= (int)((double)(Server.getRadius()) * 
				    Math.cos(angle[0]));

  	    /*  y to ring. */
	    fY+= (int)((double)(Server.getRadius()) * 
				    Math.sin(angle[0]));  
	
	    /*Checks necessary to stop the line to queue border*/
	    if (s.queueLength() != 0 && fX < 
	        s.queRingConnectionX()) { 
	        fY = s.qdownMostY();
	    	if (p.x == fX) { 
	            /* Eliminate zero divider. */
	    	}
	    	else {  
	            fX = (int)((-s.qdownMostY() + 
					 p.x * Math.tan(angle[0])
					 + p.y) / Math.tan(angle[0]));
		    if (fX < s.leftMostX()) {
		        fX = s.leftMostX();
 		        fY = (int)(-s.leftMostX() * 
					   Math.tan(angle[0]) + p.x *
					   Math.tan(angle[0]) + p.y);
		    }
	        }
   	    } 
        }

    return (new Point(fX, fY));	
    } 

    /**
    * Returns the coordinates for arrow head.
    * @param b  Line starting point
    * @param e  Line ending point
    * @param hL  Head's length
    * @return ret. 
    * Values: ret(0) int[] xPoints, ret(1) int[] yPoints
    */
    public static Vector getHead(Point b, Point e, int hL) {
 	int[] headX = new int[3];
	int[] headY = new int[3];
	headX[0] = e.x;
	headY[0] = e.y;
	double ang[] = calculateAngles(b, e);

	/* Down right. */
	if (e.x >= b.x && e.y >= b.y) {
 
	    /* Head's wide-end coordinates. */
	    headX[1] = (int)Math.round((double)headX[0] - 
				(double)hL * Math.cos(ang[1]));

	    headY[1] = (int)Math.round((double)headY[0] - 
				(double)hL * Math.sin(ang[1]));

	    headX[2] = (int)Math.round((double)headX[0] + 
				(double)hL * Math.cos(ang[2]));

	    headY[2] = (int)Math.round((double)headY[0] - 
				(double)hL * Math.sin(ang[2]));
	} 

	/* Down left. */
	else if (e.x < b.x && e.y > b.y) {

	    headX[1] = (int)Math.round((double)headX[0] + 
				(double)hL * Math.cos(ang[1]));

	    headY[1] = (int)Math.round((double)headY[0] - 
				(double)hL * Math.sin(ang[1]));

	    headX[2] = (int)Math.round((double)headX[0] - 
				(double)hL * Math.cos(ang[2]));

	    headY[2] = (int)Math.round((double)headY[0] - 
				(double)hL * Math.sin(ang[2]));
	}  

	/* Up left. */
	else if (e.x <= b.x && e.y <= b.y) {

	    headX[1] = (int)Math.round((double)headX[0] + (double)hL *
					     Math.cos(ang[1]));

	    headY[1] = (int)Math.round((double)headY[0] + (double)hL * 
					     Math.sin(ang[1]));

	    headX[2] = (int)Math.round((double)headX[0] - (double)hL * 
					     Math.cos(ang[2]));

	    headY[2] = (int)Math.round((double)headY[0] + (double)hL * 
					     Math.sin(ang[2]));
	}

	/* Up right. */
	else {
                      
	    headX[1] = (int)Math.round((double)headX[0] - 
				(double)hL * Math.cos(ang[1]));

	    headY[1] = (int)Math.round((double)headY[0] + 
				(double)hL * Math.sin(ang[1]));

	    headX[2] = (int)Math.round((double)headX[0] + 
				(double)hL * Math.cos(ang[2]));

	    headY[2] = (int)Math.round((double)headY[0] + 
				(double)hL * Math.sin(ang[2]));
 	} 
	Vector ret = new Vector();
	ret.add(headX);
	ret.add(headY);
	return ret;
    }

    /**
    * Returns information whether there is at least minimum distance between
    * given point and this line.
    * @param p  Point whose position against this line will be checked
    * @return True if distance between this edge and point 
    * is greater than minimum. 
    */
    public boolean isRouteClear(Point p) {
 	/* straight line, formula: y - y0 = k(x - x0)
	   i.e. Ax + By + C = 0
	   point's distance to line: d = (A * p.x + B * p.y + C) /
	   (SQR(A^2 + B^2)) */

	/* Selection center of edge */
	Point sc = getSelectionCenter();

	/* no action if this edge is a polyline*/
	if (xTurnPoints != null) {
	    return true;
	}
		
	/* minimum distance required between line and parameter point*/
	int minimumDistance = 2 * Server.getRadius();

	int x0 = source.getLocation().x;
	int y0 = source.getLocation().y;
	int x1 = destination.getLocation().x;
	int y1 = destination.getLocation().y;
	double A = 0;
	int l = p.x - Server.getRadius();
	int m = p.x + Server.getRadius();
	int n = p.y - Server.getRadius();
	int o = p.y + Server.getRadius();

	/* no action if server is connected to this edge */
	if ((x0 > l && x0 < m   &&  y0 > n && y0 < o  ) || 
	    (x1 > l && x1 < m   &&  y1 > n && y1 < o )) {
	    return true;
	}

	/* Checks recursive edge separately */
	if (arc) {
	    return Math.sqrt(Math.pow(sc.x - p.x, 2) +
			     Math.pow(sc.y - p.y, 2)) >
		             (Server.getRadius() +
		             2 * getHeightIncrement());
	}
	    
	/* line to fit the server borders*/
	x0 = fitBorder(source, new Point(x1, y1)).x;
	y0 = fitBorder(source, new Point(x1, y1)).y;
	x1 = fitBorder(destination, new Point(x0, y0)).x;
	y1 = fitBorder(destination, new Point(x0, y0)).y;

	if (x0 == x1) {
	    A = Math.sqrt(Integer.MAX_VALUE);
	}
	else {
	    A = (double)(y0 - y1) / (double)(x0 - x1);
	}
	
	double B = -1;
	double C = - A * x0 + (double)(y0);
	double d = 0; 

	/* point's distance to line w/o limiting the line length*/
	d = (double)(A * p.x + B * p.y + C) / (Math.sqrt(Math.pow(A, 2) + 
	Math.pow(B, 2)));
	if ((int)(Math.abs(d)) > minimumDistance) {
	    return true;
	}
	
	/* if distance is less than limit, check the range that 
           runs within server borders, not further 
	   the equation for a normal is y - y0 = (-1/k)(x - x0) */
	double k = 0;

	/*eliminate zero divider*/
	if (Math.abs(A) < 0.00001) {
	    k = Math.sqrt(Integer.MAX_VALUE);
	}
	else {
	    k = -1 / A; 
	}
	/*normal's and line's crossing point*/
	int x = (int)((double)(-k * p.x + p.y - C) / (double)(A - k));
	
	/*if crossing point's x is out of range, signal true*/
	if (x > Math.max(x0, x1) || x < Math.min(x0, x1)) {
	    return true; 
	}

	/*if line is in upright position and parameter y is above*/
	if (x0 == x1 && p.y < (int)(Math.min(y0, y1))) {
	    double dX = Math.pow(x0 - p.x, 2);
	    double dY = Math.pow(Math.min(y0, y1) - p.y, 2);

	    /* if the parameter point is above at least minimum distance*/
	    if (Math.sqrt(dX + dY) > minimumDistance) {
		return true;
	    }
	}

	/*if line is in upright position and parameter y is below*/
	else if (x0 == x1 && p.y > (int)(Math.max(y0, y1))) {
	    double dX = Math.pow(x0 - p.x, 2);
	    double dY = Math.pow(Math.max(y0, y1) - p.y, 2);

	    /* if the parameter point is below at least minimum distance*/
	    if (Math.sqrt(dX + dY) > minimumDistance) {
		return true;	
	    }
	}
	return false;	
    }

    /**
    * Draws a segment arc between the parameter servers.
    * Arrow head will be added to destination end.
    * Please use method distanceBetween to check the 
    * distance between the parameter servers before invoking
    * this method. The calculation accuracy is not guaranteed
    * over distances of 6000 pixels. 
    * After excecution of this method the class variables
    * segmentRadius, segmentCenterX and segmentCenterY
    * will hold information about the segment circle. 
    * @param c1  Source server of the edge.
    * @param c2  Destination server of the edge.
    * @param g Graphics reference for drawing.
    * @param n  Order nr of the arc. Parity changes convexity.
    * Values 1 ... 5.
    * @return Selection point center of this arc. 
    */

    public Point drawArc(Server c1, Server c2, Graphics g, int n) {
	/*For the mathematics applied in the calculation please
	  see 'Tavi mathematics declaration pages 1, 2, 3, and 4'.*/

	/* source server x coordinate */
	double soX = c1.getLocation().getX();
	
	/* source server y coordinate */
	double soY = c1.getLocation().getY();

  	/* destination server x coordinate */
	double deX = c2.getLocation().getX();
	
	/* destination server y coordinate */
	double deY = c2.getLocation().getY();

	/* radius of server */
	double sR = Server.getRadius();

	/* segment circle radius */
	double cR = 0;

	/* segment circle center x-coordinate */
	double cX = 0;

	/* segment circle center y-coordinate */
	double cY = 0;

	/* Distance between servers*/	
	double kD = Math.sqrt(Math.pow((double)(soX - deX), 2) + 
		Math.pow((double)(soY - deY), 2));

	/* distance between chord and segment arc, i.e. increment*/
	double h = 0;
	double increment = 0;
	int tot = c1.edgesTo(c2) + c2.edgesTo(c1);
	
	if (n == 1) {
	    increment = 0;
	}
	else if (n == 2 && tot % 2 == 0){
	    increment = 0;
	}
	else if (n == 2 && tot % 2 != 0){
	    increment = 1;
	}

	else if (n == 4 && tot % 2 == 0){
	    increment = 2;
	}
	else if (n == 4 && tot % 2 != 0){
	    increment = 3;
	}
	else if (n == 3 && tot % 2 != 0) {
	    increment = 1;
	}
	else if (n == 3 && tot % 2 == 0) {
	    increment = 2;
	}
	else if (n == 5) {
	    increment = 3;
	}
	
	/*set arc segment height*/
	h = (getHeightIncrement() / 2 + 1) * (1 + increment); 
	
	/* square of distance between servers*/
	double k2 = Math.pow((double)(kD), 2);

	/* square of increment*/
	double h2 = Math.pow((double)(h), 2);

	/* segment circle radius */ 	
	cR = 1.4 * (k2 + 4 * h2) / (8 * h);

	/*slope of the normal of line connecting the servers*/
	double slopeNormal = Math.sqrt(Integer.MAX_VALUE);
	if (deY - soY != 0) { /*eliminate zero divider*/
	    slopeNormal = -(double)(deX - soX) / (double)(deY - soY);
	}

	/* normal's angle to abscissa */
	double beta = Math.PI / 2;
	
	if (Math.abs(slopeNormal) < Math.sqrt(Integer.MAX_VALUE)) { 
	    beta = Math.atan(slopeNormal);
	}

	/* selection center point*/
	Point selArea = null; 

	/*middle point of line connecting the servers*/
	double nX = (soX + deX) / 2;
	double nY = (soY + deY) / 2;

	/*set unit coefficient according to convexity direction*/
	int neg = 1;
	if (n % 2 != 0) {
		neg = -1;
	} 
	
	if (slopeNormal > 0) {
	    cX = nX + neg * (cR - h) * Math.cos(beta);
	    cY = nY + neg * (cR - h) * Math.sin(beta);
	}  
	else {
	    cX = nX - neg * (cR - h) * Math.cos(beta);
	    cY = nY - neg * (cR - h) * Math.sin(beta);
	}

	/* set segment circle values as instance variables*/
	segmentRadius = cR;
	segmentCenterX = cX;
	segmentCenterY = cY;
	

	/* source server's and segment arc's crossing point's 
	x-coordinate*/ 
	double crossingX1 = 0;

	/* crossing point's y-coordinate at source end*/
 	double crossingY1 = 0;

	/* destination server's and segment arc's crossing point's 
	x-coordinate*/
	double crossingX2 = 0;
	
  	/* crossing point's y-coordinate at destination end */
	double crossingY2 = 0;
	
	/* two source server points that cross segment circle */
	Vector vSource = getFitPoints(c1);

	/* two destination server points that cross segment circle */
	Vector vDestination = getFitPoints(c2);

 	Point a = (Point)vSource.elementAt(0);
	Point b = (Point)vSource.elementAt(1);
	
	Point c = (Point)vDestination.elementAt(0);
	Point d = (Point)vDestination.elementAt(1);
	
	Point cNew = new Point(0, 0);

	/*memory for rejected meeting points*/
	Point rejectedP1 = new Point(0, 0);
	Point rejectedP2 = new Point(0, 0);
	
	/* up left */
	if (soX >= deX && soY > deY) {
	    if (n % 2 == 0) {

		/* if servers are in almost horizontal position*/
		if (Math.abs(a.y - b.y) > 1) {
		    crossingX1 = getYmin(a, b).x;
	            crossingY1 = getYmin(a, b).y;
		    rejectedP1 = getYmax(a, b);
		}
		else {
		    crossingX1 = getXmin(a, b).x;
	            crossingY1 = getXmin(a, b).y;
		    rejectedP1 = getXmax(a, b);
		}
				
		crossingX2 = getXmax(c, d).x;
	        crossingY2 = getXmax(c, d).y;
		rejectedP2 = getXmin(c, d);
		neg = 1;
	    }
	    else {
		neg = -1;
		/*if servers are in nearly upright position*/
	 	if (Math.abs(a.x - b.x) > 1) {
	            crossingX1 = getXmin(a, b).x;
	            crossingY1 = getXmin(a, b).y;
		    rejectedP1 = getXmax(a, b);
		}
		else {
		    crossingX1 = getYmin(a, b).x;
	            crossingY1 = getYmin(a, b).y;
		    rejectedP1 = getYmax(a, b);
		}

		crossingX2 = getYmax(c, d).x;
	        crossingY2 = getYmax(c, d).y;
		rejectedP2 = getYmin(c, d);
		}
	    }
	/* down left*/
	else if (soX > deX && soY <= deY) {
						
	    if (n % 2 == 0) {
		/*if servers are in nearly upright position*/
		if (Math.abs(a.x - b.x) > 1) { 
		    crossingX1 = getXmin(a, b).x;
		    crossingY1 = getXmin(a, b).y;
	  	    rejectedP1 = getXmax(a, b);
		}
		else {
		    crossingX1 = getYmax(a, b).x;
		    crossingY1 = getYmax(a, b).y;
	  	    rejectedP1 = getYmin(a, b);
		}
		crossingX2 = getYmin(c, d).x;
		crossingY2 = getYmin(c, d).y;
		rejectedP2 = getYmax(c, d);
		neg = -1;
		}
	    else {
		neg = 1;
		/*if servers are in nearly horizontal position*/
		if (Math.abs(a.y - b.y) > 1) {
		    crossingX1 = getYmax(a, b).x;
		    crossingY1 = getYmax(a, b).y;
		    rejectedP1 = getYmin(a, b);
		}
		else {
		    crossingX1 = getXmin(a, b).x;
		    crossingY1 = getXmin(a, b).y;
		    rejectedP1 = getXmax(a, b);
		}

		/* if servers are in nearly upright position*/
		if (Math.abs(c.x - d.x) > 1) {
		    crossingX2 = getXmax(c, d).x;
		    crossingY2 = getXmax(c, d).y;
		    rejectedP2 = getXmin(c, d);
		}
		else {
		    crossingX2 = getYmin(c, d).x;
		    crossingY2 = getYmin(c, d).y;
		    rejectedP2 = getYmax(c, d);
		}
	    }
	}
	/*down right*/
	else if (soX <= deX && soY < deY){
								
	    if (n % 2 == 0) {
		crossingX1 = getXmax(a, b).x;
	        crossingY1 = getXmax(a, b).y;
		rejectedP1 = getXmin(a, b);
		    
		/* if servers in almost horizontal position*/
		if (Math.abs(c.y - d.y) > 1) { 
	  	    crossingX2 = getYmin(c, d).x;
		    crossingY2 = getYmin(c, d).y;
		    rejectedP2 = getYmax(c, d);
		}
		else {
		    crossingX2 = getXmin(c, d).x;
		    crossingY2 = getXmin(c, d).y;
		    rejectedP2 = getXmax(c, d);
		}
		neg = 1;
		}
	    else {
		neg = -1;
		crossingX1 = getYmax(a, b).x;
	        crossingY1 = getYmax(a, b).y;
		rejectedP1 = getYmin(a, b);
		    
		/* if servers in nearly upright position*/
		if (Math.abs(c.x - d.x) > 1) {
	  	    crossingX2 = getXmin(c, d).x;
		    crossingY2 = getXmin(c, d).y;
		    rejectedP2 = getXmax(c, d);
		}
		else {
		    crossingX2 = getYmin(c, d).x;
		    crossingY2 = getYmin(c, d).y;
		    rejectedP2 = getYmax(c, d);
		}
	    }
	}

	/*up right*/
	else if (soX < deX && soY >= deY) {
	    	    
	    if (n % 2 == 0) {
		crossingX1 = getYmin(a, b).x;
	        crossingY1 = getYmin(a, b).y;
		rejectedP1 = getYmax(a, b);
		neg = -1;
		
		if (Math.abs(c.x - d.x) > 1) {
		    crossingX2 = getXmin(c, d).x;
	            crossingY2 = getXmin(c, d).y;
		    rejectedP2 = getXmax(c, d);
		}
		else {
		    crossingX2 = getYmax(c, d).x;
	            crossingY2 = getYmax(c, d).y;
		    rejectedP2 = getYmin(c, d);
		}
	    }
	    else {
		neg = 1;
		/* if servers are in nearly upright position*/
		if (Math.abs(a.x - b.x) > 1) {
		    crossingX1 = getXmax(a, b).x;
	            crossingY1 = getXmax(a, b).y;
		    rejectedP1 = getXmin(a, b);
		}

		else {
		    crossingX1 = getYmin(a, b).x;
	            crossingY1 = getYmin(a, b).y;
		    rejectedP1 = getYmax(a, b);
		}	
			
	        crossingX2 = getYmax(c, d).x;
	        crossingY2 = getYmax(c, d).y;
	   	rejectedP2 = getYmin(c, d);
	    }
	}

	/*prepare selection area center for return*/
	selArea = new Point ((int)Math.round(cX + neg * cR * Math.cos(beta)), 
			 (int)Math.round(cY + neg * cR * Math.sin(beta)));

	/*fit the arc to source server's border*/
	cNew = fitSegment(c1, new Point((int)Math.round(crossingX1), 
		(int)Math.round(crossingY1)), rejectedP1);
	crossingX1 = cNew.x;
	crossingY1 = cNew.y;
	
        /*fit the arc to destination server's border*/
	cNew = fitSegment(c2, new Point((int)Math.round(crossingX2), 
		(int)Math.round(crossingY2)), rejectedP2);
	crossingX2 = cNew.x;
	crossingY2 = cNew.y;
	
 	/*deviation from normal*/
	double halfFi = Math.asin(Math.sqrt(Math.pow(crossingX1 - 
			crossingX2, 2) + 
			Math.pow(crossingY1 - crossingY2, 2)) / (2 * cR));

	double cpX = (crossingX1 + crossingX2) / 2;
	double cpY = (crossingY1 + crossingY2) / 2;

	/*count normal anew*/
	beta = Math.PI / 2;
	/* eliminate zero divider*/
	if (cpX - segmentCenterX != 0) {
	    beta = Math.atan((cpY - segmentCenterY) / 
		(cpX - segmentCenterX));
	}
	
	/* memory to hold previous point coordinates*/
        int oldX = 0;
	int oldY = 0;

	/* angle to start the sweep*/
	double startAngle = beta - halfFi;

	/* angle to end the sweep*/
	double endAngle = beta + halfFi;

	/*phase sweep step*/
	double step = 30 * halfFi / (10 + kD);

	/* variables to hold arc circumference coordinates*/
	double ringX = 0;
	double ringY = 0;
	double ringX1 = 0;
	double ringX2 = 0;
	double ringY1 = 0;
	double ringY2 = 0;

	/*arrow's start x coordinate*/
	int arrowSx = 0;

	/*arrow's start y coordinate*/
	int arrowSy = 0;

	/* find arc's min/max coordinates*/
	double xmin = Math.min(crossingX1, crossingX2);
	xmin = Math.min(xmin, selArea.x);

	double xmax = Math.max(crossingX1, crossingX2);
	xmax = Math.max(xmax, selArea.x);

	double ymin = Math.min(crossingY1, crossingY2);
	ymin = Math.min(ymin, selArea.y);

	double ymax = Math.max(crossingY1, crossingY2);
	ymax = Math.max(ymax, selArea.y);
	
	/*tables to store arc coordinates */
	Vector vX = new Vector(1000, 1000);
	Vector vY = new Vector(1000, 1000);

	/* calculate the arc coordinates by sweeping the angle*/	
	for (double i = startAngle; i <= endAngle; i += step) {
	    
	    ringX1 = cX + cR * Math.cos(i);
	    ringX2 = cX - cR * Math.cos(i);
	    ringY1 = cY + cR * Math.sin(i);
	    ringY2 = cY - cR * Math.sin(i);

	    /*select the coordinates that are within range*/
	    if (ringX1 < xmax + Server.getRadius() / 5 && ringX1 > 
		xmin - Server.getRadius() / 5) {
		ringX = ringX1;	 
	    }
	    else {
		ringX = ringX2;
	    }
		
	    if (ringY1 < ymax + Server.getRadius() / 5 && ringY1 > 
		ymin - Server.getRadius() / 5) {
		ringY =ringY1;
	    }
	    else {
		ringY = ringY2;
	    }		
	
	    /*store the coordinates for later sorting*/
	    double dist1 = Math.sqrt(Math.pow(ringX - soX, 2) + 
				     Math.pow(ringY - soY, 2));
	    double dist2 = Math.sqrt(Math.pow(ringX - deX, 2) + 
				     Math.pow(ringY - deY, 2));
	    boolean c1Q = (ringX > c1.leftMostX() && ringX < 
			   c1.queRingConnectionX() && 
		ringY >c1.qupMostY() && ringY < c1.qdownMostY()) && 
		c1.queueLength() > 0;
	    boolean c2Q = (ringX > c2.leftMostX() && ringX < 
			   c2.queRingConnectionX() && 
		ringY >c2.qupMostY() && ringY < c2.qdownMostY()) && 
		c2.queueLength() > 0;
	    /*check that point to be drawn is not inside 
	      server circumference nor inside queue box*/
	    if (dist1 >= Server.getRadius() && dist2 >= 
			Server.getRadius() && !c1Q && !c2Q) {
	        vX.add(new Double(ringX));
	        vY.add(new Double(ringY));
	    }
 	} 

	/*determine whether to sort by x or by y. sort reference
	is y if arc crosses the segment circle equator*/
	boolean sort= (int) Math.min(crossingY1, crossingY2) < cY  &&
	    (int) Math.max(crossingY1, crossingY2) > cY;

	/* sort the coordinate tables*/
	if (!sort) {
		/*insertion sort by x-coordinate */
	    for (int i = 1; i < vX.size(); ++ i) {
		double storeX = ((Double)vX.elementAt(i)).intValue();
		double storeY = ((Double)vY.elementAt(i)).intValue();
		int j = i;
		while (j > 0 && 
		 ((Double)vX.elementAt(j - 1)).intValue() > storeX) {
		    vX.setElementAt(vX.elementAt(j - 1), j);
		    vY.setElementAt(vY.elementAt(j - 1), j);
		    --j;
		}
		vX.setElementAt(new Double(storeX), j);
		vY.setElementAt(new Double(storeY), j);
	    }
	}
	     /* insertion sort by y-coordinate*/
	else {
	    for (int i = 1; i < vX.size(); ++ i) {
		double storeX = ((Double)vX.elementAt(i)).intValue();
		double storeY = ((Double)vY.elementAt(i)).intValue();
		int j = i;
		while (j > 0 && 
		 ((Double)vY.elementAt(j - 1)).doubleValue() > storeY) {
		    vX.setElementAt(vX.elementAt(j - 1), j);
		    vY.setElementAt(vY.elementAt(j - 1), j);
		    --j;
		}
		vX.setElementAt(new Double(storeX), j);
		vY.setElementAt(new Double(storeY), j);  
	    } 	
	}
	
	/* coordinate values are ready to draw the arc. 
	   Get the first value pair.*/
	double xR = 0;
	double yR = 0;
	if (vX.size() > 0) {
	    xR = ((Double)vX.elementAt(0)).doubleValue();
	    yR = ((Double)vY.elementAt(0)).doubleValue();
	}

	/* end point of head tangent arrow */
	Point aE = new Point((int)Math.round(crossingX2), 
		   (int)Math.round(crossingY2));

	/*fill the possible gap between arc start and server*/
	if ((Math.pow(crossingX1 - xR, 2) + 
    	    Math.pow(crossingY1 -  yR, 2)) <
    	    (Math.pow(crossingX2 - xR, 2) + 
    	    Math.pow(crossingY2 -  yR, 2))) {
    	    oldX = (int)Math.round(crossingX1);
    	    oldY = (int)Math.round(crossingY1);
	}
	else {
     	    oldX = (int)Math.round(crossingX2);
    	    oldY = (int)Math.round(crossingY2);
	    if (vX.size() > 7) {
		/*find head end from segment arc*/
		for (int i = 0; i < vX.size() / 2; ++i) {
		    double xv = ((Double)vX.elementAt(i)).doubleValue();
		    double yv = ((Double)vY.elementAt(i)).doubleValue(); 
		    if ((int)Math.round(Math.sqrt(Math.pow(aE.getX() - xv, 2) + 
			Math.pow(aE.getY() - yv, 2))) >= arrowHeadLength) {
			arrowSx = (int)Math.round(xv);  
			arrowSy = (int)Math.round(yv);
			i = vX.size();
		    }
		}
	    }
	    
	    else if (vX.size() >= 4 && vX.size() <= 7) {
		arrowSx = (int)Math.round(
			    ((Double)vX.elementAt(1)).doubleValue());
		arrowSy = (int)Math.round(
			    ((Double)vY.elementAt(1)).doubleValue()); 
	    }
	    if (arrowSx == (int)Math.round(crossingX1) && 
		arrowSy == (int)Math.round(crossingY1)) {
		arrowSx = 0;
		arrowSy = 0;
	    }
	
	} 
	
	/*draw the arc*/	
	for (int i = 0; i< vX.size(); ++i) {
	    xR = ((Double)vX.elementAt(i)).doubleValue();
	    yR = ((Double)vY.elementAt(i)).doubleValue();
	    if (g != null) {
	        g.drawLine((int)Math.round(xR), (int)Math.round(yR), oldX, oldY);
	    }
	    oldX = (int)Math.round(xR);
	    oldY = (int)Math.round(yR);
	    }

	/*fill possible gap between arc end and server*/
	if ((Math.pow(crossingX1 - xR, 2) + 
    	    Math.pow(crossingY1 - yR, 2)) <
    	    (Math.pow(crossingX2 - xR, 2) + 
    	    Math.pow(crossingY2 - yR, 2)) && g != null) {
	    g.drawLine((int)Math.round(crossingX1), 
		(int)Math.round(crossingY1), (int)Math.round(xR), 
		(int)Math.round(yR));
	}
	else {
	    if (g != null) {
	        g.drawLine((int)Math.round(crossingX2), 
		(int)Math.round(crossingY2), (int)Math.round(xR), 
		(int)Math.round(yR));
	    }
	    if (vX.size() > 7) {
		/*find head end from segment arc*/
		for (int i = vX.size() - 1; i > vX.size() / 2; --i) {
		    double xv = ((Double)vX.elementAt(i)).doubleValue();
		    double yv = ((Double)vY.elementAt(i)).doubleValue(); 
		    if ((int)Math.round(Math.sqrt(Math.pow(aE.getX() - xv, 2) + 
			Math.pow(aE.getY() - yv, 2))) >= arrowHeadLength) {
			arrowSx = (int)Math.round(xv);  
			arrowSy = (int)Math.round(yv);
			i = 0;
		    }
		}
	    }
	    else if (vX.size() >= 4 && vX.size() <= 7) {
		arrowSx = (int)Math.round(
			((Double)vX.elementAt(vX.size() - 2)).doubleValue());
		arrowSy = (int)Math.round(
			((Double)vY.elementAt(vX.size() - 2)).doubleValue()); 
	    }
	    
	}

	/* at short distances start arrow in middle of the arc*/
	if (Math.sqrt(Math.pow(arrowSx - selArea.x, 2) + 
		Math.pow(arrowSy - selArea.y, 2)) >= 
	    Math.sqrt(Math.pow(selArea.x - (int)crossingX2, 2) + 
		Math.pow(selArea.y - (int)crossingY2, 2))) {
	    arrowSx = selArea.x;
	    arrowSy = selArea.y;
	}

	Point aS = new Point(arrowSx, arrowSy);

	/* draw head to destination */
	Vector v = getHead(aS, aE, arrowHeadLength);
	int[] xP = (int[])v.elementAt(0);
	int[] yP = (int[])v.elementAt(1);
	if (g != null) {
	    g.fillPolygon(xP, yP, xP.length);
	}

	return selArea; 
    }
	
   	 
    /**
     * Returns the parameter point that has y coordinate minimum.
     * @param a Point to be compared with second parameter point.
     * @param b Point to be compared with first parameter point.
     * @return The parameter point that has y coordinate minimum. 
     */
    public static Point getYmin(Point a, Point b) {
	if (a.y <= b.y) {
	   return a;
	}
	return b;
    }

    /**
     * Returns the parameter point that has y coordinate maximum.
     * @param a Point to be compared with second parameter point.
     * @param b Point to be compared with first parameter point.
     * @return The parameter point that has y coordinate maximum. 
     */
    public static Point getYmax(Point a, Point b) {
	if (a.y >= b.y) {
	   return a;
	}
	return b;
    }

	
    /**
     * Returns the parameter point that has x coordinate minimum.
     * @param a Point to be compared with second parameter point.
     * @param b Point to be compared with first parameter point.
     * @return The parameter point that has x coordinate minimum. 
     */
    public static Point getXmin(Point a, Point b) {
	if (a.x <= b.x) {
	   return a;
	}
	return b;
    }

    /**
     * Returns the parameter point that has x coordinate maximum.
     * @param a Point to be compared with second parameter point.
     * @param b Point to be compared with first parameter point.
     * @return The parameter point that has x coordinate maximum. 
     */
    public static Point getXmax(Point a, Point b) {
	if (a.x >= b.x) {
	   return a;
	}
	return b;
    }


    /**
     * Returns the two points from circumference of parameter server
     * that cross the segment circle.
     * @param server Server whose two crossing points with segment
     * circle are to be calculated.
     * @return Table that contains the two crossing points.
     * First element = Point a, second element = Point b.  
     */
    public Vector getFitPoints(Server server) {
	/*
	segment circle equation: x^2 + y^2 + A * x + B * y + C = 0
	server circle equation: x^2 + y^2 + A1 * x + B1 * y + C1 = 0 
	For more mathematics applied please see Tavi mathematics
	declaration pages 1, 2, 3 and 4.
	 */

	/* server x coordinate */
	double sX = server.getLocation().getX();
	
	/* server y coordinate */
	double sY = server.getLocation().getY();

  	/* radius of server */
	double sR = Server.getRadius();

	/* get segment circle variables*/ 
	double cR = getSegmentRadius();
	double cX = getSegmentCenterX();
	double cY = getSegmentCenterY();

	double A = -2 * cX;
	double B = -2 * cY;
	double C = Math.pow(cX, 2) + Math.pow(cY, 2) - Math.pow(cR, 2);

	double A1= -2 * sX;
	double B1= -2 * sY;
	double C1= Math.pow(sX, 2) + Math.pow(sY, 2) - Math.pow(sR, 2);

	double v = -(A - A1) / (B - B1); 
	double z = -(C - C1) / (B - B1); 
	
	double s = 1 + Math.pow(v, 2);
	double t = 2 * v * z + v * B + A;
	double u = Math.pow(z, 2) + z * B + C;

	double p = t / s;
	double q = u / s;
	double x1 = -p / 2 + Math.sqrt(Math.pow(p / 2, 2) - q);
	double x2 = -p / 2 - Math.sqrt(Math.pow(p / 2, 2) - q);

	/* prepare the points for return*/
	Point a = new Point((int)Math.round(x1),
			(int)Math.round(v * x1 + z));
	
	Point b = new Point((int)Math.round(x2),
			(int)Math.round(v * x2 + z));

	Vector ret = new Vector(2);
	ret.add(a);
	ret.add(b);
	return ret;
    } 


    /**
     * Draws a recursive edge, with head, to parameter server, 
     * in the given direction as an ellipse. Each new recursive edge in the
     * given direction will have height and width increased
     * automatically according to the previous edges. 
     * @param s Server to have the new recursive edge.
     * @param direction Main direction to which the recursive 
     * arc will be drawn. Values are: NORTH, EAST, SOUTH, WEST.
     * @param g graphics reference for drawing. 
     * @return The center of the selection area of the edge.
     */
    public Point drawRecursiveEdge(Server s, int direction, Graphics g) {
	/*For the mathematics and variables applied in the calculation please
	  see 'Tavi mathematics declaration page 5'.*/

	/* Ellipses's covered angle in degrees for drawing. */
	int arcAngle = 280;

 	/* Ellipse's start angle in degrees for drawing. */
	int startAngle = (arcAngle - 180) / 2;
 
	/* Ellipse's start angle in radians. */
	double gamma = Math.toRadians((double)startAngle);

	/* Ellipse's y shift at open angle. */
	double yE = 0;

	/* Ellipse's drawing height. */
	int arcHeight = Server.getRadius() * 16 / 15;

	/* Ellipse's drawing width. */
	int arcWidth = Server.getRadius() * 4 / 3;

	/* Ellipse's half axes. */
	double heightHalf = arcHeight / 2;
	double widthHalf = arcWidth / 2;

	/* Ellipse's x shift at open angle. */
	double xE = widthHalf * Math.cos(gamma);

 	/* Radius shortened by ellipse. */
	double radShort = Math.sqrt(Math.pow((double)(Server.getRadius()), 2)
				     - Math.pow(xE, 2));

	/* Ellipse start x coordinate. */
	int arcStartX = 0;

	/* Ellipse start y coordinate. */
	int arcStartY = 0;
	

	/* Selection area center point*/
	Point selection = new Point(0, 0);

	/* Arrow start x-coordinate*/
	int arrowSx = 0;

	/* Arrow start y-coordinate */
	int arrowSy = 0;

	/* Number of arcs in this direction before this arc*/
	int nrOfArcs = s.recArcsBefore(this);

	/* Server center point*/
	Point pServer = new Point(s.getLocation().x, s.getLocation().y); 
	
	/* slope of the tanget at ellipse's and server's crossing*/
	double slope = 0; 

	/*tables to hold arrow head coordinates*/
	int[] xTp = new int[3];
	int[] yTp = new int[3];

	/*arc's height increment*/
	int hIncrement = getHeightIncrement();

	/*arc's width increment*/
	int wIncrement = Server.getRadius() * 2 / 9;

	/*Calculate variables for this recursive arc*/
	arcHeight = (int)(arcHeight + nrOfArcs * hIncrement);
	arcWidth = (int)(arcWidth + nrOfArcs * wIncrement);
	heightHalf = arcHeight / 2;
	widthHalf = arcWidth / 2;
	gamma = Math.acos(xE / widthHalf);
	yE = heightHalf * Math.sin(gamma);
	startAngle = (int)Math.toDegrees(gamma);
	arcAngle = 180 + 2 * startAngle;
	double corr = (double)(1) + (double)(1) / (double)(2 + nrOfArcs);
		
	/* eliminate zero divider*/
	if (Math.tan(gamma / corr) != 0) {
	    slope = (-1/Math.tan(gamma / corr)) * heightHalf / widthHalf;
	}
	else {
	    slope = -Math.sqrt(Integer.MAX_VALUE);
	}

	/* in right and left of server the ellipse is rotated 90 degrees.
	therefore tangent lines are perpendicular*/
	if (direction == EAST || direction == WEST) {

	    /*eliminate zero divider*/
	    if (slope != 0) {
	        slope = -1 / slope;
	    }
	    else {
	 	slope = -Math.sqrt(Integer.MAX_VALUE);
	    }
	}
		        
	/* Recursion arc to be drawn above server. */
	if (direction == NORTH) { 
	    arcStartX = (int)(pServer.x - widthHalf);
	
	    /*avoid edge to drop below server circumference*/
	    if (nrOfArcs < 1 ) {
	        arcStartY = pServer.y - (int)(radShort + heightHalf + yE + 1);
	    }
	    else {
	 	arcStartY = pServer.y - (int)(radShort + heightHalf + yE + 2);
	    }

	    xTp[0] = pServer.x + (int)(xE);
	    yTp[0] = pServer.y - (int)radShort;

	    /*find x that has value bigger than x at arrow head point*/
	    arrowSx = pServer.x + 2 * Server.getRadius();
	    
	    /* avoid null reference in case of simulated drawing */
	    if (g != null) {
		g.drawArc(arcStartX, arcStartY, arcWidth, arcHeight, 
			  -startAngle, arcAngle); 
	    }

	    /* set selection area center coordinates*/
  	    selection = new Point(pServer.x, arcStartY);
	}
	
	/*recursion arc to be drawn below server*/
	else if (direction == SOUTH) { 
	    arcStartX = (int)(pServer.x - widthHalf);
	    arcStartY = pServer.y + (int)(radShort - heightHalf + yE);
	    xTp[0] = pServer.x - (int)(xE + 1);
	    yTp[0] = pServer.y + (int)radShort;
	    arrowSx = pServer.x - 2 * Server.getRadius();
	    if (g != null) {
		g.drawArc(arcStartX, arcStartY, arcWidth, arcHeight, 
			  startAngle, -arcAngle);   /* Draw arc down. */
	    }

	    /* set selection area center coordinates*/
	    selection = new Point(pServer.x, 
			 pServer.y + (int)(radShort + yE + heightHalf));
	}

	/*recursion arc to be drawn to left side of server*/
	else if (direction == WEST){             
	    arcStartX = (int)(pServer.x - radShort - yE - 
				 heightHalf - 1);
	    arcStartY = pServer.y - (int)widthHalf;
	    xTp[0] = pServer.x - (int)radShort;
	    yTp[0] = pServer.y - (int)xE;
	    arrowSx = pServer.x - 2 * Server.getRadius();
	    if (g != null) {
		g.drawArc(arcStartX, arcStartY, arcHeight, arcWidth,  
			  -90 + startAngle, - arcAngle);   
		/* Draw arc west. */
	    }

	    /* set selection area center coordinates*/
	    selection = new Point(arcStartX, pServer.y);
	}

	/*recursion arc to be drawn to right side of server*/
        else if (direction == EAST){        
	    arcStartX = pServer.x + (int)(radShort - heightHalf + yE);
	    arcStartY = pServer.y - (int)widthHalf;
	    xTp[0] = pServer.x + (int)radShort;
	    yTp[0] = pServer.y + (int)xE;
	    arrowSx = pServer.x + 2 * Server.getRadius();
    	    
	    if (g != null) {
		g.drawArc(arcStartX, arcStartY, arcHeight, arcWidth, 
			  90 + startAngle, - arcAngle);   /* Draw arc east. */
	    }

	    /* set selection area center coordinates*/
	    selection = new Point((int)(pServer.x + radShort + yE + 
			 heightHalf), pServer.y);
	 }

	/*get arrow head coordinates*/   
	arrowSy = (int)(slope * (arrowSx - xTp[0]) + yTp[0]);
	
	xTp = (int[])(getHead(new Point(arrowSx, arrowSy), 
		  new Point (xTp[0], yTp[0]), recArcHeadLength).
		  elementAt(0));
	yTp = (int[])(getHead(new Point(arrowSx, arrowSy), 
		  new Point (xTp[0], yTp[0]), recArcHeadLength).
		  elementAt(1));

	if (g != null) {
	    /* Draw arc head. */
	    g.fillPolygon(xTp, yTp, xTp.length);
	}
	return selection;
    }

	
    /**
     * Draws a direct line between the parameter servers.
     * An arrow head will be added to the destination end.
     * Please use method isRouteClear to check that there
     * are no obstacles in the way before making call to this method.
     * @param s Source server of the line.
     * @param d Destination server of the line.
     * @param g Graphics reference for drawing. 
     * @return The center of the selection area of the line.
     */
    public Point drawDirect(Server s, Server d, Graphics g) {
	
	/* start server x coordinate*/
	int sX = s.getLocation().x;

	/* start server y coordinate*/
	int sY = s.getLocation().y;

	/* end server x coordinate*/
	int eX = d.getLocation().x;

	/* end server y coordinate*/
	int eY = d.getLocation().y;

	/* arrow starting x coordinate*/
	int arrowSx = 0;

	/* arrow starting y coordinate*/
	int arrowSy = 0;

	/*table for arrow head x coordinates*/
	int[] xTp = new int [3];
	
	/*table for arrow head x coordinates*/
	int[] yTp = new int[3];

	/*selection area center point*/
	Point p = new Point(0, 0);

	/*fit the ends to servers' borders*/
	arrowSx = fitBorder(source, new Point (eX, eY)).x;
	arrowSy = fitBorder(source, new Point (eX, eY)).y;
	xTp[0] = fitBorder(destination, new Point (sX, sY)).x;
	yTp[0] = fitBorder(destination, new Point (sX, sY)).y;

	xTp = (int[])(getHead(new Point(arrowSx, arrowSy), 
		  new Point (xTp[0], yTp[0]), arrowHeadLength).
		  elementAt(0));
	yTp = (int[])(getHead(new Point(arrowSx, arrowSy), 
		  new Point (xTp[0], yTp[0]), arrowHeadLength).
		  elementAt(1));

	/* draw the line*/
	if (g != null) {
	    g.drawLine(arrowSx, arrowSy, xTp[0], yTp[0]);
	}
 
	/* set selection area center coordinates*/
	p = new Point((sX + eX) / 2, (sY + eY) / 2);

	/* Draw arrow head. */
	if (g != null) {
	    g.fillPolygon(xTp, yTp, xTp.length);
	}
    	return p;
    }

	
    /**
     * Draws a polyline, with head, between the parameter servers.
     * @param s Source server of the line.
     * @param d Destination server of the line.
     * @param xTurn Table that contains turnpoint x-coordinates
     * @param yTurn Table that contains turnpoint y-coordinates.
     * @param g Graphics reference for drawing. 
     * @return The center of the selection area of the line.
     */
    public Point drawMultiLine(Server s, Server d, int[] xTurn, 
						int[] yTurn, Graphics g) {

	/* arrow starting x coordinate*/
	int arrowSx = 0;

	/* arrow starting y coordinate*/
	int arrowSy = 0;

	/*table for arrow head x coordinates*/
	int[] xTp = new int[3];
	
	/*table for arrow head x coordinates*/
	int[] yTp = new int[3];

	/*selection area center point*/
	Point p = new Point(0, 0);

	/*polyline start and end lines initialization and fit checks with 
	  server border*/
	arrowSx = xTurn[xTurn.length - 1];
	arrowSy = yTurn[yTurn.length - 1];
	xTp[0] = fitBorder(d, new Point (arrowSx, arrowSy)).x;
	yTp[0] = fitBorder(d, new Point (arrowSx, arrowSy)).y;
	int sX = fitBorder(s, new Point (xTurn[0], yTurn[0])).x;
	int sY = fitBorder(s, new Point (xTurn[0], yTurn[0])).y;

	/* draw the polyline*/
	if (g != null) {
	    g.drawLine(sX, sY, xTurn[0], yTurn[0]);
	    g.drawPolyline(xTurn, yTurn, xTurn.length);	
	    g.drawLine(arrowSx, arrowSy, xTp[0], yTp[0]);
	}
	    
	/* prepare selection area center coordinates*/
	p = new Point(xTurnPoints[xTurnPoints.length / 2],
				  yTurnPoints[xTurnPoints.length / 2]);

	/*get arrow head coordinates*/ 
	xTp = (int[])(getHead(new Point(arrowSx, arrowSy), 
		 new Point (xTp[0], yTp[0]), arrowHeadLength).
		  elementAt(0));
	yTp = (int[])(getHead(new Point(arrowSx, arrowSy), 
		  new Point (xTp[0], yTp[0]), arrowHeadLength).
		  elementAt(1));

	/* Draw arrow head. */
	if (g != null) {
	    g.fillPolygon(xTp, yTp, xTp.length);
	}
	return p;
    }

    /**
     * Returns the point where the segment circle meets the parameter 
     * server border.
     * @param s Server that is crossing with segment circle.
     * @param p Selected point where segment arc meets the parameter server
     * circumference.
     * @param r Rejected point where segment circle meets the parameter server
     * circumference.
     * @return Point where segment circle meets the parameter server border. 
     */
    public Point fitSegment(Server s, Point p, Point r) {
	if (s.queueLength() == 0) {
	    return p;
	}
	/* if segment arc meets server ring w/o touching queue, no action*/
	if (p.x >= s.queRingConnectionX()) {
	    return p;
	}
	
	int yCross = 0;
	int xCross = p.x;

	/*calculate even arc y-value when x is at queue min*/
	if (order % 2 == 0) {
	    yCross = (int)Math.round((segmentCenterY - Math.sqrt(Math.pow(
	    segmentRadius, 2) - Math.pow(segmentCenterX - s.leftMostX(), 2))));

	    /*check if even arc meets queue's left upright line*/
	    if (yCross >= s.qupMostY() && yCross <= s.qdownMostY()) {
		xCross = s.leftMostX();
	    }
	    /*check if arc goes inside queue from top to down*/
	    else if (p.y < r.y){
		xCross = (int)Math.round((segmentCenterX + Math.sqrt(Math.pow(
		segmentRadius, 2) - Math.pow(segmentCenterY - 
		s.qupMostY(), 2))));
		yCross = s.qupMostY();
	    }
	    /*arc goes inside queue from below to up*/
	    else {
	        xCross = (int)Math.round((segmentCenterX - Math.sqrt(Math.pow(
		segmentRadius, 2) - Math.pow(segmentCenterY - 
		s.qdownMostY(), 2))));
		yCross = s.qdownMostY();
	    }
	}
	/*calculate odd arc y-value when x is at queue min*/
	else {
	    yCross = (int)Math.round((segmentCenterY +  Math.sqrt(Math.pow(
		segmentRadius, 2) - Math.pow(segmentCenterX - 
		s.leftMostX(), 2))));
	    /*check if even arc meets queue's left upright line*/
	    if (yCross >= s.qupMostY() && yCross <= s.qdownMostY()) {
		xCross = s.leftMostX();
	    }
	    
	    /*check if arc goes inside the queue from bottom to up*/
	    else  if (p.y > r.y) {
		xCross = (int)Math.round((segmentCenterX + Math.sqrt(Math.pow(
		segmentRadius, 2) - Math.pow(segmentCenterY - 
		s.qdownMostY(), 2))));
	        yCross = s.qdownMostY();
	    }

	    /* arc is going inside queue from top to down*/
	    else {
		xCross = (int)Math.round((segmentCenterX - Math.sqrt(Math.pow(
		segmentRadius, 2) - Math.pow(segmentCenterY - 
		s.qupMostY(), 2))));
	        yCross = s.qupMostY();	
	    }
	}
	return new Point(xCross, yCross);
    }
}
