/* 
 * @(#)Transaction 13.4.2002 0.1.1 
 * 
 * Copyright University of Helsinki / Department of computer science
 * 
 */ 

package tavi.graph;

import java.util.*;
import java.io.Serializable;

/** 
 * Class which defines a transaction. Transactions are necessary to
 * successfully create TFlow objects, as they are the environment
 * where TFlows belong to. 
 * <p>
 * Transactions represent some sort of execution path in the system,
 * where all flows are somehow related to each other to complete certain
 * task. The task itself is unimportant in the model. 
 * <p>
 * There is no 'global' transaction, so all transactions' TFlow objects
 * cannot be drawn at the same time. All transactions have an identifier
 * object that assigns identification numbers to transaction flows related
 * to the transaction. 
 * 
 * @version     0.1.1 13 Apr 2002 
 * @author      Ukkonen Johannes
 * @author      Airamaa Kimmo
 */ 
public class Transaction implements Serializable {
    /**
     * Name of the transaction. Used primarily for identification by user.
     */
    private String name;
    /**
     * Short identifier of transaction (unique). Used to identify transactions
     * so that none of them have the same identifier. This is helpful, if
     * user wants to create several transactions with the same name.
     */
    private String identifier;
    /**
     * Reference to Identifier object. This identifier assigns unique
     * identification values for the transaction flows that belong to
     * the transaction. 
     */
    private Identifier flowIdentifier;
    /**
     * Vector containing transaction's variables. These variables are
     * local to this transaction and should not be used globally. 
     */
    private Vector variables;
    /**
     * Constructor of transaction. Sets name and unique identification for 
     * transaction. Note that it is possible to assign same names to the
     * transactions, because identifier String identifies transactions.
     * @param transactionName Name of a transaction.
     * @param id Identifier of a transaction.
     * @param flowId Identifier object which will allocate identifiers 
     * to this transaction's tflows.
     */
    public Transaction(String transactionName, String id, Identifier
		       flowId) {
	if (transactionName.length() > 30) {
	    /* Cut too long names. */
	    String tmpName = "" + transactionName.substring(0, 29);
	    transactionName = tmpName;
	}
	name = "" + transactionName;
	identifier = id;
	flowIdentifier = flowId;
	variables = new Vector();
    }

    /**
     * Adds a variable for this transaction. If there is already a
     * variable with the same name, the addition is not possible. 
     * @param variable Variable to be added.
     * @return True if it was possible to add new variable. False
     * otherwise. 
     */
    public boolean addVariable(Variable variable) {
	for (int i = 0; i < variables.size(); ++i) {
	    /* Check for duplicate names. If found, don't create new. */
	    if (((Variable)variables.elementAt(i))
		.getName().equals(variable.getName())) {
		return false;
	    }
	}
	variables.add(variable);
	return true;
    }

    /**
     * Removes a variable if found. If there are no variables matching
     * the required one, nothing needs to be done.
     * @param variable Variable to be removed.
     */
    public void removeVariable(Variable variable) {
	variables.removeElement(variable);
    }

    /**
     * Returns an instance to vector containing all variables. These
     * variables are local to the transaction, so it must be checked
     * separately that variables belong to the currently active 
     * transaction wherever used.
     * @return Vector that contains all transaction's variables.
     */
    public Vector getVariables() {
	return variables;
    }

    /**
     * Returns transaction's unique identifier. 
     * @return Identifier of a transaction.
     */
    public String getIdentifier() {
	return identifier;
    }

    /**
     * Returns transaction's flow identifier object that assigns
     * identification numbers to transaction's TFlow objects.  
     * @return Flow identifier of a transaction.
     */
    public Identifier getFlowIdentifier() {
	return flowIdentifier;
    }

    /**
     * Returns the name of this transaction. Name is never null.
     * @return Name of a transaction.
     */
    public String getName() {
	return name;
    }

    /**
     * Sets the name of a transaction. Null values are possible to give as a
     * parameter, but don't result in null name.
     * @param newName New name of a transaction.
     */
    public void setName(String newName) {
	name = "" + newName;
    }

    /**
     * Method that returns a String representation of this class.
     * Useful when creating JList or JComboBox objects.
     * @return String representation of this class.
     */
    public String toString() {
	String temp = new String(name);
	while (temp.length() < 30) {
	    temp += " ";
	}
	return "" + temp + " " + identifier;
    }
}
