/* 
 * @(#)TransactionManager 13.4.2002 0.1.1 
 * 
 * Copyright University of Helsinki / Department of computer science
 * 
 */ 

package tavi.graph;

import java.util.*;
import java.io.Serializable;

/** 
 * Class which stores all transactions and global variables. Acts as a
 * manager between Tavi main class and transactions. Defines an interface
 * for transaction and variable handling. One transaction is active at all
 * times. Local variables are added to the active transaction and also removed
 * from it. This, however, is done so that the dialog manager object fetches
 * <p>
 * TransactionManager.getCurrent().getVariables();
 * <p>
 * And asks the <b>transaction</b> to remove variables. User cannot get an
 * instance to any transaction variable list, only the currently active
 * transaction. This provides minimal security for the variables.
 * 
 * @version     0.1.1 13 Apr 2002 
 * @author      Ukkonen Johannes
 * @author      Airamaa Kimmo
 */ 
public class TransactionManager implements Serializable {
    /**
     * Structure that stores all transactions. Usually, the active
     * transaction is et the first index. However, it shouldn't be
     * taken for granted.
     */
    private Vector transactions;
    /**
     * Structure that contains all global variables. Global 
     * variables can be used everywhere in the model.
     */
    private Vector globalVariables;
    /**
     * Reference to currently active transaction. Only one transaction
     * can be active at the time.
     */
    private Transaction currentTransaction;
    /**
     * Constructor of a transaction manager. Initializes transaction vector
     * and other data structures. There is initially a default transaction
     * in the transaction manager, called "default" and identified as "Tr0".
     */
    public TransactionManager() {
	transactions = new Vector();
	globalVariables = new Vector();
	Transaction defaultTransaction  = 
	    new Transaction("default", "Tr0", new Identifier());
	transactions.add(defaultTransaction);
	currentTransaction = defaultTransaction;
    }

    /**
     * Adds a global variable. Names must be unique.
     * @param name Name of the variable.
     * @param value Value of the variable.
     * @return True if name is not null and there isn't a variable 
     * with a same name. False otherwise.
     */
    public boolean addGlobalVariable(String name, float value) {
	if (name == null) {
	    return false;
	}
	for (int i = 0; i < globalVariables.size(); ++i) {
	    /* Search for duplicate names. */
	    if (name.equals(((Variable)globalVariables.elementAt(i)).
			    getName())) {
		return false;
	    }
	}
	globalVariables.add(new Variable(name, value));
	return true;
    }

    /**
     * Removes a global variable if found. Otherwise there is no such
     * variable and it is ok.
     * @param variable Variable to be removed.
     */
    public void removeGlobalVariable(Variable variable) {
	globalVariables.removeElement(variable);
    }

    /**
     * Adds a new transaction to the list. Transactions cannot have
     * empty or null names. New transaction is set active when it is
     * created. 
     * @param name Name of a transaction. 
     * @param id Unique identifier of a transaction.
     * @return True if operation succesful, false otherwise (i.e. in case 
     * of a null reference in id). 
     */
    public boolean addTransaction(String name, String id) {
	if (name == null || id == null) {
	    return false;
	}
	Transaction newTransaction = 
	    new Transaction(name, id, new Identifier());
	transactions.add(0, newTransaction);
	currentTransaction = newTransaction;
	return true;
    }

    /**
     * Removes a transaction. It is not possible to remove all 
     * transactions. One must remain at all times. If currently
     * active transaction is removed, the first transaction left in
     * the structure is assigned as active transaction. 
     * @param target Transaction to be removed.
     * @return True if it is possible to remove transaction,
     * false otherwise.
     */
    public boolean removeTransaction(Object target) {
	if (transactions.size() == 1) {
	    return false;
	}
	transactions.removeElement(target);
	/* in case when currentTransaction is removed */
	if (!transactions.contains(currentTransaction)) {
	    currentTransaction = (Transaction) transactions.elementAt(0);
	}
	return true;
    }

    /**
     * Changes the currently active transaction, if proposed
     * transaction exists and moves it to first index.
     * Otherwise does nothing.
     * @param newOne Transaction to be set current. 
     * @return True if changing succeeded, false otherwise.
     */
    public boolean setCurrent(Transaction newOne) {
	/* If transaction isn't in the vector, it can't
	   be set currently active transaction. */
	if (!transactions.contains(newOne)) {
	    return false;
	}
	currentTransaction = newOne;
	transactions.remove(newOne);
	transactions.insertElementAt(newOne, 0); 
	return true;
    }

    /**
     * Returns a reference to currently active transaction. 
     * @return Currently active transaction.
     */
    public Transaction getCurrent() {
	return currentTransaction;
    }
    
    /**
     * Returns the reference to transaction vector. It is still highly 
     * recommended to access the transactions via TransactionManager's
     * getCurrent() and setCurrent() methods. 
     * @return Vector of all transactions.
     */
    public Vector getTransactions() {
	return transactions;
    }

    /**
     * Returns the reference to global variables vector. These variables
     * are 'visible' at all times. There is no such harm using this method
     * than with getTransactions().
     * @return Vector that contains all global variables.
     */
    public Vector getGlobalVariables() {
	return globalVariables;
    }
}
