/* 
 * @(#)Variable 25.4.2002 0.1.1 
 * 
 * Copyright University of Helsinki / Department of computer science
 * 
 */ 

package tavi.graph;

import java.util.*;
import java.io.Serializable;

/** 
 * Class that represents variables that can be used in
 * transactions. Contains the name of variable and its real
 * value. Variables can be used in expressions quite like 
 * 'conventional' programming environment variables. Variables'
 * scope is not implicated in this class. Transactions or transaction
 * managers define the scope of the variables in Tavi.
 * <p>
 * Variables are used in transaction flows and probably elsewhere.
 * Typical use of variable might be some kind of probability of
 * branching in the current model or perhaps service times. 
 * 
 * @version     0.1.1 25 Apr 2002 
 * @author      Ukkonen Johannes
 * @author      Airamaa Kimmo
 */ 
public class Variable implements Serializable {
    /**
     * Name field of the variable. Typically, the variable might be
     * instantiated in the Tavi program by its name.
     */
    private String name;
    /**
     * Value of the variable. All variables have float values and can have
     * their values changed at all times. 
     */
    private float value;

    /**
     * Constructor that creates a variable with certain value. If variable's
     * name is null or empty then name will be set as "default". 
     * @param variableName Name of the variable.
     * @param realValue Value of the variable.
     */
    public Variable(String variableName, float realValue) {
	if (variableName == null || variableName.equals("")) {
	    variableName = "default";
	} else if (variableName.length() > 20) {
	    String tmpName = "" + variableName.substring(0, 19);
	    variableName = tmpName;
	}
	name = variableName;
	value = realValue;
    }

    /**
     * Returns the name of the variable.
     * @return Name of the variable.
     */
    public String getName() {
	return name;
    }
    
    /**
     * Renames the variable. Variable's name cannot be null or empty
     * String. IT should be noted, that single variable DOES NOT know
     * of other variables, so if variables' names should be unique,
     * it must be elsewhere.
     * @param newName New name of the variable.
     */
    public void setName(String newName) {
	if (newName == null || newName.equals("")) {
	    newName = "default";
	}	
	name = newName;
    }
    
    /**
     * Returns the value of the variable.
     * @return Value of the variable.
     */
    public float getValue() {
	return value;
    }

    /**
     * Replaces the value of the variable with new float. Used when
     * changing value of the variable. 
     * @param newValue New value for the variable.
     */
    public void setValue(float newValue) {
	value = newValue;
    }

    /**
     * Returns a String representation of the variable. Useful for
     * example in JList structure.
     * @return String representation of the variable.
     */
    public String toString() {
	String temp = new String(name);
	while (temp.length() < 20) {
	    temp += " ";
	}
	return "" + temp + " " + value;
    }
}
